(*  Title:      ContextLocalRed.thy
    ID:         $Id: ContextLocalRed.thy,v 1.3 2009/03/24 17:07:58 flokam Exp $
    Author:     Ludovic Henrio and Florian Kammuller
                2006

    Note:      This file defines another version of the local semantics 
                  (ie the semantics of sigma calculus).
               It relies on the definition of contexts, and consequently relies on 
               a sinngle rule for reducing a term inside a subterm
*)

header {* Context-based semanics of the sigma calculus *}

theory ContextLocalRed imports contexts begin

inductive cbeta :: "[sterm, sterm] \<Rightarrow> bool"  (infixl "\<rightarrow>\<^bsub>c\<beta>\<^esub>" 50)
  where
    cbeta [simp, intro!]: "\<lbrakk> l \<in> dom f; lc (Obj f T); lc b \<rbrakk> 
                          \<Longrightarrow> Call (Obj f T) l b \<rightarrow>\<^bsub>c\<beta>\<^esub> (the (f l)\<^bsup>[Obj f T, b]\<^esup>)" 
 |  cupd  [simp, intro!]: "\<lbrakk> l \<in> dom f; lc (Obj f T); body a \<rbrakk> 
                          \<Longrightarrow> Upd (Obj f T) l a \<rightarrow>\<^bsub>c\<beta>\<^esub>  Obj (f(l \<mapsto> a)) T"
 |  cupdR [simp, intro!]: "\<lbrakk> finite L; \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p 
                                        \<longrightarrow> (\<exists>t''. t\<^bsup>[Fvar s,Fvar p]\<^esup> \<rightarrow>\<^bsub>c\<beta>\<^esub> t''\<and> t'= \<sigma>[s,p]t'');
                             lc u \<rbrakk> 
                           \<Longrightarrow> Upd u l t \<rightarrow>\<^bsub>c\<beta>\<^esub> Upd u l t'"
 |  cobj  [simp, intro!]: "\<lbrakk> l \<in> dom f; finite L; 
                             \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p
                              \<longrightarrow> (\<exists>t''. t\<^bsup>[Fvar s,Fvar p]\<^esup> \<rightarrow>\<^bsub>c\<beta>\<^esub> t'' \<and> t'= \<sigma>[s,p]t'');
                             \<forall>l\<in>dom f. body (the(f l)) \<rbrakk> 
                           \<Longrightarrow> Obj (f(l \<mapsto> t)) T \<rightarrow>\<^bsub>c\<beta>\<^esub> Obj (f(l \<mapsto> t')) T"
 |  cont  [simp, intro!]: "\<lbrakk> s \<rightarrow>\<^bsub>c\<beta>\<^esub> t; lc_ctxt E \<rbrakk> \<Longrightarrow> (E::general_context)\<up>s \<rightarrow>\<^bsub>c\<beta>\<^esub> E\<up>t"

(* equivalence of the two semantics *)
lemma cbeta_and_beta: "(s \<rightarrow>\<^bsub>c\<beta>\<^esub> t) = (s \<rightarrow>\<^sub>\<beta> t)"
proof -
  fix s :: sterm and t :: sterm
(* \<Longrightarrow> *)
  have "s \<rightarrow>\<^bsub>c\<beta>\<^esub> t \<Longrightarrow> s \<rightarrow>\<^sub>\<beta> t" by (erule cbeta.induct, auto)
(* \<Longleftarrow> *)
  moreover
  have "s \<rightarrow>\<^sub>\<beta> t \<Longrightarrow> s \<rightarrow>\<^bsub>c\<beta>\<^esub> t"
  proof (induct s t rule: beta.induct, auto)
    case (beta_CallL t t' u l) note cont[OF `t \<rightarrow>\<^bsub>c\<beta>\<^esub> t'`]
    from `lc u` this[of "cCallL cHole l u"] show ?case by simp
  next
    case (beta_CallR t t' u l) note cont[OF `t \<rightarrow>\<^bsub>c\<beta>\<^esub> t'`]
    from `lc u` this[of "cCallR u l cHole"] show ?case by simp
  next
    case (beta_Act s t) note cont[OF `s \<rightarrow>\<^bsub>c\<beta>\<^esub> t`]
    from this[of "cActive cHole"] show ?case by simp
  next
    case (beta_UpdL t t' u l)
    from `body u` obtain L where "finite L"
                                 "\<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p
                                   \<longrightarrow> lc (u\<^bsup>[Fvar s,Fvar p]\<^esup>)"
      by (auto simp: body_def)
    from cont[OF `t \<rightarrow>\<^bsub>c\<beta>\<^esub> t'` lc_ctxt_UpdL[OF lc_ctxt_Hole this]]
    show ?case by simp
  qed
  ultimately
  show "(s \<rightarrow>\<^bsub>c\<beta>\<^esub> t) = (s \<rightarrow>\<^sub>\<beta> t)"
    using iffI[of "s \<rightarrow>\<^bsub>c\<beta>\<^esub> t" "s \<rightarrow>\<^sub>\<beta> t"] by simp
qed

end