(*  Title:      Typed_Sigma.thy

    ID:         $Id: TypedSigma.thy,v 1.1 2009/11/12 11:55:09 flokam Exp $
    Author:     Florian Kammuller and Henry Sudhof
                2006

    Note: 
    TODO:
       
*)

header {* First Order Types for Sigma terms *}

theory TypedASP imports "../preliminary/Environments" CyclesOfFutures begin

subsubsection {* Types and typing rules *}
text{* The inductive definition of the typing relation.*}

(*configuration type *)
datatype Ctype = TConfig "ActivityRef ~=> type" "FutureRef ~=> type"

constdefs
  return :: "(type \<times> type) \<Rightarrow> type " 
  "return a == fst a"
  param :: "(type \<times> type) \<Rightarrow> type " 
  "param a == snd a"

primrec
  do :: "type \<Rightarrow> (Label set)"  
where
 "do (Object l) = (dom l)"

primrec
  type_get :: "type \<Rightarrow> Label \<Rightarrow> (type \<times> type) option"  ("_^_")
where
 "(Object l)^n = (l n)"

  (* we need to restrict objects to ok environments, 
     as the empty object does not yield ok env otherwise *)
inductive 
  typing :: "Ctype \<Rightarrow> (type environment) \<Rightarrow> sterm \<Rightarrow> type \<Rightarrow> bool"    
  ("_, _ \<turnstile> _ : _" [50, 50, 50] 50)
where 
  T_Var[intro!]:
    "\<lbrakk> ok env; x \<in> env_dom env; (the (env!x)) = T \<rbrakk> 
    \<Longrightarrow> Tconf, env \<turnstile> (Fvar x) : T"
| T_Obj[intro!]:
    "\<lbrakk> ok env; dom b = do A; finite F; 
       \<forall>l\<in>do A. \<forall>s p. s \<notin> F \<and> p \<notin> F \<and> s \<noteq> p 
        \<longrightarrow> Tconf, env\<lparr>s:A\<rparr>\<lparr>p:param(the (A^l))\<rparr> 
            \<turnstile> (the (b l)\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the (A^l)) \<rbrakk> 
    \<Longrightarrow> Tconf, env \<turnstile> (Obj b A) : A"
| T_Upd[intro!]:  
    "\<lbrakk> finite F; 
       \<forall>s p. s \<notin> F \<and> p \<notin> F \<and> s \<noteq> p 
        \<longrightarrow>  Tconf, env\<lparr>s:A\<rparr>\<lparr>p:param(the (A^l))\<rparr>
             \<turnstile> (n\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the (A^l));
    Tconf, env \<turnstile> a : A; l \<in> do A \<rbrakk> \<Longrightarrow> Tconf, env \<turnstile> Upd a l n : A"
| T_Call[intro!]: 
    "\<lbrakk> Tconf, env \<turnstile> a : A; Tconf, env \<turnstile> b : param(the (A^l)); l \<in> do A \<rbrakk> 
    \<Longrightarrow> Tconf, env \<turnstile> (Call a l b) : return(the (A^l))"
| T_Active[intro!]:
    "\<lbrakk> ok env; Tconf, env \<turnstile> a : A \<rbrakk> \<Longrightarrow> Tconf, env \<turnstile> Active a : A"
| T_ActRef[intro!]:
    "\<lbrakk> ok env; amap B = Some T; B \<in> dom amap \<rbrakk> \<Longrightarrow> TConfig amap futmap, env \<turnstile> ActRef B : T"
| T_FutRef[intro!]:
    "\<lbrakk> ok env; futmap fi = Some T; fi \<in> dom futmap \<rbrakk> 
    \<Longrightarrow> TConfig amap futmap, env \<turnstile> FutRef fi : T"

inductive_cases typing_elims [elim!]:
  "Tconf, e \<turnstile> Obj b T : T"
  "Tconf, e \<turnstile> Fvar x : T"
  "Tconf, e \<turnstile> Call a l b : T"
  "Tconf, e \<turnstile> Upd a l n : T"
  "Tconf, e \<turnstile> Active a : T"
  "Tconf, e \<turnstile> ActRef B : T"
  "Tconf, e \<turnstile> FutRef fi : T"

inductive  Ctyping :: "configuration \<Rightarrow> Ctype \<Rightarrow> bool" ("\<turnstile> _ : _" [50] 50)
where
  T_conf: "\<lbrakk> dom amap  = dom c; 
             dom futmap = (\<Union>A\<in>dom c. dom (c.\<^sub>RA));
             \<forall>A\<in>dom c. (TConfig amap futmap, (Env empty) \<turnstile> c.\<^sub>AA : the (amap A))
                     \<and> (\<forall>fi\<in>dom (c.\<^sub>RA). (TConfig amap futmap), (Env empty) 
                                         \<turnstile> (c.\<^sub>FA<fi>) : the (futmap fi)) 
  \<rbrakk> \<Longrightarrow> \<turnstile> c : TConfig amap futmap"

subsubsection {*Basic lemmas *}
text{*Basic treats of the type system.*}
lemma not_bvar: "Tconf, e \<turnstile> t : T \<Longrightarrow> \<forall>i. t \<noteq> Bvar i"
  by (erule typing.cases, simp_all)

lemma typing_regular': "Tconf, e \<turnstile> t : T \<Longrightarrow> ok e" 
  by (induct rule:typing.induct, auto)

lemma typing_regular'': "Tconf, e \<turnstile> t : T \<Longrightarrow> lc t" 
  by (induct rule:typing.induct, auto)

theorem typing_regular: "Tconf, e \<turnstile> t : T \<Longrightarrow> ok e \<and> lc t" 
  by (simp add: typing_regular' typing_regular'')

lemma obj_inv: "Tconf, e \<turnstile> Obj f U : A \<Longrightarrow> A = U"
  by (erule typing.cases, auto)

lemma obj_inv_elim: 
  "Tconf, e \<turnstile> Obj f U : U 
  \<Longrightarrow> (dom f = do U) 
    \<and> (\<exists>F. finite F \<and> (\<forall>l\<in>do U. \<forall>s p. s \<notin> F \<and> p \<notin> F \<and> s \<noteq> p 
        \<longrightarrow> Tconf, e\<lparr>s:U\<rparr>\<lparr>p:param(the U^l)\<rparr>
            \<turnstile> (the (f l)\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the (U^l))))"
  by (erule typing.cases, simp_all, blast)

lemma typing_induct[consumes 1, 
  case_names Fvar Call Active ActRef FutRef Upd Obj Bnd]:
  fixes 
  Tconf :: Ctype and env :: "type environment" and t :: sterm and T :: type and 
  P1 :: "Ctype \<Rightarrow> type environment \<Rightarrow> sterm \<Rightarrow> type \<Rightarrow> bool" and
  P2 :: "Ctype \<Rightarrow> type environment \<Rightarrow> sterm \<Rightarrow> type \<Rightarrow> Label \<Rightarrow> bool"
  assumes
  "Tconf, env \<turnstile> t : T" and
  "\<And>C env T x. \<lbrakk> ok env; x \<in> env_dom env; the env!x = T \<rbrakk>
  \<Longrightarrow> P1 C env (Fvar x) T" and
  "\<And>C env T t l p. \<lbrakk> C, env \<turnstile> t : T; P1 C env t T; C, env \<turnstile> p : param (the(T^l));
                    P1 C env p (param (the(T^l))); l \<in> do T \<rbrakk>
  \<Longrightarrow> P1 C env (Call t l p) (return (the(T^l)))" and
  "\<And>C env T a. \<lbrakk> ok env; C, env \<turnstile> a : T; P1 C env a T \<rbrakk> 
  \<Longrightarrow> P1 C env (Active a) T" and
  "\<And>amap futmap env T B. \<lbrakk> ok env; amap B = Some T; B \<in> dom amap \<rbrakk> 
  \<Longrightarrow> P1 (TConfig amap futmap) env (ActRef B) T" and
  "\<And>amap futmap env T fi. \<lbrakk> ok env; futmap fi = Some T; fi \<in> dom futmap \<rbrakk> 
  \<Longrightarrow> P1 (TConfig amap futmap) env (FutRef fi) T" and
  "\<And>C env T t l u. \<lbrakk> C, env \<turnstile> t : T; P1 C env t T; l \<in> do T; P2 C env u T l \<rbrakk>
  \<Longrightarrow> P1 C env (Upd t l u) T" and
  "\<And>C env T f. \<lbrakk> ok env; dom f = do T; \<forall>l\<in>dom f. P2 C env (the(f l)) T l \<rbrakk>
  \<Longrightarrow> P1 C env (Obj f T) T" and
  "\<And>C env T l t L. \<lbrakk> ok env; finite L; 
                  \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p 
                   \<longrightarrow> C, env\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> \<turnstile> (t\<^bsup>[Fvar s, Fvar p]\<^esup>) : return (the(T^l))
                       \<and> P1 C (env\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr>) (t\<^bsup>[Fvar s, Fvar p]\<^esup>) (return (the(T^l))) \<rbrakk>
  \<Longrightarrow> P2 C env t T l"
  shows
  "P1 Tconf env t T"
  using assms by (induct rule: typing.induct, auto simp: typing_regular')

(* TODO: delete after refactoring of type_renaming' *)
lemma bnd_renaming_lem:
  assumes 
  "s \<notin> FV t'" and "p \<notin> FV t'" and "x \<notin> FV t'" and "y \<notin> FV t'" and
  "x \<notin> env_dom env'" and "y \<notin> env_dom env'" and "s \<noteq> p" and "x \<noteq> y" and
  "t = {Suc n \<rightarrow> [Fvar s, Fvar p]} t'" and "env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>" and
  pred_bnd:
  "\<forall>sa pa. sa \<notin> F \<and> pa \<notin> F \<and> sa \<noteq> pa 
    \<longrightarrow> Tconf, env\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr> \<turnstile> (t\<^bsup>[Fvar sa,Fvar pa]\<^esup>) : return(the(T^l))
        \<and> (\<forall>env'' t'' s' p' x' y' A' B' n'. 
            s' \<notin> FV t'' \<longrightarrow> p' \<notin> FV t'' \<longrightarrow> x' \<notin> FV t'' \<longrightarrow> y' \<notin> FV t'' \<longrightarrow> 
            x' \<notin> env_dom env'' \<longrightarrow> y' \<notin> env_dom env'' \<longrightarrow> x' \<noteq> y' \<longrightarrow> s' \<noteq> p' 
            \<longrightarrow> (t\<^bsup>[Fvar sa,Fvar pa]\<^esup>) = {n' \<rightarrow> [Fvar s',Fvar p']} t'' 
            \<longrightarrow> env\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr> = env''\<lparr>s':A'\<rparr>\<lparr>p':B'\<rparr> 
            \<longrightarrow> Tconf, env''\<lparr>x':A'\<rparr>\<lparr>y':B'\<rparr> 
                \<turnstile> {n' \<rightarrow> [Fvar x',Fvar y']} t'' : return(the(T^l)))" and
  "FV t' \<subseteq> F'"
  shows
  "\<forall>sa pa. sa \<notin> F \<union> {s,p,x,y} \<union> F' \<union> env_dom env' 
         \<and> pa \<notin> F \<union> {s,p,x,y} \<union> F' \<union> env_dom env' 
         \<and> sa \<noteq> pa
    \<longrightarrow> Tconf, env'\<lparr>x:A\<rparr>\<lparr>y:B\<rparr>\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>
        \<turnstile> ({Suc n \<rightarrow> [Fvar x, Fvar y]} t'\<^bsup>[Fvar sa,Fvar pa]\<^esup>) : return (the(T^l))"
proof (intro strip, elim conjE)
  fix sa pa 
  assume 
    nin_sa: "sa \<notin> F \<union> {s,p,x,y} \<union> F' \<union> env_dom env'" and
    nin_pa: "pa \<notin> F \<union> {s,p,x,y} \<union> F' \<union> env_dom env'" and "sa \<noteq> pa"
  hence "sa \<notin> F \<and> pa \<notin> F \<and> sa \<noteq> pa" by auto
  moreover
  {
    fix a assume "a \<notin> FV t'" and "a \<in> {s,p,x,y}"
    with 
      `FV t' \<subseteq> F'` nin_sa nin_pa `sa \<noteq> pa` 
      sopen_FV[of 0 "Fvar sa" "Fvar pa" t']
    have "a \<notin> FV (t'\<^bsup>[Fvar sa,Fvar pa]\<^esup>)" by (auto simp: openz_def)
  } note 
      this[OF `s \<notin> FV t'`] this[OF `p \<notin> FV t'`] 
      this[OF `x \<notin> FV t'`] this[OF `y \<notin> FV t'`]
  moreover
  from 
    not_in_env_bigger_2[OF `x \<notin> env_dom env'`] 
    not_in_env_bigger_2[OF `y \<notin> env_dom env'`]
    nin_sa nin_pa
  have 
    "x \<notin> env_dom (env'\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>) 
    \<and> y \<notin> env_dom (env'\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>)" by auto
  moreover
  from `t = {Suc n \<rightarrow> [Fvar s, Fvar p]} t'` sopen_commute[OF Suc_not_Zero] 
  have "(t\<^bsup>[Fvar sa,Fvar pa]\<^esup>) = {Suc n \<rightarrow> [Fvar s,Fvar p]} (t'\<^bsup>[Fvar sa,Fvar pa]\<^esup>)"
    by (auto simp: openz_def)
  moreover
  from 
    subst_add[of s sa env' A T] subst_add[of sa p "env'\<lparr>s:A\<rparr>" T B] 
    subst_add[of s pa "env'\<lparr>sa:T\<rparr>" A "param(the(T^l))"]
    subst_add[of p pa "env'\<lparr>sa:T\<rparr>\<lparr>s:A\<rparr>" B "param(the(T^l))"]
    `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` nin_sa nin_pa
  have "env\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr> = env'\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>"
    by auto
  ultimately
  have 
    "Tconf, env'\<lparr>sa:T\<rparr>\<lparr>pa:param(the T^l)\<rparr>\<lparr>x:A\<rparr>\<lparr>y:B\<rparr>
     \<turnstile> {Suc n \<rightarrow> [Fvar x, Fvar y]} (t'\<^bsup>[Fvar sa,Fvar pa]\<^esup>) : return(the(T^l))"
    using `s \<noteq> p` `x \<noteq> y` pred_bnd by auto
  moreover
  from 
    subst_add[of y sa "env'\<lparr>x:A\<rparr>" B T] subst_add[of x sa env' A T] 
    subst_add[of y pa "env'\<lparr>sa:T\<rparr>\<lparr>x:A\<rparr>" B "param(the(T^l))"]
    subst_add[of x pa "env'\<lparr>sa:T\<rparr>" A "param(the(T^l))"] 
    nin_sa nin_pa
  have 
    "env'\<lparr>x:A\<rparr>\<lparr>y:B\<rparr>\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>
     = env'\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>\<lparr>x:A\<rparr>\<lparr>y:B\<rparr>"
    by auto
  ultimately 
  show 
    "Tconf, env'\<lparr>x:A\<rparr>\<lparr>y:B\<rparr>\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>
     \<turnstile> ({Suc n \<rightarrow> [Fvar x, Fvar y]} t'\<^bsup>[Fvar sa,Fvar pa]\<^esup>) : return (the(T^l))"
    using sopen_commute[OF not_sym[OF Suc_not_Zero]]
    by (simp add: openz_def)
qed

(* TODO: refactor to work with typing_induct *)
lemma type_renaming'[rule_format]:
  "Tconf, e \<turnstile> t : C \<Longrightarrow>
  (\<And>env t' s p x y A B n. \<lbrakk> s \<notin> FV t'; p \<notin> FV t'; x \<notin> FV t'; y \<notin> FV t'; 
                             x \<notin> env_dom env; y \<notin> env_dom env; s \<noteq> p;  x \<noteq> y;
                             t = {n \<rightarrow> [Fvar s,Fvar p]} t'; e = env\<lparr>s:A\<rparr>\<lparr>p:B\<rparr> \<rbrakk>
    \<Longrightarrow> Tconf, env\<lparr>x:A\<rparr>\<lparr>y:B\<rparr> \<turnstile> {n \<rightarrow> [Fvar x,Fvar y]} t' : C)"
proof (induct set:typing)
  case (T_Call C env t1 T t2 l env' t' s p x y A B n)
  with sopen_eq_Call[OF sym[OF `Call t1 l t2 = {n \<rightarrow> [Fvar s,Fvar p]} t'`]]
  show ?case by auto
next
  case (T_Var env a T C env' t' s p x y A B n)
  note ok = 
    ok_add_ok2[OF `ok env` `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` 
                  `x \<notin> env_dom env'` `y \<notin> env_dom env'` `x \<noteq> y`]
  from sopen_eq_Fvar[OF sym[OF `Fvar a = {n \<rightarrow> [Fvar s,Fvar p]} t'`]] 
  show ?case
  proof (elim disjE conjE)
    assume "t' = Fvar a" with prems(4-7) 
    obtain "a \<noteq> s" and "a \<noteq> p" and "a \<noteq> x" and "a \<noteq> y" by auto
    note in_env_smaller2[OF _ this(1-2)]
    from `a \<in> env_dom env` `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` this[of env' A B]
    have "a \<in> env_dom env'" by simp
    from env_bigger2[OF `x \<notin> env_dom env'` `y \<notin> env_dom env'` this `x \<noteq> y`]
    have inenv: "a \<in> env_dom (env'\<lparr>x:A\<rparr>\<lparr>y:B\<rparr>)" by assumption
    note get_env_bigger2[OF _ `a \<noteq> s` `a \<noteq> p`]
    from 
      this[of env' A B] `a \<in> env_dom env` `the env!a = T` 
      `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` get_env_bigger2[OF inenv `a \<noteq> x` `a \<noteq> y`] 
    have "the (env'\<lparr>x:A\<rparr>\<lparr>y:B\<rparr>!a) = T" by simp
    from typing.T_Var[OF ok inenv this] `t' = Fvar a` show ?case by simp
  next
    assume "a = s" and "t' = Bvar (Self n)"
    from 
      this(1) `ok env` `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` `the env!a = T` 
      add_get2_1[of env' s A p B]
    have "T = A" by simp
    moreover
    from `t' = Bvar (Self n)` have "{n \<rightarrow> [Fvar x,Fvar y]} t' = Fvar x" by simp
    ultimately
    show ?case using in_add_2[OF ok] typing.T_Var[OF ok _ add_get2_1[OF ok]]
      by simp
  next
    note subst = subst_add[OF `x \<noteq> y`] 
    from subst[of env' A B] ok have ok': "ok (env'\<lparr>y:B\<rparr>\<lparr>x:A\<rparr>)" by simp
    assume "a = p" and "t' = Bvar (Param n)"
    from 
      this(1) `ok env` `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` `the env!a = T`
      add_get2_2[of env' s A p B]
    have "T = B" by simp
    moreover
    from `t' = Bvar (Param n)` have "{n \<rightarrow> [Fvar x,Fvar y]} t' = Fvar y" by simp
    ultimately
    show ?case 
      using 
	subst[of env' A B] in_add_2[OF ok'] 
        typing.T_Var[OF ok' _ add_get2_1[OF ok']]
      by simp
  qed
next
  case (T_Upd F C env T l t2 t1 env' t' s p x y A B n)
  from sopen_eq_Upd[OF sym[OF `Upd t1 l t2 = {n \<rightarrow> [Fvar s,Fvar p]} t'`]]
  obtain t1' t2' where 
    t1: "t1 = {n \<rightarrow> [Fvar s,Fvar p]} t1'" and
    t2: "t2 = {Suc n \<rightarrow> [Fvar s,Fvar p]} t2'" and
    t': "t' = Upd t1' l t2'" 
    by auto
  { fix a assume "a \<notin> FV t'" with t' have "a \<notin> FV t1'" by simp } 
  note 
    t1' = prems(4)[OF this[OF `s \<notin> FV t'`] this[OF `p \<notin> FV t'`] 
                      this[OF `x \<notin> FV t'`] this[OF `y \<notin> FV t'`] 
                      `x \<notin> env_dom env'` `y \<notin> env_dom env'`
                      `s \<noteq> p` `x \<noteq> y` t1 `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>`]
  from ok_finite[of env'] ok_add_2[OF typing_regular'[OF this]]
  have findom: "finite (env_dom env')" by simp

  { fix a assume "a \<notin> FV t'" with t' have "a \<notin> FV t2'" by simp }
  note 
    bnd_renaming_lem[OF this[OF `s \<notin> FV t'`] this[OF `p \<notin> FV t'`] 
                        this[OF `x \<notin> FV t'`] this[OF `y \<notin> FV t'`] 
                        `x \<notin> env_dom env'` `y \<notin> env_dom env'`
                        `s \<noteq> p` `x \<noteq> y` t2 `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>`]
  from this[of F C T l "FV t2'"] prems(2)
  have 
    "\<forall>sa pa. sa \<notin> F \<union> {s, p, x, y} \<union> FV t2' \<union> env_dom env' 
           \<and> pa \<notin> F \<union> {s, p, x, y} \<union> FV t2' \<union> env_dom env' 
           \<and> sa \<noteq> pa 
      \<longrightarrow> C, env'\<lparr>x:A\<rparr>\<lparr>y:B\<rparr>\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>
          \<turnstile> ({Suc n \<rightarrow> [Fvar x,Fvar y]} t2'\<^bsup>[Fvar sa,Fvar pa]\<^esup>) : return(the(T^l))"
    by simp
  from 
    typing.T_Upd[OF _ this t1' `l \<in> do T`]
    `finite F` findom t' 
  show ?case by simp
next
  case (T_Obj env f T F C env' t' s p x y A B n)
  note ok =
    ok_add_ok2[OF `ok env` `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` 
                  `x \<notin> env_dom env'` `y \<notin> env_dom env'` `x \<noteq> y`]
  from sopen_eq_Obj[OF sym[OF `Obj f T = {n \<rightarrow> [Fvar s,Fvar p]} t'`]] 
  obtain f' where 
    obj: "{n \<rightarrow> [Fvar s,Fvar p]} Obj f' T = Obj f T" and
    t': "t' = Obj f' T" by auto
  from 
    this(1) `dom f = do T`
    sym[OF dom_sopenoption_lem[of "Suc n" "Fvar s" "Fvar p" f']]
    dom_sopenoption_lem[of "Suc n" "Fvar x" "Fvar y" f']
  have dom: "dom (\<lambda>l. sopen_option (Suc n) (Fvar x) (Fvar y) (f' l)) = do T" 
    by simp
    
  from 
    `finite F` finite_FV[of "Obj f' T"] 
    ok_finite[of env'] ok_add_2[OF ok]
  have finF: "finite (F \<union> {s,p,x,y} \<union>  FV (Obj f' T) \<union> env_dom env')"
    by simp

  have 
    "\<forall>l\<in>do T. \<forall>sa pa. sa \<notin> F \<union> {s, p, x, y} \<union> FV (Obj f' T) \<union> env_dom env' 
                     \<and> pa \<notin> F \<union> {s, p, x, y} \<union> FV (Obj f' T) \<union> env_dom env' 
                     \<and> sa \<noteq> pa 
      \<longrightarrow> C, env'\<lparr>x:A\<rparr>\<lparr>y:B\<rparr>\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>
          \<turnstile> (the(sopen_option (Suc n) (Fvar x) (Fvar y) 
            (f' l))\<^bsup>[Fvar sa,Fvar pa]\<^esup>) : return(the(T^l))"
  proof 
    fix l assume "l \<in> do T" with prems(4)
    have cof: 
      "\<forall>sa pa. sa \<notin> F \<and> pa \<notin> F \<and> sa \<noteq> pa
	\<longrightarrow> C, env\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>
            \<turnstile> (the(f l)\<^bsup>[Fvar sa,Fvar pa]\<^esup>) : return(the(T^l))
	  \<and> (\<forall>env'' t'' s' p' x' y' A' B' n'.
               s' \<notin> FV t'' \<longrightarrow> p' \<notin> FV t'' \<longrightarrow> x' \<notin> FV t'' \<longrightarrow> y' \<notin> FV t'' 
               \<longrightarrow> x' \<notin> env_dom env'' \<longrightarrow> y' \<notin> env_dom env'' \<longrightarrow> x' \<noteq> y' 
               \<longrightarrow> s' \<noteq> p' 
               \<longrightarrow> (the(f l)\<^bsup>[Fvar sa,Fvar pa]\<^esup>) = {n' \<rightarrow> [Fvar s',Fvar p']} t'' 
               \<longrightarrow> env\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr> = env''\<lparr>s':A'\<rparr>\<lparr>p':B'\<rparr> 
               \<longrightarrow> C, env''\<lparr>x':A'\<rparr>\<lparr>y':B'\<rparr>
                   \<turnstile> {n' \<rightarrow> [Fvar x',Fvar y']} t'' : return(the(T^l)))"
      by simp
    from 
      `l \<in> do T` `dom f = do T` `Obj f T = {n \<rightarrow> [Fvar s,Fvar p]} t'` obj t'
      dom_sopenoption_lem[of "Suc n" "Fvar s" "Fvar p" f']
    have indomf': "l \<in> dom f'" by auto
    hence 
      opened: "the (sopen_option (Suc n) (Fvar x) (Fvar y) (f' l)) 
               = {Suc n \<rightarrow> [Fvar x,Fvar y]} the(f' l)" 
      by force
    from indomf' have FVsubset: "FV (the(f' l)) \<subseteq> FV (Obj f' T)" by force
    with 
      `s \<notin> FV t'` `p \<notin> FV t'` `x \<notin> FV t'` `y \<notin> FV t'` obj t'
      indomf' FV_option_lem[of f']
    obtain 
      "s \<notin> FV (the(f' l))" and "p \<notin> FV (the(f' l))" and
      "x \<notin> FV (the(f' l))" and "y \<notin> FV (the(f' l))" and
      "the(f l) = {Suc n \<rightarrow> [Fvar s,Fvar p]} the(f' l)" by auto
    from 
      bnd_renaming_lem[OF this(1-4) `x \<notin> env_dom env'` `y \<notin> env_dom env'`
                          `s \<noteq> p` `x \<noteq> y` this(5) `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` 
                          cof FVsubset]
    show 
      "\<forall>sa pa. sa \<notin> F \<union> {s, p, x, y} \<union> FV (Obj f' T) \<union> env_dom env' 
             \<and> pa \<notin> F \<union> {s, p, x, y} \<union> FV (Obj f' T) \<union> env_dom env' 
             \<and> sa \<noteq> pa 
        \<longrightarrow> C, env'\<lparr>x:A\<rparr>\<lparr>y:B\<rparr>\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>
      \<turnstile> (the(sopen_option (Suc n) (Fvar x) (Fvar y) 
              (f' l))\<^bsup>[Fvar sa,Fvar pa]\<^esup>) : return(the(T^l))"
      by (subst opened, assumption)
  qed
  from typing.T_Obj[OF ok dom finF this] t' show ?case by simp
next
  case (T_Active e C t T e' t' s p x y A B n)
  note pred = this(3)
  from sopen_eq_Active[OF sym[OF `Active t = {n \<rightarrow> [Fvar s,Fvar p]} t'`]]
  obtain a where "t = {n \<rightarrow> [Fvar s, Fvar p]} a" and "t' = Active a"
    by auto
  from this(2) `s \<notin> FV t'` `p \<notin> FV t'` `x \<notin> FV t'` `y \<notin> FV t'`
  have "s \<notin> FV a" and "p \<notin> FV a" and "x \<notin> FV a" and "y \<notin> FV a"
    by auto
  from
    typing.T_Active[OF ok_add_ok2[OF `ok e` `e = e'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` 
                                     `x \<notin> env_dom e'` 
                                     `y \<notin> env_dom e'` `x \<noteq> y`]
                       pred[OF this `x \<notin> env_dom e'` 
                               `y \<notin> env_dom e'` `s \<noteq> p` `x \<noteq> y`
                               `t = {n \<rightarrow> [Fvar s, Fvar p]} a` 
                               `e = e'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>`]]
    `t' = Active a`
  show ?case by simp
next
  case (T_ActRef e am G T fm e' t' s p x y A B n)
  from 
    sopen_eq_ActRef[OF sym[OF `ActRef G = {n \<rightarrow> [Fvar s,Fvar p]} t'`]]
    typing.T_ActRef[OF ok_add_ok2[OF `ok e` `e = e'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` 
                                     `x \<notin> env_dom e'` 
                                     `y \<notin> env_dom e'` `x \<noteq> y`]
                       `am G = Some T` `G \<in> dom am`]
  show ?case by simp
next
  case (T_FutRef e fm fi T am e' t' s p x y A B n)
  from 
    sopen_eq_FutRef[OF sym[OF `FutRef fi = {n \<rightarrow> [Fvar s,Fvar p]} t'`]]
    typing.T_FutRef[OF ok_add_ok2[OF `ok e` `e = e'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` 
                                     `x \<notin> env_dom e'` 
                                     `y \<notin> env_dom e'` `x \<noteq> y`]
                       `fm fi = Some T` `fi \<in> dom fm`]
  show ?case by simp
qed

lemma type_renaming: 
  "\<lbrakk> C, e\<lparr>s:A\<rparr>\<lparr>p:B\<rparr> \<turnstile> {n \<rightarrow> [Fvar s,Fvar p]} t : T; 
     s \<notin> FV t; p \<notin> FV t; x \<notin> FV t; y \<notin> FV t; 
     x \<notin> env_dom e; y \<notin> env_dom e; x \<noteq> y; s \<noteq> p\<rbrakk> 
  \<Longrightarrow> C, e\<lparr>x:A\<rparr>\<lparr>y:B\<rparr> \<turnstile> {n \<rightarrow> [Fvar x,Fvar y]} t : T"
  by (auto simp: type_renaming')
(* too weak, as we need specific s,p *)

lemma obj_inv_elim': 
  assumes 
  "C, e \<turnstile> Obj f U : U" and 
  nin_s: "s \<notin> FV (Obj f U) \<union> env_dom e" and
  nin_p: "p \<notin> FV (Obj f U) \<union> env_dom e" and "s \<noteq> p"
  shows 
  "(dom f = do U) \<and> (\<forall>l\<in>do U. C, e\<lparr>s:U\<rparr>\<lparr>p:param(the(U^l))\<rparr>
                               \<turnstile> (the(f l)\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the(U^l)))"
  using assms
proof (cases rule: typing.cases)
  case T_Var thus ?thesis by simp (* contradiction *)
next
  case T_Upd thus ?thesis by simp (* contradiction *)
next
  case T_Call thus ?thesis by simp (* contradiction *)
next
  case T_Active thus ?thesis by simp (* contradiction *)
next
  case T_ActRef thus ?thesis by simp (* contradiction *)
next
  case T_FutRef thus ?thesis by simp (* contradiction *)
next
  case (T_Obj env f' T L C')
  note pred = this(8)
  from `e = env` `Obj f U = Obj f' T` `dom f' = do T`
  have "dom f = do U" by simp
  thus ?thesis 
  proof (simp, intro strip)
    fix l assume "l \<in> do U"
    from `finite L` finite_FV[of "Obj f U"] have "finite (L \<union> FV (Obj f U) \<union> {s,p})"
      by simp
    from exFresh_s_p_cof[OF this]
    obtain sa pa where 
      "sa \<noteq> pa" and
      nin_sa: "sa \<notin> L \<union> FV (Obj f U)" and
      nin_pa: "pa \<notin> L \<union> FV (Obj f U)" by auto
    with `e = env` `Obj f U = Obj f' T` `U = T` `l \<in> do U` `C = C'` pred
    have "C, e\<lparr>sa:U\<rparr>\<lparr>pa:param(the(U^l))\<rparr> \<turnstile> (the(f l)\<^bsup>[Fvar sa,Fvar pa]\<^esup>) : return(the(U^l))"
      by simp
    moreover
    from `l \<in> do U` `Obj f U = Obj f' T` `U = T` `dom f' = do T`  
    have "l \<in> dom f" by simp
    with nin_s nin_p nin_sa nin_pa FV_option_lem[of f]
    have 
      "sa \<notin> FV (the(f l)) \<and> pa \<notin> FV (the(f l)) 
       \<and> s \<notin> FV (the(f l)) \<and> p \<notin> FV (the(f l))
       \<and> s \<notin> env_dom e \<and> p \<notin> env_dom e" by auto
    ultimately
    show "C, e\<lparr>s:U\<rparr>\<lparr>p:param(the(U^l))\<rparr> \<turnstile> (the(f l)\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the(U^l))"
      using type_renaming[OF _ _ _ _ _ _ _ `s \<noteq> p` `sa \<noteq> pa`]
      by (simp add: openz_def)
  qed
qed

lemma dom_lem: "C, e \<turnstile> Obj f (Object fun) : Object fun \<Longrightarrow> dom f = dom fun"
  by (erule typing.cases, auto)

lemma abs_typeE: 
  assumes "C, e \<turnstile> Call (Obj f U) l b : T"
  shows 
  "(\<exists>F. finite F 
      \<and> (\<forall>s p. s \<notin> F \<and> p \<notin> F \<and> s \<noteq> p 
          \<longrightarrow> C, e\<lparr>s:U\<rparr>\<lparr>p: param(the(U^l))\<rparr> \<turnstile> (the(f l)\<^bsup>[Fvar s,Fvar p]\<^esup>) : T) \<Longrightarrow> P) 
  \<Longrightarrow> P"
  using assms
proof (cases rule: typing.cases)
  case T_Var thus ?thesis by simp (* contradiction *)
next
  case T_Obj thus ?thesis by simp (* contradiction *)
next
  case T_Upd thus ?thesis by simp (* contradiction *)
next
  case T_Active thus ?thesis by simp (* contradiction *)
next
  case T_ActRef thus ?thesis by simp (* contradiction *)
next
  case T_FutRef thus ?thesis by simp (* contradiction *)
next
  case (T_Call C' env t1 Ta t2 la)
  assume 
    cof: "\<exists>F. finite F 
            \<and> (\<forall>s p. s \<notin> F \<and> p \<notin> F \<and> s \<noteq> p 
                \<longrightarrow> C, e\<lparr>s:U\<rparr>\<lparr>p: param(the(U^l))\<rparr> \<turnstile> (the(f l)\<^bsup>[Fvar s,Fvar p]\<^esup>) : T) 
          \<Longrightarrow> P"
  from 
    `e = env` `Call (Obj f U) l b = Call t1 la t2` `T = return (the(Ta^la))`
    `C', env \<turnstile> t1 : Ta` `la \<in> do Ta` obj_inv[of C e f U Ta] `C = C'`
  obtain "C, e \<turnstile> (Obj f U) : U" and "T = return(the(U^l))" and "l \<in> do U" 
    by simp
  from obj_inv_elim[OF this(1)] this(2-3) cof show ?thesis by blast
qed

subsubsection {* Substitution preserves Well-Typedness *}
lemma bigger_env_lemma[rule_format]: 
  assumes "C, e \<turnstile> t : T"
  shows "\<forall>x X. x \<notin> env_dom e \<longrightarrow> C, e\<lparr>x:X\<rparr> \<turnstile> t: T"
proof -
  def pred_cof \<equiv> "\<lambda>L C env t T l. 
    \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p 
     \<longrightarrow> C, env\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> \<turnstile> (t\<^bsup>[Fvar s,Fvar p]\<^esup>) : return (the(T^l))"
  from assms show ?thesis
  proof (induct
      taking: "\<lambda>C env t T l. \<forall>x X. x \<notin> env_dom env 
                            \<longrightarrow> (\<exists>L. finite L \<and> pred_cof L C (env\<lparr>x:X\<rparr>) t T l)"
      rule: typing_induct)
    case Call thus ?case by auto
  next
    case Active thus ?case by (auto simp: ok_add_ok)
  next
    case ActRef thus ?case by (auto simp: ok_add_ok)
  next
    case FutRef thus ?case by (auto simp: ok_add_ok)
  next
    case (Fvar C env Ta xa) thus ?case
    proof (intro strip)
      fix x X assume "x \<notin> env_dom env"
      from 
	get_env_smaller[OF `xa \<in> env_dom env` this]
        T_Var[OF ok_add_ok[OF `ok env` this]
        env_bigger[OF this `xa \<in> env_dom env`]]
        `the env!xa = Ta`
      show "C, env\<lparr>x:X\<rparr> \<turnstile> Fvar xa : Ta" by simp
    qed
  next
    case (Obj C env Ta f) note pred_o = this(3)
    def pred_cof' \<equiv> "\<lambda>x X b l. \<exists>L. finite L \<and> pred_cof L C (env\<lparr>x:X\<rparr>) (the b) Ta l"
    from pred_o
    have pred: "\<forall>x X. x \<notin> env_dom env \<longrightarrow> (\<forall>l\<in>dom f. pred_cof' x X (f l) l)"
      by (intro fmap_ball_all_2'[of f "\<lambda>x X. x \<notin> env_dom env" pred_cof'],
	unfold pred_cof_def pred_cof'_def, simp)
    show ?case
    proof (intro strip)
      fix x X 
      def pred_bnd \<equiv> "\<lambda>s p b l. C, env\<lparr>x:X\<rparr>\<lparr>s:Ta\<rparr>\<lparr>p:param (the(Ta^l))\<rparr> 
                                \<turnstile> (the b\<^bsup>[Fvar s,Fvar p]\<^esup>) : return (the(Ta^l))"
      assume "x \<notin> env_dom env"
      with pred fmap_ex_cof[of f pred_bnd] `dom f = do Ta`
      obtain L where
	"finite L" and "\<forall>l\<in>do Ta. pred_cof L C (env\<lparr>x:X\<rparr>) (the(f l)) Ta l"
	unfolding pred_bnd_def pred_cof_def pred_cof'_def
	by auto
      from 
	T_Obj[OF ok_add_ok[OF `ok env` `x \<notin> env_dom env`] `dom f = do Ta` this(1)]
	this(2)
      show "C, env\<lparr>x:X\<rparr> \<turnstile> Obj f Ta : Ta"
	unfolding pred_cof_def
	by simp
    qed
  next
    case (Upd C env Ta t l u) 
    note pred_t = this(2) and pred_u = this(4)
    show ?case
    proof (intro strip)
      fix x X assume "x \<notin> env_dom env" 
      with pred_u obtain L where
	"finite L" and "pred_cof L C (env\<lparr>x:X\<rparr>) u Ta l" by auto
      with `l \<in> do Ta` `x \<notin> env_dom env` pred_t
      show "C, env\<lparr>x:X\<rparr> \<turnstile> Upd t l u : Ta" 
	unfolding pred_cof_def
	by auto
    qed
  next
    case (Bnd C env Ta l t L) note pred = this(3)
    show ?case
    proof (intro strip)
      fix x X assume "x \<notin> env_dom env"
      thus "\<exists>L. finite L \<and> pred_cof L C (env\<lparr>x:X\<rparr>) t Ta l"
      proof (rule_tac x = "L \<union> {x}" in exI, simp add: `finite L`, 
	  unfold pred_cof_def, auto)
	fix s p
	assume 
	  "s \<notin> L" and "p \<notin> L" and "s \<noteq> p" and
	  "s \<noteq> x" and "p \<noteq> x"
	note 
	  subst_add[OF not_sym[OF `s \<noteq> x`]]
	  subst_add[OF not_sym[OF `p \<noteq> x`]]
	from 
	  this(1)[of env X Ta] this(2)[of "env\<lparr>s:Ta\<rparr>" X "param (the(Ta^l))"]
          pred `s \<notin> L` `p \<notin> L` `s \<noteq> p`
	  not_in_env_bigger_2[OF `x \<notin> env_dom env` 
	                         not_sym[OF `s \<noteq> x`] not_sym[OF `p \<noteq> x`]]
	show 
	  "C, env\<lparr>x:X\<rparr>\<lparr>s:Ta\<rparr>\<lparr>p:param (the(Ta^l))\<rparr> 
	  \<turnstile> (t\<^bsup>[Fvar s,Fvar p]\<^esup>) : return (the(Ta^l))"
	  by auto
      qed
    qed
  qed
qed

lemma subst_lemma[rule_format]: 
  assumes
  "C, e \<turnstile> t : T"
  shows
  "\<forall>e' x X v. x \<notin> env_dom e' \<longrightarrow> C, e' \<turnstile> v : X \<longrightarrow> e = e'\<lparr>x:X\<rparr> 
    \<longrightarrow> C, e' \<turnstile> [x \<rightarrow> v] t : T"
proof -
  def pred_cof \<equiv> "\<lambda>L C e t T l. \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p 
                             \<longrightarrow> C, e\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> 
                                 \<turnstile> (t\<^bsup>[Fvar s, Fvar p]\<^esup>) : return (the(T^l))"
  from assms show ?thesis
  proof (induct 
      taking: "\<lambda>C e t T l. \<forall>e' x X v. x \<notin> env_dom e' \<longrightarrow> C, e' \<turnstile> v : X 
                          \<longrightarrow> e = e'\<lparr>x:X\<rparr> 
                          \<longrightarrow> (\<exists>L. finite L \<and> pred_cof L C e' ([x \<rightarrow> v] t) T l)"
      rule: typing_induct)
    case Call thus ?case by auto
  next
    case Active thus ?case by (auto simp: not_malformed_smaller)
  next
    case ActRef thus ?case by (auto simp: not_malformed_smaller)
  next
    case FutRef thus ?case by (auto simp: not_malformed_smaller)
  next
    case (Fvar C e T y) 
    show ?case
    proof (intro strip)
      fix e' x X v
      assume "x \<notin> env_dom e'" and "C, e' \<turnstile> v : X" and "e = e'\<lparr>x:X\<rparr>"
      with 
	add_get[OF this(1) not_malformed_smaller[of e' x X]]
	get_env_smaller[OF in_env_smaller[of y e' x X] this(1)]
	`ok e` `y \<in> env_dom e` `the e!y = T`
      show "C, e' \<turnstile> [x \<rightarrow> v] Fvar y : T"
	by (cases "x = x", auto intro: not_malformed_smaller in_env_smaller)
    qed
  next
    case (Upd C e T t l u) 
    note pred_t = this(2) and pred_u = this(4)
    show ?case
    proof (intro strip)
      fix e' x X v
      assume "x \<notin> env_dom e'" and "C, e' \<turnstile> v : X" and "e = e'\<lparr>x:X\<rparr>"
      with pred_u
      obtain L where
	"finite L" and "pred_cof L C e' ([x \<rightarrow> v] u) T l"
	by blast
      with `l \<in> do T` `x \<notin> env_dom e'` `C, e' \<turnstile> v : X` `e = e'\<lparr>x:X\<rparr>` pred_t
      show "C, e' \<turnstile> [x \<rightarrow> v] Upd t l u : T"
	unfolding pred_cof_def
	by auto
    qed
  next
    case (Obj C e T f) note pred_o = this(3)
    def pred_pre \<equiv> "\<lambda>e' x X v. x \<notin> env_dom e' \<and> C, e' \<turnstile> v : X \<and> e = e'\<lparr>x:X\<rparr>"
    def pred_cof' \<equiv> "\<lambda>e' x (X::type) v b l::Label. 
      \<exists>L. finite L \<and> pred_cof L C e' ([x \<rightarrow> v] the b) T l"
    from pred_o
    have pred: "\<forall>e' x X v. pred_pre e' x X v 
                 \<longrightarrow> (\<forall>l\<in>dom f. pred_cof' e' x X v (f l) l)"
      by (intro fmap_ball_all_4'[of f pred_pre pred_cof'],
	unfold pred_pre_def pred_cof'_def, blast)
    show ?case
    proof (intro strip)
      fix e' x X v
      def pred_bnd \<equiv> "\<lambda>s p b l. C, e'\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> 
                                \<turnstile> ([x \<rightarrow> v] the b\<^bsup>[Fvar s,Fvar p]\<^esup>) : return (the(T^l))"
      assume "x \<notin> env_dom e'" and "C, e' \<turnstile> v : X" and "e = e'\<lparr>x:X\<rparr>"
      hence "pred_pre e' x X v" unfolding pred_pre_def by simp
      with 
	pred fmap_ex_cof[of f pred_bnd] `dom f = do T`
      obtain L where
	"finite L" and "\<forall>l\<in>do T. pred_cof L C e' ([x \<rightarrow> v] the(f l)) T l"
	unfolding pred_cof'_def pred_bnd_def pred_cof_def
	by auto
      from this(2) `dom f = do T` ssubst_option_lem[of f x v]
      have "\<forall>l\<in>do T. pred_cof L C e' (the(ssubst_option x v (f l))) T l"
	by auto
      moreover
      from `dom f = do T` dom_ssubstoption_lem[of x v f]
      have "dom (\<lambda>l. ssubst_option x v (f l)) = do T"
	by simp
      note T_Obj[OF _ this `finite L`]
      ultimately
      show "C, e' \<turnstile> [x \<rightarrow> v] Obj f T : T"
	using
	  `ok e` `e = e'\<lparr>x:X\<rparr>` not_malformed_smaller[of e' x X]
	unfolding pred_cof_def
	by auto
    qed
  next
    case (Bnd C e T l t L) note pred = this(3)
    show ?case
    proof (intro strip)
      fix e' x X v
      assume "x \<notin> env_dom e'" and "C, e' \<turnstile> v : X" and "e = e'\<lparr>x:X\<rparr>"
      with `finite L` `ok e` not_malformed_smaller[of e' x X]
      show "\<exists>L. finite L \<and> pred_cof L C e' ([x \<rightarrow> v] t) T l"
      proof (rule_tac x = "L \<union> {x} \<union> env_dom e'" in exI, intro conjI, simp,
	unfold pred_cof_def, auto)
	fix s p 
	assume
	  "s \<notin> L" and "p \<notin> L" and "s \<noteq> p" and
	  "s \<notin> env_dom e'" and "p \<notin> env_dom e'" and
	  "s \<noteq> x" and "p \<noteq> x"
	note 
	  typing =
	  bigger_env_lemma[OF bigger_env_lemma[OF `C, e' \<turnstile> v : X` this(4)]
	                      not_in_env_bigger[OF this(5) 
	                                           not_sym[OF this(3)]]] and
	  env_dom =
	  not_in_env_bigger_2[OF `x \<notin> env_dom e'` not_sym[OF this(6)]
	                         not_sym[OF this(7)]]
	from 
	  `e = e'\<lparr>x:X\<rparr>`
	  `s \<noteq> x` subst_add[of x s e' X T]
	  `p \<noteq> x` subst_add[of x p "e'\<lparr>s:T\<rparr>" X "param (the(T^l))"]
	  `s \<notin> L` `p \<notin> L` `s \<noteq> p` pred
	  env_dom[of T "param (the(T^l))"] typing[of T "param (the(T^l))"]
	have "C, e'\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> \<turnstile> [x \<rightarrow> v] (t\<^bsup>[Fvar s, Fvar p]\<^esup>) : return (the(T^l))"
	  by simp
	with 
	  `s \<noteq> x` `p \<noteq> x`
	  ssubst_openz_distrib[OF typing_regular''[OF `C, e' \<turnstile> v : X`]]
	show "C, e'\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> \<turnstile> [x \<rightarrow> v] t\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
	  by simp
      qed
    qed
  qed
qed

lemma bnd_disj_env_lem: 
  assumes 
  "ok e1" and "env_dom e1 \<inter> env_dom e2 = {}" and "ok e2" and
  "\<forall>s p. s \<notin> F \<and> p \<notin> F \<and> s \<noteq> p 
    \<longrightarrow> C, e1\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr>
        \<turnstile> (t2\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the(T^l)) 
        \<and> (env_dom (e1\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr>) \<inter> env_dom e2 = {} 
    \<longrightarrow> ok e2 
    \<longrightarrow> C, e1\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr>+e2 
        \<turnstile> (t2\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the(T^l)))"
  shows 
  "\<forall>s p. s \<notin> F \<union> env_dom (e1+e2) \<and> p \<notin> F \<union> env_dom (e1+e2) \<and> s \<noteq> p
    \<longrightarrow> C, (e1+e2)\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr> \<turnstile> (t2\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the(T^l))"
proof (intro strip, elim conjE)
  fix s p assume 
    nin_s: "s \<notin> F \<union> env_dom (e1+e2)" and
    nin_p: "p \<notin> F \<union> env_dom (e1+e2)" and "s \<noteq> p"
  from 
    this(1-2) env_add_dom_2[OF assms(1) _ _ this(3)]
    assms(2) env_app_dom[OF assms(1-3)]
  have "env_dom (e1\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr>) \<inter> env_dom e2 = {}" by simp
  with 
    env_app_add2[OF assms(1-3) _ _ _ _ `s \<noteq> p`]
    env_app_dom[OF assms(1-3)] `ok e2` prems(4) nin_s nin_p `s \<noteq> p`
  show "C, (e1+e2)\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr> \<turnstile> (t2\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the(T^l))"
    by auto
qed

lemma disjunct_env[rule_format]: 
  assumes "C, e \<turnstile> t : A" 
  shows "\<forall>e'. env_dom e \<inter> env_dom e' = {} \<longrightarrow> ok e' \<longrightarrow> C, (e+e') \<turnstile> t : A"
proof -
  def pred_cof \<equiv> "\<lambda>L C e t T l. \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p
                   \<longrightarrow> C, e\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> \<turnstile> t\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
  from assms show ?thesis
  proof (induct 
      taking: "\<lambda>C e t T l. \<forall>e'. env_dom e \<inter> env_dom e' = {} \<longrightarrow> ok e'
                            \<longrightarrow> (\<exists>L. finite L \<and> pred_cof L C e+e' t T l)"
      rule: typing_induct)
    case Call thus ?case by auto
  next
    case Active thus ?case by auto
  next
    case ActRef thus ?case by auto
  next
    case FutRef thus ?case by auto
  next
    case (Fvar C e T x) 
    show ?case proof (intro strip)
      fix e':: "Sigma.type environment" 
      assume "env_dom e \<inter> env_dom e' = {}" and "ok e'"
      from env_app_dom[OF `ok e` this] `x \<in> env_dom e`
      have indom: "x \<in> env_dom (e+e')" by simp
      from `ok e` `x \<in> env_dom e` `the e!x = T` `env_dom e \<inter> env_dom e' = {}` `ok e'` 
      have "the (e+e')!x = T" by simp
      from 
	typing.T_Var[OF env_app_ok[OF `ok e` `env_dom e \<inter> env_dom e' = {}` `ok e'`] 
                        indom this] 
      show "C, e+e' \<turnstile> Fvar x : T" by assumption
    qed
  next
    case (Upd C e T t l u) note pred_t = this(2) and pred_u = this(4)
    show ?case proof (intro strip)
      fix e':: "Sigma.type environment"
      assume "env_dom e \<inter> env_dom e' = {}" and "ok e'"
      with pred_t have "C, e+e' \<turnstile> t : T" by blast
      moreover
      from `env_dom e \<inter> env_dom e' = {}` `ok e'` pred_u
      obtain L where "finite L" and "pred_cof L C e+e' u T l"
	by auto
      ultimately show "C, e+e' \<turnstile> Upd t l u : T"
	using `l \<in> do T` by (auto simp: pred_cof_def)
    qed
  next
    case (Obj C e T f) note pred_o = this(3)
    def pred_pre \<equiv> "\<lambda>e'::type environment. env_dom e \<inter> env_dom e' = {} \<and> ok e'"
    def pred_cof' \<equiv> "\<lambda>(e'::type environment) b l. \<exists>L. finite L \<and> pred_cof L C e+e' (the b) T l"
    from pred_o 
    have pred: "\<forall>e'::type environment. pred_pre e' \<longrightarrow> (\<forall>l\<in>dom f. pred_cof' e' (f l) l)"
      by (intro fmap_ball_all_1'[of f pred_pre pred_cof'],
	unfold pred_pre_def pred_cof'_def, blast)
    show ?case proof (intro strip)
      fix e':: "Sigma.type environment"
      def pred_bnd \<equiv> "\<lambda>s p b l. C, (e+e')\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> 
	                        \<turnstile> the b\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
      assume "env_dom e \<inter> env_dom e' = {}" and "ok e'"
      hence "\<forall>l\<in>dom f. (\<exists>L. finite L \<and> pred_cof L C e+e' (the (f l)) T l)"
	using pred
	unfolding pred_pre_def pred_cof'_def
	by blast
      with fmap_ex_cof[of f pred_bnd] `dom f = do T`
      obtain L where "finite L" and "\<forall>l\<in>do T. pred_cof L C e+e' (the(f l)) T l"
	unfolding pred_bnd_def pred_cof_def
	by auto
      from this(2) `ok e` `env_dom e \<inter> env_dom e' = {}` `ok e'`
      show "C, e+e' \<turnstile> Obj f T : T"
	unfolding pred_cof_def
	by (intro T_Obj[OF env_app_ok[of e e'] `dom f = do T` `finite L`], 
	  simp+)
    qed
  next
    case (Bnd C e T l t L) note pred = this(3)
    from `finite L`
    show ?case 
    proof (intro strip)
      fix e':: "Sigma.type environment"
      assume "env_dom e \<inter> env_dom e' = {}" and "ok e'"
      note eappdom = env_app_dom[OF `ok e` this]
      from 
	sym[OF this]
	finite_UnI[OF `finite L` finite_UnI[OF ok_finite[OF `ok e`] ok_finite[OF `ok e'`]]]
      show "\<exists>L. finite L \<and> pred_cof L C e+e' t T l"
      proof (rule_tac x = "L \<union> env_dom (e+e')" in exI, simp, 
	  unfold pred_cof_def, simp (no_asm_simp), 
	  intro strip, elim conjE)
	fix s p assume 
	  "s \<notin> L" and "p \<notin> L" and "s \<noteq> p" and 
	  "s \<notin> env_dom (e+e')" and "p \<notin> env_dom (e+e')"
	from this(4-5) eappdom
	have 
	  "s \<notin> env_dom e" and "s \<notin> env_dom e'" and 
	  "p \<notin> env_dom e" and "p \<notin> env_dom e'"
	  by auto
	with 
	  env_add_dom_2[OF `ok e` this(1) this(3) `s \<noteq> p`]
	  `env_dom e \<inter> env_dom e' = {}` `s \<noteq> p`
	  env_app_add2[OF `ok e` `env_dom e \<inter> env_dom e' = {}` `ok e'`
	                  this `s \<noteq> p`]
	have 
	  "env_dom (e\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr>) \<inter> env_dom e' = {}" and
	  "(e+e')\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr> = e\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr>+e'"
	  by auto
	with pred `ok e'` `s \<notin> L` `p \<notin> L` `s \<noteq> p`
	show "C, (e+e')\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr> \<turnstile> t\<^bsup>[Fvar s, Fvar p]\<^esup> : return(the(T^l))"
	  by simp
      qed
    qed
  qed
qed

text {* Typed in the Empty Environment implies typed in any Environment *}
lemma empty_env: 
  assumes "C, (Env empty) \<turnstile> t : A" and "ok env"
  shows "C, env \<turnstile> t : A"
proof -
  from `ok env` have "env = (Env empty)+env" by (cases env, auto)
  with disjunct_env[OF assms(1) _ assms(2)] show ?thesis by simp
qed

lemma bnd_open_lem:
  assumes
  pred_bnd:
  "\<forall>sa pa. sa \<notin> F \<and> pa \<notin> F \<and> sa \<noteq> pa 
    \<longrightarrow> C, env\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr> 
        \<turnstile> (t\<^bsup>[Fvar sa,Fvar pa]\<^esup>) : return(the(T^l)) 
     \<and> (\<forall>env'' t'' s' p' x' y' A' B' n'. s' \<notin> FV t'' \<union> FV x' \<union> FV y' 
         \<longrightarrow> p' \<notin> FV t'' \<union> FV x' \<union> FV y' \<longrightarrow> s' \<noteq> p' 
         \<longrightarrow> C, env'' \<turnstile> x' : A' \<longrightarrow> C, env'' \<turnstile> y' : B' 
         \<longrightarrow> (t\<^bsup>[Fvar sa,Fvar pa]\<^esup>) = {n' \<rightarrow> [Fvar s',Fvar p']} t'' 
         \<longrightarrow> env\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr> = env''\<lparr>s':A'\<rparr>\<lparr>p':B'\<rparr> 
         \<longrightarrow> C, env'' \<turnstile> {n' \<rightarrow> [x',y']} t'' : return(the(T^l)))" and
  "ok env" and "env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>" and 
  "s \<notin> FV t'' \<union> FV x \<union> FV y" and "p \<notin> FV t'' \<union> FV x \<union> FV y" and "s \<noteq> p" and 
  "C, env' \<turnstile> x : A" and "C, env' \<turnstile> y : B" and 
  "t = {Suc n \<rightarrow> [Fvar s,Fvar p]} t'" and "FV t' \<subseteq> FV t''"
  shows 
  "\<forall>sa pa. sa \<notin> F \<union> {s,p} \<union> env_dom env' 
         \<and> pa \<notin> F \<union> {s,p} \<union> env_dom env' \<and> sa \<noteq> pa
    \<longrightarrow> C, env'\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>
        \<turnstile> ({Suc n \<rightarrow> [x,y]} t'\<^bsup>[Fvar sa, Fvar pa]\<^esup>) : return(the(T^l))"
proof (intro strip, elim conjE)
  fix sa pa assume 
    nin_sa: "sa \<notin> F \<union> {s,p} \<union> env_dom env'" and 
    nin_pa: "pa \<notin> F \<union> {s,p} \<union> env_dom env'" and "sa \<noteq> pa"
  hence "sa \<notin> F \<and> pa \<notin> F \<and> sa \<noteq> pa" by auto
  moreover
  {
    fix a assume "a \<notin> FV t'' \<union> FV x \<union> FV y" and "a \<in> {s,p}"
    with 
      `FV t' \<subseteq> FV t''` nin_sa nin_pa `sa \<noteq> pa` 
      sopen_FV[of 0 "Fvar sa" "Fvar pa" t']
    have "a \<notin> FV (t'\<^bsup>[Fvar sa,Fvar pa]\<^esup>) \<union> FV x \<union> FV y" by (auto simp: openz_def)
  } note 
      this[OF `s \<notin> FV t'' \<union> FV x \<union> FV y`] 
      this[OF `p \<notin> FV t'' \<union> FV x \<union> FV y`]
  moreover
  {
    from `ok env` `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` ok_add_2[of env' s A p B]
    have "ok env'" by simp
    from nin_sa nin_pa `sa \<noteq> pa` env_add_dom[OF this] 
    obtain "sa \<notin> env_dom env'" and "pa \<notin> env_dom (env'\<lparr>sa:T\<rparr>)" by auto
    note 
      bigger_env_lemma[OF bigger_env_lemma[OF `C, env' \<turnstile> x : A` this(1)] this(2)]
      bigger_env_lemma[OF bigger_env_lemma[OF `C, env' \<turnstile> y : B` this(1)] this(2)]
  }note 
      this(1)[of "param(the(T^l))"] 
      this(2)[of "param(the(T^l))"]
  moreover
  from `t = {Suc n \<rightarrow> [Fvar s,Fvar p]} t'` sopen_commute[of 0 "Suc n" sa pa s p t']
  have "(t\<^bsup>[Fvar sa,Fvar pa]\<^esup>) = {Suc n \<rightarrow> [Fvar s,Fvar p]} (t'\<^bsup>[Fvar sa,Fvar pa]\<^esup>)"
    by (simp add: add openz_def)
  moreover
  from 
    subst_add[of p sa "env'\<lparr>s:A\<rparr>" B T] subst_add[of s sa env' A T]
    subst_add[of p pa "env'\<lparr>sa:T\<rparr>\<lparr>s:A\<rparr>" B "param(the(T^l))"] 
    subst_add[of s pa "env'\<lparr>sa:T\<rparr>" A "param(the(T^l))"]
    `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` nin_sa nin_pa
  have "env\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr> = env'\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>"
    by auto
  ultimately
  show 
    "C, env'\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>
     \<turnstile> ({Suc n \<rightarrow> [x,y]} t'\<^bsup>[Fvar sa, Fvar pa]\<^esup>) : return(the(T^l))"
    using 
      pred_bnd `s \<noteq> p`
      sopen_commute_gen[OF lc_Fvar[of sa] lc_Fvar[of pa] 
                           typing_regular''[OF `C, env' \<turnstile> x : A`] 
                           typing_regular''[OF `C, env' \<turnstile> y : B`]
                           not_sym[OF Suc_not_Zero]]
    by (auto simp: openz_def)
qed

(* A variation of the Type Renaming lemma above. This is stronger and could be extended to show type renaming, using that a term typed in one environment is typed in any bigger environment *)
(* TODO: refactor to work with typing_induct *)
lemma open_lemma':
  shows 
  "C, e \<turnstile> t : T 
  \<Longrightarrow> (\<And>env t' s p x y A B n. s \<notin> FV t' \<union> FV x \<union> FV y 
         \<Longrightarrow> p \<notin> FV t' \<union> FV x \<union> FV y \<Longrightarrow> s \<noteq> p
         \<Longrightarrow> C, env \<turnstile> x : A \<Longrightarrow> C, env \<turnstile> y : B 
         \<Longrightarrow> t = {n \<rightarrow> [Fvar s,Fvar p]} t'
         \<Longrightarrow> e = env\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>
         \<Longrightarrow> C, env \<turnstile> {n \<rightarrow> [x,y]} t' : T)"
proof (induct set:typing)
  case (T_ActRef env am G T fm env' t' s p x y A B n)
  with ok_add_2[of env' s A p B]
  have "ok env'" by simp
  from 
    typing.T_ActRef[OF this `am G = Some T` `G \<in> dom am`]
    sopen_eq_ActRef[OF sym[OF `ActRef G = {n \<rightarrow> [Fvar s, Fvar p]} t'`]]
  show ?case by simp
next
  case (T_FutRef env fm fi T am env' t' s p x y A B n)
  with ok_add_2[of env' s A p B]
  have "ok env'" by simp
  from 
    typing.T_FutRef[OF this `fm fi = Some T` `fi \<in> dom fm`]
    sopen_eq_FutRef[OF sym[OF `FutRef fi = {n \<rightarrow> [Fvar s, Fvar p]} t'`]]
  show ?case by simp
next
  case (T_Active env C t T env' t' s p x y A B n)
  note pred = this(3)
  from sopen_eq_Active[OF sym[OF `Active t = {n \<rightarrow> [Fvar s, Fvar p]} t'`]]
  obtain a where "t = {n \<rightarrow> [Fvar s, Fvar p]} a" and "t' = Active a"
    by auto
  from this(2) `s \<notin> FV t' \<union> FV x \<union> FV y` `p \<notin> FV t' \<union> FV x \<union> FV y`
  have "s \<notin> FV a \<union> FV x \<union> FV y" and "p \<notin> FV a \<union> FV x \<union> FV y"
    by auto
  from 
    `t' = Active a`
    typing.T_Active[OF _ pred[OF this `s \<noteq> p` 
                                 `C, env' \<turnstile> x : A` `C, env' \<turnstile> y : B`
                                 `t = {n \<rightarrow> [Fvar s, Fvar p]} a` 
                                 `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>`]]
    `ok env` `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>`
    ok_add_2[of env' s A p B]
  show ?case by auto
next
  case (T_Var env x T C env' t' s p y z A B n)
  from sopen_eq_Fvar[OF sym[OF `Fvar x = {n \<rightarrow> [Fvar s,Fvar p]} t'`]] 
  show ?case
  proof (elim disjE conjE)
    assume "t' = Fvar x" 
    with `s \<notin> FV t' \<union> FV y \<union> FV z` `p \<notin> FV t' \<union> FV y \<union> FV z` 
    obtain "x \<noteq> s" and "x \<noteq> p" by auto
    from `x \<in> env_dom env` `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` in_env_smaller2[OF _ this]
    have indom: "x \<in> env_dom env'" by simp
    from 
      `ok env` `the env!x = T` `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>`
      ok_add_2[of env' s A p B] get_env_smaller2[OF this _ _ `s \<noteq> p`]
    have "the env'!x = T" by simp
    from 
      `ok env` `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` `t' = Fvar x`
      ok_add_2[of env' s A p B] typing.T_Var[OF _ indom this]
    show ?case by simp
  next
    assume "x = s"
    with 
      `ok env` `the env!x = T` `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` 
      add_get2_1[of env' s A p B] 
    have "T = A" by simp
    moreover assume "t' = Bvar (Self n)"
    ultimately show ?thesis using `C, env' \<turnstile> y : A` by simp
  next
    assume "x = p" 
    with 
      `ok env` `the env!x = T` `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` 
      add_get2_2[of env' s A p B] have "T = B" by simp
    moreover assume "t' = Bvar (Param n)"
    ultimately show ?thesis using `C, env' \<turnstile> z : B` by simp
  qed
next
  case (T_Upd F C env T l t2 t1 env' t' s p x y A B n)
  from sopen_eq_Upd[OF sym[OF `Upd t1 l t2 = {n \<rightarrow> [Fvar s,Fvar p]} t'`]]
  obtain t1' t2' where 
    t1': "t1 = {n \<rightarrow> [Fvar s,Fvar p]} t1'" and
    t2': "t2 = {Suc n \<rightarrow> [Fvar s,Fvar p]} t2'" and
    t': "t' = Upd t1' l t2'" by auto
  hence "FV t2' \<subseteq> FV t'" by auto
  from 
    `s \<notin> FV t' \<union> FV x \<union> FV y` `p \<notin> FV t' \<union> FV x \<union> FV y`
    t' `finite F` ok_finite[OF typing_regular'[OF `C, env' \<turnstile> x : A`]]
    typing.T_Upd[OF _ bnd_open_lem[OF prems(2) 
                    typing_regular'[OF `C, env \<turnstile> t1 : T`] 
                    `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` 
                    `s \<notin> FV t' \<union> FV x \<union> FV y` 
                    `p \<notin> FV t' \<union> FV x \<union> FV y` `s \<noteq> p`
                    `C, env' \<turnstile> x : A` `C, env' \<turnstile> y : B` t2' this]
    prems(4)[OF _ _ `s \<noteq> p` `C, env' \<turnstile> x : A` `C, env' \<turnstile> y : B` 
                t1' `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>`] `l \<in> do T`]
  show ?case by simp
next
  case (T_Obj env f T F C env' t' s p x y A B n)
  from sopen_eq_Obj[OF sym[OF `Obj f T = {n \<rightarrow> [Fvar s,Fvar p]} t'`]]
  obtain f' where 
    obj: "Obj f T = {n \<rightarrow> [Fvar s,Fvar p]} Obj f' T" and 
    t': "t' = Obj f' T" by auto
  from 
    sym[OF this(1)] `dom f = do T`
    sym[OF dom_sopenoption_lem[of "Suc n" "Fvar s" "Fvar p" f']]
    dom_sopenoption_lem[of "Suc n" x y f']
  have dom: "dom (\<lambda>l. sopen_option (Suc n) x y (f' l)) = do T" by simp

  from `finite F` ok_finite[OF typing_regular'[OF `C, env' \<turnstile> x : A`]] 
  have finF: "finite (F \<union> {s,p} \<union> env_dom env')"
    by simp

  have
    "\<forall>l\<in>do T. \<forall>sa pa. sa \<notin> F \<union> {s,p} \<union> env_dom env' 
                     \<and> pa \<notin> F \<union> {s,p} \<union> env_dom env' 
                     \<and> sa \<noteq> pa 
      \<longrightarrow> C, env'\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>
          \<turnstile> (the(sopen_option (Suc n) x y (f' l))\<^bsup>[Fvar sa,Fvar pa]\<^esup>) : return(the(T^l))"
  proof 
    fix l assume "l \<in> do T" with prems(4)
    have 
      cof: 
      "\<forall>sa pa. sa \<notin> F \<and> pa \<notin> F \<and> sa \<noteq> pa
	\<longrightarrow> C, env\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr> 
            \<turnstile> (the(f l)\<^bsup>[Fvar sa,Fvar pa]\<^esup>) : return(the(T^l))
	  \<and> (\<forall>env'' t'' s' p' x' y' A' B' n'.
               s' \<notin> FV t'' \<union> FV x' \<union> FV y' \<longrightarrow> p' \<notin> FV t'' \<union> FV x' \<union> FV y' 
               \<longrightarrow> s' \<noteq> p' \<longrightarrow> C, env'' \<turnstile> x' : A' \<longrightarrow> C, env'' \<turnstile> y' : B'
               \<longrightarrow> (the(f l)\<^bsup>[Fvar sa,Fvar pa]\<^esup>) = {n' \<rightarrow> [Fvar s',Fvar p']} t''
               \<longrightarrow> env\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr> = env''\<lparr>s':A'\<rparr>\<lparr>p':B'\<rparr> 
               \<longrightarrow> C, env'' \<turnstile> {n' \<rightarrow> [x',y']} t'' : return(the(T^l)))"
      by simp
    from 
      `l \<in> do T` `dom f = do T` `Obj f T = {n \<rightarrow> [Fvar s,Fvar p]} t'` obj t'
      dom_sopenoption_lem[of "Suc n" "Fvar s" "Fvar p" f']
    have indomf': "l \<in> dom f'" by auto
    with obj sopen_option_lem[of f' "Suc n" "Fvar s" "Fvar p"] FV_option_lem[of f'] t'
    obtain 
      "the(f l) = {Suc n \<rightarrow> [Fvar s,Fvar p]} the(f' l)" and
      "FV (the(f' l)) \<subseteq> FV t'" by auto
    from 
      bnd_open_lem[OF cof `ok env` `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>` 
                      `s \<notin> FV t' \<union> FV x \<union> FV y` `p \<notin> FV t' \<union> FV x \<union> FV y`
                      `s \<noteq> p` `C, env' \<turnstile> x : A` `C, env' \<turnstile> y : B` this]
      indomf' sopen_option_lem[of f' "Suc n" x y] prems(4)
    show 
      "\<forall>sa pa. sa \<notin> F \<union> {s,p} \<union> env_dom env' 
             \<and> pa \<notin> F \<union> {s,p} \<union> env_dom env' \<and> sa \<noteq> pa 
        \<longrightarrow> C, env'\<lparr>sa:T\<rparr>\<lparr>pa:param(the(T^l))\<rparr>
            \<turnstile> (the(sopen_option (Suc n) x y (f' l))\<^bsup>[Fvar sa,Fvar pa]\<^esup>) : return(the(T^l))"
      by simp
  qed
  from typing.T_Obj[OF typing_regular'[OF `C, env' \<turnstile> x : A`] dom finF this] t' 
  show ?case by simp
next
  case (T_Call C env t1 T t2 l env' t' s p x y A B n)
  from sopen_eq_Call[OF sym[OF `Call t1 l t2 = {n \<rightarrow> [Fvar s,Fvar p]} t'`]]
  obtain t1' t2' where 
    t1: "t1 = {n \<rightarrow> [Fvar s,Fvar p]} t1'" and
    t2: "t2 = {n \<rightarrow> [Fvar s,Fvar p]} t2'" and
    t': "t' = Call t1' l t2'" by auto
  { fix a assume "a \<notin> FV t' \<union> FV x \<union> FV y" 
    with t' have "a \<notin> FV t1' \<union> FV x \<union> FV y" by simp thm prems
  }note 
      t1' = prems(2)[OF this[OF `s \<notin> FV t' \<union> FV x \<union> FV y`]
                        this[OF `p \<notin> FV t' \<union> FV x \<union> FV y`]
                        `s \<noteq> p` `C, env' \<turnstile> x : A` `C, env' \<turnstile> y : B`
                        t1 `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>`]
  { fix a assume "a \<notin> FV t' \<union> FV x \<union> FV y" 
    with t' have "a \<notin> FV t2' \<union> FV x \<union> FV y" by simp thm prems
  }
  from 
    typing.T_Call[OF t1' prems(4)[OF this[OF `s \<notin> FV t' \<union> FV x \<union> FV y`]
                                     this[OF `p \<notin> FV t' \<union> FV x \<union> FV y`]
                                     `s \<noteq> p` `C, env' \<turnstile> x : A` `C, env' \<turnstile> y : B`
                                     t2 `env = env'\<lparr>s:A\<rparr>\<lparr>p:B\<rparr>`] 
                     `l \<in> do T`]
    t'
  show ?case by simp
qed

lemma open_lemma: 
  "\<lbrakk> C, env\<lparr>s:A\<rparr>\<lparr>p:B\<rparr> \<turnstile> {n \<rightarrow> [Fvar s,Fvar p]} t : T; 
     s \<notin> FV t \<union> FV x \<union> FV y;  p \<notin> FV t \<union> FV x \<union> FV y; s \<noteq> p; 
     C, env \<turnstile> x : A; C, env \<turnstile> y : B \<rbrakk> 
  \<Longrightarrow> C, env \<turnstile> {n \<rightarrow> [x,y]} t : T"  
  by (simp add: open_lemma')

subsubsection {* Subject reduction *}
lemma type_dom[simp]: "C, env \<turnstile> (Obj a A) : A \<Longrightarrow> dom a = do A" 
  by (erule typing.cases, auto)

lemma select_preserve_type[simp]:
  assumes 
  "C, env \<turnstile> Obj f (Object t) : Object t" and "s \<notin> FV a" and "p \<notin> FV a" and
  "C, env\<lparr>s:(Object t)\<rparr>\<lparr>p:param(the(t l2))\<rparr> \<turnstile> (a\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the(t l2))" and
  "l1 \<in> dom t" and "l2 \<in> dom t"
  shows 
  "\<exists>F. finite F 
     \<and> (\<forall>s p. s \<notin> F \<and> p \<notin> F \<and> s \<noteq> p
         \<longrightarrow> C, env\<lparr>s:(Object t)\<rparr>\<lparr>p:param(the(t l1))\<rparr>
             \<turnstile> (the((f(l2 \<mapsto> a)) l1)\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the(t l1)))"
proof -
  from ok_finite[OF typing_regular'[OF `C, env \<turnstile> Obj f (Object t) : Object t`]]
  have finF: "finite ({s,p} \<union> env_dom env)" by simp

  { 
    note 
      ok_env = typing_regular'[OF `C, env \<turnstile> Obj f (Object t) : Object t`] and
      ok_env_sp = typing_regular'[OF prems(4)]
    fix sa pa assume 
      nin_sa: "sa \<notin> {s,p} \<union> env_dom env" and
      nin_pa: "pa \<notin> {s,p} \<union> env_dom env" and "sa \<noteq> pa"
    from this(1) ok_add_2[OF ok_env_sp] env_add_dom_2[OF ok_env]
    have "sa \<notin> env_dom (env\<lparr>s:Object t\<rparr>\<lparr>p:param(the(t l2))\<rparr>)" by simp
    from 
      nin_sa bigger_env_lemma[OF prems(4) this]
      subst_add[of sa p "env\<lparr>s:Object t\<rparr>" "Object t" "param(the(t l2))"]
      subst_add[of sa s env "Object t" "Object t"]
    have 
      aT_sa: "C, env\<lparr>sa:Object t\<rparr>\<lparr>s:Object t\<rparr>\<lparr>p:param(the(t l2))\<rparr>
              \<turnstile> (a\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the(t l2))" by simp
    from `sa \<noteq> pa` nin_sa nin_pa env_add_dom[OF ok_env] ok_add_2[OF ok_env_sp]
    obtain 
      "s \<notin> env_dom (env\<lparr>sa:Object t\<rparr>)" and
      "p \<notin> env_dom (env\<lparr>sa:Object t\<rparr>)" and "s \<noteq> p" and 
      "sa \<notin> env_dom env" and "pa \<notin> env_dom (env\<lparr>sa:Object t\<rparr>)" 
      by auto
    with env_add_dom_2[OF ok_add_ok[OF ok_env this(4)] this(1-3)] nin_pa
    have "pa \<notin> env_dom (env\<lparr>sa:Object t\<rparr>\<lparr>s:Object t\<rparr>\<lparr>p:param(the(t l2))\<rparr>)"
      by simp
    from 
      nin_pa bigger_env_lemma[OF aT_sa this]
      subst_add[of pa p "env\<lparr>sa:Object t\<rparr>\<lparr>s:Object t\<rparr>" 
                   "param(the(t l2))" "param(the(t l2))"]
      subst_add[of pa s "env\<lparr>sa:Object t\<rparr>" "param(the(t l2))" "Object t"]
    have 
      aT_sapa: 
      "C, env\<lparr>sa:Object t\<rparr>\<lparr>pa:param(the(t l2))\<rparr>\<lparr>s:Object t\<rparr>\<lparr>p:param(the(t l2))\<rparr>
      \<turnstile> {0 \<rightarrow> [Fvar s, Fvar p]} a : return(the(t l2))" by (simp add: openz_def)
    from nin_sa nin_pa `s \<notin> FV a` `p \<notin> FV a` ok_add_2[OF ok_env_sp] 
    obtain 
      ninFV_s: "s \<notin> FV a \<union> FV (Fvar sa) \<union> FV (Fvar pa)" and
      ninFV_p: "p \<notin> FV a \<union> FV (Fvar sa) \<union> FV (Fvar pa)" and "s \<noteq> p"
      by auto
    from ok_add_2[OF typing_regular'[OF aT_sapa]]
    have ok_env_sapa: "ok (env\<lparr>sa:Object t\<rparr>\<lparr>pa:param(the(t l2))\<rparr>)"
      by simp
    with ok_add_reverse[OF this]
    have ok_env_pasa: "ok (env\<lparr>pa:param(the(t l2))\<rparr>\<lparr>sa:Object t\<rparr>)"
      by simp

    from 
      open_lemma[OF aT_sapa ninFV_s ninFV_p `s \<noteq> p` _
                    T_Var[OF ok_env_sapa in_add[OF ok_env_sapa] 
                             add_get2_2[OF ok_env_sapa]]]
      T_Var[OF ok_env_pasa in_add[OF ok_env_pasa] add_get2_2[OF ok_env_pasa]]
      ok_add_reverse[OF ok_env_sapa]
    have 
      "C, env\<lparr>sa:(Object t)\<rparr>\<lparr>pa:param(the(t l2))\<rparr> 
       \<turnstile> (a\<^bsup>[Fvar sa,Fvar pa]\<^esup>) : return(the(t l2))"
      by (simp add: openz_def)
  }note alem = this

(* case split *)
  show ?thesis
  proof (cases "l1 = l2")
    case True with assms obj_inv_elim'[OF assms(1)] show ?thesis
      by (simp (no_asm_simp), rule_tac x = "{s,p} \<union> env_dom env" in exI,
          auto simp: finF alem)
  next
    case False
    from obj_inv_elim[OF `C, env \<turnstile> Obj f (Object t) : Object t`]
    obtain F where 
      "finite F" and
      "\<forall>l\<in>dom t. 
        \<forall>s p. s \<notin> F \<and> p \<notin> F \<and> s \<noteq> p
         \<longrightarrow> C, env\<lparr>s:Object t\<rparr>\<lparr>p:param(the(Object t^l))\<rparr>
             \<turnstile> (the(f l)\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the(Object t^l))"
      by auto
    from this(2) `l1 \<in> dom t`
    have 
      "\<forall>s p. s \<notin> F \<and> p \<notin> F \<and> s \<noteq> p
        \<longrightarrow> C, env\<lparr>s:Object t\<rparr>\<lparr>p:param(the(Object t^l1))\<rparr>
            \<turnstile> (the(f l1)\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the(Object t^l1))"
      by auto
    thus ?thesis using `finite F` `l1 \<noteq> l2` by (simp,blast)
  qed
qed

lemma beta_lc': "t \<rightarrow>\<^sub>\<beta> t' \<Longrightarrow> lc t"
  by (simp add: beta_lc)

lemma beta_lc'': "t \<rightarrow>\<^sub>\<beta> t' \<Longrightarrow> lc t'"
  by (simp add: beta_lc)

text {* Main Lemma *}
lemma subject_reduction: "C, e \<turnstile> t : T \<Longrightarrow> (\<And>t'. t \<rightarrow>\<^sub>\<beta> t' \<Longrightarrow> C, e \<turnstile> t' : T)"
proof (induct set: typing)
  case (T_Var env x T t') from Fvar_beta[OF `Fvar x \<rightarrow>\<^sub>\<beta> t'`] show ?case by simp 
next
  case T_Active thus ?case by auto
next
  case T_ActRef thus ?case by (auto elim: beta.cases)
next
  case T_FutRef thus ?case by (auto elim: beta.cases)
next
  case (T_Upd F C e T l t2 t1 t') 
  note pred_u = this(2) and pred_t = this(4)
  from Upd_beta[OF `Upd t1 l t2 \<rightarrow>\<^sub>\<beta> t'`] show ?case
  proof (elim disjE exE conjE)
    fix t1' assume "t1 \<rightarrow>\<^sub>\<beta> t1'" and "t' = Upd t1' l t2"
    from 
      this(2) pred_u  
      typing.T_Upd[OF `finite F` _ pred_t[OF this(1)] `l \<in> do T`]
    show ?case by simp
  next
    fix t2' F' 
    assume 
      "finite F'" and
      pred_F': "\<forall>s p. s \<notin> F' \<and> p \<notin> F' \<and> s \<noteq> p 
                 \<longrightarrow> (\<exists>t''. t2\<^bsup>[Fvar s,Fvar p]\<^esup> \<rightarrow>\<^sub>\<beta> t'' \<and> t2' = \<sigma>[s,p] t'')"  and
      "t' = Upd t1 l t2'"
    from `finite F` this(1) this(3) `C, e \<turnstile> t1 : T` `l \<in> do T` 
    show ?case 
    proof (simp, intro typing.T_Upd[of "F \<union> F'"], auto)
      fix s p 
      assume "s \<notin> F" and "p \<notin> F" and "s \<noteq> p" and "s \<notin> F'" and "p \<notin> F'"
      with pred_F' obtain t'' where "t2\<^bsup>[Fvar s,Fvar p]\<^esup> \<rightarrow>\<^sub>\<beta> t''" and "t2' = \<sigma>[s,p] t''"
	by auto
      with beta_lc[OF this(1)] sopen_sclose_eq_t[of t'' 0 s p]
      have "t2\<^bsup>[Fvar s,Fvar p]\<^esup> \<rightarrow>\<^sub>\<beta> (t2'\<^bsup>[Fvar s,Fvar p]\<^esup>)" 
	by (simp add: openz_def closez_def)
      with `s \<notin> F` `p \<notin> F` `s \<noteq> p` pred_u
      show "C, e\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr> \<turnstile> (t2'\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the(T^l))"
	by auto
    qed
  next
    fix f U 
    def pred_bnd \<equiv> "\<lambda>s p b l. C, e\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr>
                              \<turnstile> the b\<^bsup>[Fvar s,Fvar p]\<^esup> : return(the(T^l))"
    def pred_cof \<equiv> "\<lambda>b l L. \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p \<longrightarrow> pred_bnd s p b l"

    assume "l \<in> dom f" and "Obj f U = t1" and "t' = Obj (f(l \<mapsto> t2)) U"
    from this(1-2) `C, e \<turnstile> t1 : T` obj_inv[of C e f U T]
    obtain t where 
      "C, e \<turnstile> Obj f (Object t) : (Object t)" and 
      "Object t = T" and "T = U" 
      by (cases T, auto)
    hence "C, e \<turnstile> Obj f T : T" by auto
    from obj_inv_elim[OF this]
    obtain F' where "finite F'" and "\<forall>l\<in>do T. pred_cof (f l) l F'"
      unfolding pred_cof_def pred_bnd_def
      by auto
    moreover
    from obj_inv_elim[OF `C, e \<turnstile> Obj f T : T`] `l \<in> dom f` 
    have "l \<in> dom (f(l \<mapsto> t2))" and "dom (f(l \<mapsto> t2)) = do T"      
      by auto
    ultimately show ?case
      using
	`t' = Obj (f(l \<mapsto> t2)) U` `finite F` `finite F'`sym[OF `T = U`]
	typing_regular'[OF `C, e \<turnstile> t1 : T`] pred_u
      unfolding pred_cof_def pred_bnd_def
      by (simp (no_asm_simp), intro typing.T_Obj[of _ _ _ "F' \<union> F \<union> FV t2"], auto)
  qed
next
  case (T_Obj e f T F C t') note pred = this(4)
  from Obj_beta[OF `Obj f T \<rightarrow>\<^sub>\<beta> t'`] show ?case
  proof (elim exE conjE)
    fix l f' a a' F' assume 
      "f = f'(l \<mapsto> a)" and "l \<in> dom f'" and
      "t' = Obj (f'(l \<mapsto> a')) T" and "finite F'" and
      red_sp: "\<forall>s p. s \<notin> F' \<and> p \<notin> F' \<and> s \<noteq> p 
                \<longrightarrow> (\<exists>t''. a\<^bsup>[Fvar s, Fvar p]\<^esup> \<rightarrow>\<^sub>\<beta> t'' \<and> a' = \<sigma>[s,p] t'')"
    from this(1-5) `ok e` `dom f = do T` `finite F` show ?case
    proof (simp, intro typing.T_Obj[of _ _ _ "F \<union> F'"], 
	simp, simp, simp, intro strip, elim conjE)
      fix l' s p assume
	"l' \<in> do T" and 
	nin_s: "s \<notin> F \<union> F'" and
	nin_p: "p \<notin> F \<union> F'" and "s \<noteq> p"
      with red_sp obtain t'' where "a\<^bsup>[Fvar s,Fvar p]\<^esup> \<rightarrow>\<^sub>\<beta> t''" and "a' = \<sigma>[s,p] t''" 
	by auto
      with 
	beta_lc[OF this(1)] sopen_sclose_eq_t[of t'' 0 s p] 
	`f = f'(l \<mapsto> a)`
      have "the (f l)\<^bsup>[Fvar s,Fvar p]\<^esup> \<rightarrow>\<^sub>\<beta> (the((f'(l \<mapsto> a')) l)\<^bsup>[Fvar s,Fvar p]\<^esup>)" 
	by (simp add: openz_def closez_def)
      with prems(4) nin_s nin_p `s \<noteq> p` `l' \<in> do T` `f = f'(l \<mapsto> a)`
      show 
	"C, e\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l'))\<rparr>
	 \<turnstile> (the((f'(l \<mapsto> a')) l')\<^bsup>[Fvar s,Fvar p]\<^esup>) : return(the(T^l'))"
	by auto
    qed
  qed
next
  case (T_Call C e t1 T t2 l t') 
  note pred_t1 = this(2) and pred_t2 = this(4)
  from Call_beta[OF `Call t1 l t2 \<rightarrow>\<^sub>\<beta> t'`] show ?case
  proof (elim disjE conjE exE)
    fix t1' assume "t1 \<rightarrow>\<^sub>\<beta> t1'" and "t' = Call t1' l t2"
    with typing.T_Call[OF pred_t1[OF this(1)] `C, e \<turnstile> t2 : param(the(T^l))` `l \<in> do T`]
    show ?case by simp
  next
    fix t2' assume "t2 \<rightarrow>\<^sub>\<beta> t2'" and "t' = Call t1 l t2'"
    with typing.T_Call[OF `C, e \<turnstile> t1 : T` pred_t2[OF this(1)] `l \<in> do T`]
    show ?case by simp
  next
    fix f U 
    def pred_bnd \<equiv> "\<lambda>s p b l. C, e\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr>
                              \<turnstile> the b\<^bsup>[Fvar s,Fvar p]\<^esup> : return(the(T^l))"
    def pred_cof \<equiv> "\<lambda>b l L. \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p \<longrightarrow> pred_bnd s p b l"

    assume "l \<in> dom f" and "Obj f U = t1" and "t' = (the(f l)\<^bsup>[Obj f U,t2]\<^esup>)"
    from this(1-2) `C, e \<turnstile> t1 : T` obj_inv[of C e f U T]
    obtain t where 
      "C, e \<turnstile> Obj f (Object t) : (Object t)" and 
      "Object t = T" and "T = U" 
      by (cases T, auto)
    hence "C, e \<turnstile> Obj f T : T" by auto
    from obj_inv_elim[OF this]
    obtain F where "dom f = do T" and "finite F" and "\<forall>l\<in>do T. pred_cof (f l) l F"
      unfolding pred_cof_def pred_bnd_def
      by auto
    moreover
    from `finite F` finite_FV[of "Obj f T"]
    have "finite (F \<union> FV (Obj f T) \<union> FV t2)" by simp
    from exFresh_s_p_cof[OF this]
    obtain s p where 
      "s \<notin> F \<union> FV (Obj f T) \<union> FV t2" and
      "p \<notin> F \<union> FV (Obj f T) \<union> FV t2" and "s \<noteq> p" 
      by auto
    ultimately
    have 
      "pred_bnd s p (f l) l" and 
      nin_s: "s \<notin> FV (the(f l)) \<union> FV (Obj f T) \<union> FV t2" and 
      nin_p: "p \<notin> FV (the(f l)) \<union> FV (Obj f T) \<union> FV t2"
      using `l \<in> dom f` `dom f = do T`
      unfolding pred_cof_def by (auto simp: FV_option_lem)
    from this(1) `t' = (the(f l)\<^bsup>[Obj f U,t2]\<^esup>)` sym[OF `T = U`]
    show ?case
      by (simp, unfold pred_bnd_def openz_def,
	intro open_lemma[OF _ nin_s nin_p `s \<noteq> p` 
	                   `C, e \<turnstile> Obj f T : T` `C, e \<turnstile> t2 : param(the(T^l))`],
	simp)
  qed
qed

theorem subject_reduction': "t \<rightarrow>\<^sub>\<beta>\<^sup>* t' \<Longrightarrow> C, e \<turnstile> t : T \<Longrightarrow> C, e \<turnstile> t' : T"
  by (induct set: rtranclp) (iprover intro: subject_reduction)+

lemma Union_eq: "A = B \<Longrightarrow> \<Union>A = \<Union>B"
  by auto

lemma Insert_eq: "A = B \<Longrightarrow> insert a A = insert a B"
  by auto

lemma type_members_equal: 
  fixes A :: type and B :: type
  assumes "do A = do B" and "\<forall>i. (A^i) = (B^i)"
  shows "A = B"
proof (cases A)
  case (Object ta) thus ?thesis
  proof (cases B)
    case (Object tb)
    from `\<forall>i. (A^i) = (B^i)` `A = Object ta` `B = Object tb` 
    have "\<And>i. ta i = tb i" by auto
    with `A = Object ta` `B = Object tb` show ?thesis by (simp add: ext)
  qed
qed

lemma not_var: "C, Env empty \<turnstile> a : A \<Longrightarrow> \<forall>x. a \<noteq> Fvar x"
  by (rule allI, case_tac x, auto)

lemma Call_label_range: "C, (Env empty) \<turnstile> Call (Obj c T) l b : A \<Longrightarrow> l \<in> dom c"
  by (erule typing_elims, erule typing.cases, simp_all)

lemma  Call_subterm_type: "C, Env empty \<turnstile> Call t l b: T 
  \<Longrightarrow> (\<exists>T'. C, Env empty \<turnstile> t : T') \<and> (\<exists>T'. C, Env empty \<turnstile> b : T')"
  by (erule typing.cases) auto

lemma Upd_label_range: "C, Env empty \<turnstile> Upd (Obj c T) l x : A \<Longrightarrow> l \<in> dom c"
  by (erule typing_elims, erule typing.cases, simp_all)

lemma Upd_subterm_type: 
  "C, Env empty \<turnstile> Upd t l x : T \<Longrightarrow> \<exists>T'. C, Env empty \<turnstile> t : T'" 
  by (erule typing.cases) auto

lemma no_fvar: "C, Env empty \<turnstile> Fvar x : T \<Longrightarrow> False"
  by (case_tac x, auto)

lemma no_bvar: "C, e \<turnstile> Bvar b : T \<Longrightarrow> False" 
  by (erule typing.cases, auto)

lemma typing_FV_env_dom:
  assumes "C, env \<turnstile> t : T" 
  shows "FV t \<subseteq> env_dom env"
  using assms
proof (induct 
    taking: "\<lambda>C e t T l. FV t \<subseteq> env_dom e"
    rule: typing_induct)
  case Fvar thus ?case by auto
next
  case Call thus ?case by auto
next
  case Active thus ?case by auto
next
  case ActRef thus ?case by auto
next
  case FutRef thus ?case by auto
next
  case Upd thus ?case by auto
next
  case Obj thus ?case by (auto simp: FV_option_lem)
next
  case (Bnd C e T l t L) note pred = this(3)
  from 
    `finite L` ok_finite[OF `ok e`]
    exFresh_s_p_cof[of "L \<union> FV t \<union> env_dom e"]
  obtain s p where 
    "s \<notin> L" and "p \<notin> L" and "s \<noteq> p" and
    "s \<notin> FV t" and "p \<notin> FV t" and
    "s \<notin> env_dom e" and "p \<notin> env_dom e"
    by auto
  from this(1-3) pred
  have "FV (t\<^bsup>[Fvar s, Fvar p]\<^esup>) \<subseteq> env_dom (e\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr>)"
    by blast
  note Diff_mono[OF this equalityD1[of "{s,p}" "{s,p}"]]
  moreover
  from 
    sclose_sopen_eq_t[OF `s \<notin> FV t` `p \<notin> FV t` `s \<noteq> p`]
    sclose_FV[of 0 s p "t\<^bsup>[Fvar s, Fvar p]\<^esup>"]
  have "FV t = FV (t\<^bsup>[Fvar s, Fvar p]\<^esup>) - {s, p}"
    by (auto simp: openz_def)
  moreover
  from 
    `s \<notin> env_dom e` `p \<notin> env_dom e`
  have "env_dom e = env_dom e \<union> {s,p} - {s,p}" by auto
  with
    env_add_dom_2[OF `ok e` `s \<notin> env_dom e` `p \<notin> env_dom e` `s \<noteq> p`]
  have "env_dom e = env_dom (e\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr>) - {s,p}"
    by auto
  ultimately
  show "FV t \<subseteq> env_dom e" by auto
qed

(*
lemma smaller_env[rule_format]:
  assumes "ok (Env env)"
  shows "\<forall>x\<in>dom env. \<exists>env'. x \<notin> dom env' \<and> Env env = (Env env')\<lparr>x:the(env x)\<rparr>"
proof
  fix x assume "x \<in> dom env"
  from one_more_dom[OF this] obtain env' where
    "env = env'(x \<mapsto> the(env x))" and "x \<notin> dom env'"
    by auto
  thus "\<exists>env'. x \<notin> dom env' \<and> Env env = (Env env')\<lparr>x:the(env x)\<rparr>"
    unfolding add_def
    by auto
qed
*)

lemma typing_closed: 
  assumes "C, Env empty \<turnstile> t : T"
  shows "closed t"
proof -
  from typing_FV_env_dom[OF assms]
  have "FV t = {}" by auto
  thus ?thesis by (simp add: closed_def)
qed

lemma typing_env_smaller'[rule_format]:
  assumes "C, e \<turnstile> t : T"
  shows "\<forall>e' x X. e = e'\<lparr>x:X\<rparr> \<longrightarrow> x \<notin> env_dom e' \<longrightarrow> x \<notin> FV t \<longrightarrow> C, e' \<turnstile> t : T"
proof -
  def pred_cof \<equiv> "\<lambda>L C e t T l. \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p 
                          \<longrightarrow> C, e\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> \<turnstile> t\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
  from assms show ?thesis
  proof (induct
      taking: "\<lambda>C e t T l. \<forall>e' x X. e = e'\<lparr>x:X\<rparr> \<longrightarrow> x \<notin> env_dom e' \<longrightarrow> x \<notin> FV t 
                            \<longrightarrow> (\<exists>L. finite L \<and> pred_cof L C e' t T l)"
      rule: typing_induct)
    case Call thus ?case by force
  next
    case Active thus ?case by (auto simp: not_malformed_smaller)
  next
    case ActRef thus ?case by (auto simp: not_malformed_smaller)
  next
    case FutRef thus ?case by (auto simp: not_malformed_smaller)
  next
    case (Fvar C e T x)
    show ?case
    proof (intro strip)
      fix e' y Y assume "e = e'\<lparr>y:Y\<rparr>" and "y \<notin> env_dom e'" and "y \<notin> FV (Fvar x)"
      from this(1) this(3) in_env_smaller[of x e' y Y] `ok e` `x \<in> env_dom e`
      have "x \<in> env_dom e'" by simp
      moreover
      from `e = e'\<lparr>y:Y\<rparr>` `ok e` have "ok e'" 
	by (simp add: not_malformed_smaller[of e' y Y])
      moreover
      from 
	get_env_bigger[of x e' y Y] `e = e'\<lparr>y:Y\<rparr>` `x \<in> env_dom e` `y \<notin> FV (Fvar x)`
	`the e!x = T`
      have "the e'!x = T" by simp
      ultimately
      show "C, e' \<turnstile> Fvar x : T" by auto
    qed
  next
    case (Upd C e T t l u)
    note pred_t = this(2) and pred_u = this(4)
    show ?case
    proof (intro strip)
      fix e' x X assume "x \<notin> FV (Upd t l u)"
      hence "x \<notin> FV t" and "x \<notin> FV u"
	by auto
      moreover
      assume "e = e'\<lparr>x:X\<rparr>" and "x \<notin> env_dom e'"
      ultimately
      have "C, e' \<turnstile> t : T" and "\<exists>L. finite L \<and> pred_cof L C e' u T l"
	using pred_u pred_t 
	by blast+
      then obtain L where "finite L" and "pred_cof L C e' u T l"
	by auto
      with `C, e' \<turnstile> t : T` `l \<in> do T` show "C, e' \<turnstile> Upd t l u : T"
	unfolding pred_cof_def 
	by auto
    qed
  next
    case (Obj C e T f) note pred_o = this(3)
    def pred_cof' \<equiv> "\<lambda>e' x (X::type) b l. x \<notin> FV (the b) 
                     \<longrightarrow> (\<exists>L. finite L \<and> pred_cof L C e' (the b) T l)"
    def pred_pre \<equiv> "\<lambda>e' x X. e = e'\<lparr>x:X\<rparr> \<and> x \<notin> env_dom e'"
    from pred_o
    have pred: "\<forall>e' x X. pred_pre e' x X \<longrightarrow> (\<forall>l\<in>dom f. pred_cof' e' x X (f l) l)"
      by (intro fmap_ball_all_3'[of f pred_pre pred_cof'],
	unfold pred_pre_def pred_cof'_def, blast)
    show ?case proof (intro strip)
      fix e' x X
      def pred_bnd \<equiv> "\<lambda>s p b l. C, e'\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> 
	                        \<turnstile> the b\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
      assume "x \<notin> FV (Obj f T)" 
      hence "\<forall>l\<in>dom f. x \<notin> FV (the(f l))" by (auto simp: FV_option_lem)
      moreover assume "e = e'\<lparr>x:X\<rparr>" and "x \<notin> env_dom e'"
      ultimately
      have "\<forall>l\<in>dom f. (\<exists>L. finite L \<and> pred_cof L C e' (the (f l)) T l)"
	using pred
	unfolding pred_pre_def pred_cof'_def
	by blast
      with fmap_ex_cof[of f pred_bnd] `dom f = do T`
      obtain L where "finite L" and "\<forall>l\<in>do T. pred_cof L C e' (the(f l)) T l"
	unfolding pred_bnd_def pred_cof_def
	by auto
      from this(2) `e = e'\<lparr>x:X\<rparr>` `ok e`
      show "C, e' \<turnstile> Obj f T : T"
	unfolding pred_cof_def
	by (intro T_Obj[OF not_malformed_smaller[of e' x X] `dom f = do T` `finite L`],
	  simp+)
    qed
  next
    case (Bnd C e T l t L) note pred = this(3)
    thus ?case
    proof (intro strip)
      fix e' x X
      assume "e = e'\<lparr>x:X\<rparr>" and "x \<notin> env_dom e'" and "x \<notin> FV t"
      show "\<exists>L. finite L \<and> pred_cof L C e' t T l"
	unfolding pred_cof_def
      proof (rule_tac x = "L \<union> {x}" in exI, auto simp: `finite L`)
	fix s p assume 
	  "s \<notin> L" and "p \<notin> L" and "s \<noteq> p" and "s \<noteq> x" and "p \<noteq> x"
	from this(4-5) `x \<notin> FV t` sopen_FV[of 0 "Fvar s" "Fvar p" t]
	have "x \<notin> FV (t\<^bsup>[Fvar s, Fvar p]\<^esup>)" by (auto simp: openz_def)
	moreover
	from 
	  not_sym[OF `s \<noteq> x`] subst_add[of x s e' X T]
	  not_sym[OF `p \<noteq> x`] subst_add[of x p "e'\<lparr>s:T\<rparr>" X "param (the(T^l))"]
	  `e = e'\<lparr>x:X\<rparr>`
	have "e\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr> = e'\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr>\<lparr>x:X\<rparr>"
	  by simp
	moreover
	from 
	  `x \<notin> env_dom e'` `s \<noteq> x` `p \<noteq> x` 
	  not_in_env_bigger_2[of x e' s p]
	have "x \<notin> env_dom (e'\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr>)"
	  by auto
	ultimately
	show "C, e'\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr> \<turnstile> t\<^bsup>[Fvar s,Fvar p]\<^esup> : return (the(T^l))"
	  using `s \<notin> L` `p \<notin> L` `s \<noteq> p` pred
	  by blast
      qed
    qed
  qed
qed

lemma typing_restricted[rule_format]:
  assumes "C, env \<turnstile> t : T"
  shows "\<forall>e A. Env e = env \<longrightarrow> finite A \<longrightarrow> FV t \<subseteq> A \<longrightarrow> C, Env (e|`A) \<turnstile> t : T"
proof -
  def pred_cof \<equiv> "\<lambda>L C e t T l. \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p 
                   \<longrightarrow> C, e\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> \<turnstile> t\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
  from assms show ?thesis
  proof (induct 
      taking: "\<lambda>C env t T l. \<forall>e A. Env e = env \<longrightarrow> finite A \<longrightarrow> FV t \<subseteq> A 
                \<longrightarrow> (\<exists>L. finite L \<and> pred_cof L C (Env (e|`A)) t T l)"
      rule: typing_induct)
    case Fvar thus ?case by auto
  next
    case Call thus ?case by auto
  next
    case Active thus ?case by auto
  next
    case ActRef thus ?case by auto
  next
    case FutRef thus ?case by auto
  next
    case (Upd C env T t l u) 
    note pred_t = this(2) and pred_u = this(4)
    show ?case
    proof (intro strip)
      fix e A assume "Env e = env" and "finite A" and "FV (Upd t l u) \<subseteq> A"
      hence "FV t \<subseteq> A" and "FV u \<subseteq> A" by auto
      from this(1) `Env e = env` `finite A` pred_t 
      have "C, Env (e|`A) \<turnstile> t : T" by auto
      moreover
      from `Env e = env` `finite A` `FV u \<subseteq> A` pred_u
      obtain L where
	"finite L" and "pred_cof L C (Env (e|`A)) u T l"
	by blast
      ultimately
      show "C, Env (e|`A) \<turnstile> Upd t l u : T"
	using T_Upd[of L C "Env (e|`A)" T l u t] `l \<in> do T`
	unfolding pred_cof_def
	by auto
    qed
  next
    case (Obj C env T f) note pred_o = this(3)
    def pred_pre \<equiv> "\<lambda>e A :: char list \<Rightarrow> bool. Env e = env \<and> finite A"
    def pred_cof' \<equiv> "\<lambda>e A b l. FV (the b) \<subseteq> A 
                      \<longrightarrow> (\<exists>L. finite L \<and> pred_cof L C (Env(e|`A)) (the b) T l)"
    from pred_o
    have pred: "\<forall>e A. pred_pre e A \<longrightarrow> (\<forall>l\<in>dom f. pred_cof' e A (f l) l)"
      by (intro fmap_ball_all_2'[of f pred_pre pred_cof'],
	unfold pred_pre_def pred_cof'_def, simp)
    show ?case
    proof (intro strip)
      fix e A 
      def pred_bnd \<equiv> "\<lambda>s p b l. C, (Env (e|`A))\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> 
	                        \<turnstile> the b\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
      assume "FV (Obj f T) \<subseteq> A" 
      hence "\<forall>l\<in>dom f. FV (the(f l)) \<subseteq> A" by (auto simp: FV_option_lem)
      moreover assume "Env e = env" and "finite A"
      ultimately
      have "\<forall>l\<in>dom f. (\<exists>L. finite L \<and> pred_cof L C (Env (e|`A)) (the (f l)) T l)"
	using pred
	unfolding pred_pre_def pred_cof'_def
	by blast
      with fmap_ex_cof[of f pred_bnd] `dom f = do T` 
      obtain L where "finite L" and "\<forall>l\<in>do T. pred_cof L C (Env (e|`A)) (the(f l)) T l"
	unfolding pred_bnd_def pred_cof_def
	by auto
      moreover
      from not_malformed_restricted[of e A] `ok env` `Env e = env` `finite A`
      have "ok (Env (e|`A))" by simp
      note T_Obj[OF this `dom f = do T` `finite L`]
      ultimately
      show "C, (Env (e|`A)) \<turnstile> Obj f T : T"
	unfolding pred_cof_def
	by auto
    qed
  next
    case (Bnd C env T l t L) note pred = this(3)
    show ?case
    proof (intro strip)
      fix e A assume "Env e = env" and "finite A" and "FV t \<subseteq> A"
      with `ok env` `finite L` 
      show "\<exists>L. finite L \<and> pred_cof L C (Env (e|`A)) t T l"
      proof (rule_tac x = "L \<union> A \<union> env_dom env" in exI, simp, 
	  unfold pred_cof_def, intro strip, elim conjE)
	fix s p
	from `FV t \<subseteq> A` `finite A` sopen_FV[of 0 "Fvar s" "Fvar p" t]
	have "finite (A \<union> {s,p})" and "FV (t\<^bsup>[Fvar s, Fvar p]\<^esup>) \<subseteq> A \<union> {s,p}"
	  by (auto simp: openz_def)
	moreover
	assume 
	  "s \<notin> L \<union> A \<union> env_dom env" and 
	  "p \<notin> L \<union> A \<union> env_dom env" and "s \<noteq> p"
	hence 
	  "s \<notin> L" and "p \<notin> L" and "s \<notin> A" and "p \<notin> A" and 
	  "s \<notin> env_dom env" and "p \<notin> env_dom env"
	  by auto
	with `s \<noteq> p` `Env e = env`
	obtain e' where eq_e': "Env e' = env\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr>"
	  by auto
	ultimately
	have 
	  typing_e': "C, Env e' \<turnstile> t\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))" and
	  typing_e'sp: "C, Env (e'|`(A \<union> {s,p})) \<turnstile> t\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
	  using pred `s \<notin> L` `p \<notin> L` `s \<noteq> p`
	  by auto

	from 
	  eq_e' `Env e = env` `s \<notin> env_dom env` `p \<notin> env_dom env` 
	  `s \<noteq> p` `s \<notin> A` `p \<notin> A`
	have "Env (e' |` A) = Env (e |` A)" by auto
	moreover
	{
	  from 	    
	    typing_regular'[OF typing_e'] eq_e'
	    in_add_2[of env s T p "param(the(T^l))"]
	  have "s \<in> env_dom (Env e')" and "p \<in> env_dom (Env e')"
	    by auto
	  hence "s \<in> dom e'" and "p \<in> dom e'"
	    unfolding env_dom_def
	    by simp+
	  moreover
	  from
	    typing_regular'[OF typing_e'] eq_e'
	    add_get2_1[of env s T p "param(the(T^l))"]
	    add_get2_2[of env s T p "param(the(T^l))"]
	  have "the (Env e'!s) = T" and "the (Env e'!p) = param(the(T^l))"
	    by auto
	  ultimately
	  have "e' s = Some T" and "e' p = Some (param(the(T^l)))"
	    unfolding env_get_def
	    by auto
	}
	{
	  fix x
	  from `e' s = Some T` `e' p = Some (param(the(T^l)))`
	  have "(e' |` {s,p}) x = [s \<mapsto> T, p \<mapsto> param(the(T^l))] x"
	    unfolding restrict_map_def
	    by (auto split: split_if_asm)
	}
	with `s \<noteq> p`
	have "Env (e' |` {s,p}) = Env empty\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr>"
	  unfolding restrict_map_def
	  by (auto simp: ext)
	moreover
	{
	  from `s \<notin> A` `p \<notin> A` have "A \<inter> {s,p} = {}" by simp
	  with restrict_disj_union[of A "{s,p}" e']
	  have "e'|`(A \<union> {s,p}) = (e'|`A) ++ (e'|`{s,p})"
	    by simp
	  moreover
	  from 
	    not_malformed_restricted[OF typing_regular'[OF typing_e']]
	    `finite A`
	  have "ok (Env (e'|`A))" and "ok (Env (e'|`{s,p}))"
	    by auto
	  ultimately
	  have "Env (e'|`(A \<union> {s,p})) = Env (e'|`A) + Env (e'|`{s,p})"
	    using `A \<inter> {s,p} = {}`
	    by auto
	}
	ultimately
	have "Env (e'|`(A \<union> {s,p})) = (Env (e|`A))\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr>"
	  using env_app_add2[of "Env (e|`A)" "Env empty" s p]
	  by auto
	with typing_e'sp
	show "C, Env (e|`A)\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr> \<turnstile> t\<^bsup>[Fvar s, Fvar p]\<^esup> : return(the(T^l))"
	  by simp
      qed
    qed
  qed
qed

lemma typing_FV_smaller:
  assumes "C, Env env \<turnstile> t : T"
  shows "\<exists>env'. dom env' = FV t \<and> C, Env env' \<turnstile> t : T"
proof (rule_tac x = "(env|`(FV t))" in exI, 
    auto simp: typing_restricted[OF assms _ finite_FV[of t]])
  fix x assume "x \<in> FV t" 
  with typing_FV_env_dom[OF assms] show "\<exists>y. env x = Some y"
    unfolding env_dom_def
    by auto
qed

lemma closed_typing:
  assumes "C, e \<turnstile> t : T" and "closed t"
  shows "C, Env empty \<turnstile> t : T"
proof -
  from ok_ok[OF typing_regular'[OF `C, e \<turnstile> t : T`]] `C, e \<turnstile> t : T`
  obtain env where "C, Env env \<turnstile> t : T" by auto
  from typing_FV_smaller[OF this] `closed t`
  obtain env' where "dom env' = {}" and "C, Env env' \<turnstile> t : T"
    unfolding closed_def
    by auto
  from this(2) empty_dom[OF sym[OF this(1)]]
  show ?thesis by simp
qed

lemma subterm_typing_pre[rule_format]:
  assumes "C, env' \<turnstile> t : T"
  shows 
  "\<forall>(E::general_context) a. t = E\<up>a \<longrightarrow> lc a \<longrightarrow> (\<exists>e T. C, e \<turnstile> a : T)"
  using assms
proof (induct rule: typing.induct)
  case (T_Var e x T C) 
  show ?case
  proof (intro strip)
    fix E::general_context and a assume "Fvar x = E\<up>a"
    from context_fvar[OF sym[OF this]] `ok e` `x \<in> env_dom e` `the e!x = T`
    show "\<exists>e T. C, e \<turnstile> a : T" by auto
  qed
next
  case (T_Call C e t T u l) 
  note pred_t = this(2) and pred_u = this(4)
  show ?case
  proof (intro strip)
    fix E::general_context and a assume "Call t l u = E\<up>a" and "lc a"
    from context_call[OF sym[OF this(1)]]
    show "\<exists>e T. C, e \<turnstile> a : T"
    proof (elim disjE)
      assume "E = cHole" 
      with
	`Call t l u = E\<up>a`
	`C, e \<turnstile> t : T` `C, e \<turnstile> u : param(the(T^l))` `l \<in> do T`
      show ?thesis by auto
    next
      assume "\<exists>(E'::general_context). t = E'\<up>a"
      then obtain E'::general_context where "t = E'\<up>a"
	by auto
      with `C, e \<turnstile> t : T` `lc a` pred_t
      show ?thesis by blast
    next
      assume "\<exists>(E'::general_context). u = E'\<up>a"
      then obtain E'::general_context where "u = E'\<up>a"
	by auto
      with `C, e \<turnstile> u : param(the(T^l))` `lc a` pred_u
      show ?thesis by blast
    qed
  qed
next
  case (T_Active e C t T) note pred = this(3)
  show ?case
  proof (intro strip)
    fix E::general_context and a assume "Active t = E\<up>a" and "lc a"
    from context_active[OF sym[OF this(1)]]
    show "\<exists>e T. C, e \<turnstile> a : T"
    proof (elim disjE)
      assume "E = cHole" with `Active t = E\<up>a` `C, e \<turnstile> t : T` `ok e`
      show ?thesis by auto
    next
      assume "\<exists>(E'::general_context). t = E'\<up>a"
      then obtain E'::general_context where "t = E'\<up>a"
	by auto
      with `C, e \<turnstile> t : T` `lc a` pred
      show ?thesis by blast
    qed
  qed
next
  case (T_ActRef e am G T fm)
  show ?case
  proof (intro strip)
    fix E::general_context and a assume "ActRef G = E\<up>a"
    with context_actrefs[OF this] `ok e` `am G = Some T` `G \<in> dom am`
    show "\<exists>e T. TConfig am fm, e \<turnstile> a : T" by blast
  qed
next
  case (T_FutRef e fm fi T am)
  show ?case
  proof (intro strip)
    fix E::general_context and a assume "FutRef fi = E\<up>a"
    with context_futrefs[OF this] `ok e` `fm fi = Some T` `fi \<in> dom fm`
    show "\<exists>e T. TConfig am fm, e \<turnstile> a : T" by blast
  qed
next
  case (T_Upd L C e T l u t) 
  note pred_u = this(2) and pred_t = this(4)
  show ?case
  proof (intro strip)
    fix E::general_context and a assume "Upd t l u = E\<up>a" and "lc a"
    from context_upd[OF sym[OF this(1)]]
    show "\<exists>e T. C, e \<turnstile> a : T"
    proof (elim disjE)
      assume "E = cHole" 
      with `Upd t l u = E\<up>a` `C, e \<turnstile> t : T` pred_u `finite L` `l \<in> do T`
      show ?thesis by auto
    next
      assume "\<exists>(E'::general_context). t = E'\<up>a"
      then obtain E'::general_context where "t = E'\<up>a"
	by auto
      with `C, e \<turnstile> t : T` `lc a` pred_t
      show ?thesis by blast
    next
      from exFresh_s_p_cof[OF `finite L`]
      obtain s p where "s \<notin> L" and "p \<notin> L" and "s \<noteq> p"
	by auto
      assume "\<exists>(E'::general_context). u = E'\<up>a"
      with `lc a` sopen_and_copen[of a 0 "Fvar s" "Fvar p"]
      obtain E'::general_context 
	where "u\<^bsup>[Fvar s, Fvar p]\<^esup> = {0 \<rightarrow>\<^sub>c [Fvar s, Fvar p]} E'\<up>a"
	unfolding openz_def
	by auto
      with `s \<notin> L` `p \<notin> L` `s \<noteq> p` pred_u `lc a`
      show "\<exists>e T. C, e \<turnstile> a : T" by blast
    qed
  qed
next
  case (T_Obj e f T L C) 
  note pred = this(4)
  show ?case
  proof (intro strip)
    fix E::general_context and a assume "Obj f T = E\<up>a" and "lc a"
    from context_obj[OF sym[OF this(1)]]
    show "\<exists>e T. C, e \<turnstile> a : T"
    proof (elim disjE)
      assume "E = cHole" 
      with `Obj f T = E\<up>a` `ok e` `dom f = do T` `finite L` pred
      show ?thesis by (rule_tac x = e in exI, rule_tac x = T in exI, auto)
    next
      from exFresh_s_p_cof[OF `finite L`]
      obtain s p where "s \<notin> L" and "p \<notin> L" and "s \<noteq> p"
	by auto
      assume "\<exists>l (E'::general_context). f l = Some (E'\<up>a)"
      then obtain l and E'::general_context where "f l = Some (E'\<up>a)"
	by auto
      with `lc a` sopen_and_copen[of a 0 "Fvar s" "Fvar p" E'] `dom f = do T`
      have 
	"the (f l)\<^bsup>[Fvar s, Fvar p]\<^esup> = {0 \<rightarrow>\<^sub>c [Fvar s, Fvar p]} E'\<up>a" and 
	"l \<in> do T"
	unfolding openz_def
	by auto
      with `s \<notin> L` `p \<notin> L` `s \<noteq> p` pred `lc a`
      show "\<exists>e T. C, e \<turnstile> a : T" by blast
    qed
  qed
qed

lemma subterm_typing: 
  "\<lbrakk> C, Env empty \<turnstile> (E::general_context)\<up>a : T; lc a \<rbrakk> 
  \<Longrightarrow> \<exists>T e. C, e \<turnstile> a : T"
  using subterm_typing_pre[of C "Env empty" "E\<up>a" T E a]
  by auto

lemma closed_subterm_type: 
  assumes 
  "C, Env empty \<turnstile> (E::general_context)\<up>a : T" and 
  "lc a" and "closed a"
  shows "\<exists>T. C, Env empty \<turnstile> a : T"
  using subterm_typing[OF assms(1-2)] closed_typing[OF _ assms(3)]
  by auto

subsubsection{*Unique Type*}
theorem type_unique[rule_format]: 
  assumes "C, env \<turnstile> a: T"
  shows "\<forall>T'. C, env \<turnstile> a: T' \<longrightarrow> T = T'"
  using assms
proof (induct rule: typing.induct)
  case T_ActRef thus ?case by auto
next 
  case T_FutRef thus ?case by auto
next
  case T_Var thus ?case by (auto simp: add_get_eq)
next
  case T_Obj show ?case by (auto simp: sym[OF obj_inv])
next
  case T_Call from this(2) show ?case by auto
next
  case T_Active from this(3) show ?case by auto
next 
  case T_Upd from this(4) show ?case by auto
qed

subsubsection{*Typing ensures well-formedness*}
lemma Typing_implies_WF1a: 
  assumes "\<turnstile> c : CT"
  shows "wf_1a c"
  unfolding wf_1a_def
  using assms
proof (induct rule: Ctyping.induct)
  case (T_conf amap c fmap) note pred = this (3)
  show ?case
  proof (intro strip)
    fix A G and E::general_context
    assume "A \<in> dom c" and "(c.\<^sub>AA) = E\<up>ActRef G"
    from this(1) pred have "TConfig amap fmap, Env empty \<turnstile> c.\<^sub>AA : the (amap A)"
      by auto
    from subterm_typing_pre[OF this `(c.\<^sub>AA) = E\<up>ActRef G`]
    obtain e T where "TConfig amap fmap, e \<turnstile> ActRef G : T" by auto
    with `dom amap = dom c` show "G \<in> dom c" by auto
  qed
qed

lemma Typing_implies_WF1b:
  assumes "\<turnstile> c : CT"
  shows "wf_1b c"
  unfolding wf_1b_def
  using assms
proof (induct rule: Ctyping.induct)
  case (T_conf amap c fmap) note pred = this (3)
  show ?case
  proof (intro strip)
    fix A fi G and E::general_context
    assume "A \<in> dom c" and "fi \<in> dom (c.\<^sub>RA)" and "(c.\<^sub>FA<fi>) = E\<up>ActRef G"
    from this(1-2) pred have "TConfig amap fmap, Env empty \<turnstile> c.\<^sub>FA<fi> : the (fmap fi)"
      by auto
    from subterm_typing_pre[OF this `(c.\<^sub>FA<fi>) = E\<up>ActRef G`]
    obtain e T where "TConfig amap fmap, e \<turnstile> ActRef G : T" by auto
    with `dom amap = dom c` show "G \<in> dom c" by auto
  qed
qed

lemma Typing_implies_WF2a:
  assumes "\<turnstile> c : CT"
  shows "wf_2a c"
  unfolding wf_2a_def
  using assms
proof (induct rule: Ctyping.induct)
  case (T_conf amap c fmap) 
  note domfmap = this(2) and pred = this (3)
  show ?case
  proof (intro strip)
    fix A fi and E::general_context
    assume "A \<in> dom c" and "(c.\<^sub>AA) = E\<up>FutRef fi"
    from this(1) pred have "TConfig amap fmap, Env empty \<turnstile> c.\<^sub>AA : the (amap A)"
      by auto
    from subterm_typing_pre[OF this `(c.\<^sub>AA) = E\<up>FutRef fi`]
    obtain e T where "TConfig amap fmap, e \<turnstile> FutRef fi : T" by auto
    hence "fi \<in> dom fmap" by auto
    with domfmap show "\<exists>B\<in>dom c. fi \<in> dom (c.\<^sub>RB)" by auto
  qed
qed

lemma Typing_implies_WF2b:
  assumes "\<turnstile> c : CT"
  shows "wf_2b c"
  unfolding wf_2b_def
  using assms
proof (induct rule: Ctyping.induct)
  case (T_conf amap c fmap) 
  note domfmap = this(2) and pred = this (3)
  show ?case
  proof (intro strip)
    fix A fk fi and E::general_context
    assume "A \<in> dom c" and "fk \<in> dom (c.\<^sub>RA)" and "(c.\<^sub>FA<fk>) = E\<up>FutRef fi"
    from this(1-2) pred have "TConfig amap fmap, Env empty \<turnstile> c.\<^sub>FA<fk> : the (fmap fk)"
      by auto
    from subterm_typing_pre[OF this `(c.\<^sub>FA<fk>) = E\<up>FutRef fi`]
    obtain e T where "TConfig amap fmap, e \<turnstile> FutRef fi : T" by auto
    hence "fi \<in> dom fmap" by auto
    with domfmap show "\<exists>B\<in>dom c. fi \<in> dom (c.\<^sub>RB)" by auto
  qed
qed

theorem Typing_implies_WF: "\<turnstile> c : CT \<Longrightarrow> wf_conf c"
  unfolding wf_conf_def
  by (simp add: Typing_implies_WF1a Typing_implies_WF1b Typing_implies_WF2a Typing_implies_WF2b)

lemma typing_amap_bigger_pre:
  assumes "C, Env empty \<turnstile> t : T"
  shows 
  "\<forall>amap fmap G T'. C = TConfig amap fmap 
    \<longrightarrow> (\<forall>E::general_context. t \<noteq> E\<up>ActRef G) 
    \<longrightarrow> TConfig (amap(G \<mapsto> T')) fmap, Env empty \<turnstile> t : T"
proof -
  def pred_cof \<equiv> "\<lambda>L C e t T l. \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p 
                   \<longrightarrow> C, e\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> \<turnstile> t\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
  from assms show ?thesis
  proof (induct 
      taking: "\<lambda>C e t T l. \<forall>amap fmap G T'. C = TConfig amap fmap 
                            \<longrightarrow> (\<forall>E::general_context. t \<noteq> E\<up>ActRef G)
                            \<longrightarrow> (\<exists>L. finite L 
                                   \<and> pred_cof L (TConfig (amap(G \<mapsto> T')) fmap) e t T l)"
      rule: typing_induct)
    case Fvar thus ?case by auto
  next
    case FutRef thus ?case by auto
  next
    case (ActRef amap fmap e T B) thus ?case 
    proof (auto)
      fix T'
      have "\<exists>E::general_context. ActRef B = E\<up>ActRef B" 
	by (rule_tac x = cHole in exI, auto)
      moreover
      assume "\<forall>E::general_context. ActRef B \<noteq> E\<up>ActRef B"
      ultimately show "T' = T" by auto
    qed
  next
    case (Active C e A a) 
    note pred_a = this(3)
    show ?case proof (intro strip)
      fix amap fmap G T assume "C = TConfig amap fmap" 
      moreover
      assume noGAct: "\<forall>E::general_context. Active a \<noteq> E\<up>ActRef G"
      hence "\<forall>E::general_context. a \<noteq> E\<up>ActRef G"
      proof (intro strip)
	fix E::general_context
	{
	  assume "a = E\<up>ActRef G"
	  hence "\<exists>E::general_context. Active a = E\<up>ActRef G"
	    by (rule_tac x = "cActive E" in exI, simp)
	  with noGAct have False by simp
	} thus "a \<noteq> E\<up>ActRef G" by auto
      qed
      ultimately
      show "TConfig (amap(G \<mapsto> T)) fmap, e \<turnstile> Active a : A" 
	using `ok e` pred_a by (auto, blast)
    qed
  next
    case (Call C e T t l u) 
    note pred_t = this(2) and pred_u = this(4)
    show ?case proof (intro strip)
      fix amap fmap G T' 
      assume 
	"C = TConfig amap fmap" and 
	noGCall: "\<forall>E::general_context. Call t l u \<noteq> E\<up>ActRef G"
      have "\<forall>E::general_context. t \<noteq> E\<up>ActRef G"
      proof (intro strip)
	fix E::general_context
	{
	  assume "t = E\<up>ActRef G"
	  hence "\<exists>E::general_context. Call t l u = E\<up>ActRef G"
	    by (rule_tac x = "cCallL E l u" in exI, simp)
	  with noGCall have False by simp
	} thus "t \<noteq> E\<up>ActRef G" by auto
      qed
      with `C = TConfig amap fmap` pred_t
      have "TConfig (amap(G \<mapsto> T')) fmap, e \<turnstile> t : T" by blast
      moreover
      have "\<forall>E::general_context. u \<noteq> E\<up>ActRef G"
      proof (intro strip)
	fix E::general_context
	{
	  assume "u = E\<up>ActRef G"
	  hence "\<exists>E::general_context. Call t l u = E\<up>ActRef G"
	    by (rule_tac x = "cCallR t l E" in exI, simp)
	  with noGCall have False by simp
	} thus "u \<noteq> E\<up>ActRef G" by auto
      qed
      with `C = TConfig amap fmap` pred_u
      have "TConfig (amap(G \<mapsto> T')) fmap, e \<turnstile> u : param(the(T^l))" by blast
      ultimately
      show "TConfig (amap(G \<mapsto> T')) fmap, e \<turnstile> Call t l u : return(the(T^l))"
	using `l \<in> do T` by auto
    qed
  next
    case (Upd C e T t l u)
    note pred_t = this(2) and pred_u = this(4)
    show ?case proof (intro strip)
      fix amap fmap G T' 
      assume 
	"C = TConfig amap fmap" and 
	noGUpd: "\<forall>E::general_context. Upd t l u \<noteq> E\<up>ActRef G"
      have "\<forall>E::general_context. t \<noteq> E\<up>ActRef G"
      proof (intro strip)
	fix E::general_context
	{
	  assume "t = E\<up>ActRef G"
	  hence "\<exists>E::general_context. Upd t l u = E\<up>ActRef G"
	    by (rule_tac x = "cUpdL E l u" in exI, simp)
	  with noGUpd have False by simp
	} thus "t \<noteq> E\<up>ActRef G" by auto
      qed
      with `C = TConfig amap fmap` pred_t
      have "TConfig (amap(G \<mapsto> T')) fmap, e \<turnstile> t : T" by blast
      moreover
      have noGu: "\<forall>E::general_context. u \<noteq> E\<up>ActRef G"
      proof (intro strip)
	fix E::general_context
	{
	  assume "u = E\<up>ActRef G"
	  hence "\<exists>E::general_context. Upd t l u = E\<up>ActRef G"
	    by (rule_tac x = "cUpdR t l E" in exI, simp)
	  with noGUpd have False by simp
	} thus "u \<noteq> E\<up>ActRef G" by auto
      qed
      with `C = TConfig amap fmap` pred_u
      obtain L where 
	"finite L" and 
	"pred_cof L (TConfig (amap(G \<mapsto> T')) fmap) e u T l"
	by blast 
      ultimately
      show "TConfig (amap(G \<mapsto> T')) fmap, e \<turnstile> Upd t l u : T"
	using `l \<in> do T`
	by (auto simp: pred_cof_def)
    qed
  next
    case (Obj C e T f) note pred_o = this(3)
    def pred_pre \<equiv> "\<lambda>amap fmap (G::ActivityRef) (T'::Sigma.type). C = TConfig amap fmap"
    def pred_cof' \<equiv> "\<lambda>amap fmap G T' b l. (\<forall>E::general_context. the b \<noteq> E\<up>ActRef G) 
                      \<longrightarrow> (\<exists>L. finite L 
                             \<and> pred_cof L (TConfig (amap(G \<mapsto> T')) fmap) e (the b) T l)"
    show ?case proof (intro strip)
      fix amap fmap G T' 
      def pred_bnd \<equiv> "\<lambda>s p b l. TConfig (amap(G \<mapsto> T')) fmap, e\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr> 
                                \<turnstile> (the b\<^bsup>[Fvar s, Fvar p]\<^esup>) :  return(the(T^l))"
      from pred_o
      have 
	pred: "\<forall>amap fmap G T'. pred_pre amap fmap G T' 
	        \<longrightarrow> (\<forall>l\<in>dom f. pred_cof' amap fmap G T' (f l) l)"
	by (intro fmap_ball_all_4'[of f pred_pre pred_cof'], 
	  unfold pred_pre_def pred_cof'_def, simp)

      assume "C = TConfig amap fmap"
      moreover
      assume noGObj: "\<forall>E::general_context. Obj f T \<noteq> E\<up>ActRef G"
      hence "\<forall>l\<in>dom f. \<forall>E::general_context. the (f l) \<noteq> E\<up>ActRef G"
      proof (intro strip)
	fix l and E::general_context
	assume "l \<in> dom f"
	{
	  assume "the (f l) = E\<up>ActRef G"
	  with `l \<in> dom f` have "f = f(l \<mapsto> E \<up> ActRef G)" 
	    by (auto simp: map_upd_triv[of f l "E\<up>ActRef G"])
	  with `l \<in> dom f` have "\<exists>E::general_context. Obj f T = E\<up>ActRef G"
	    by (rule_tac x = "cObj (Abs_FmapLabel(f,l)) T E" in exI, auto)
	  with noGObj have False by simp
	} thus "the (f l) \<noteq> E\<up>ActRef G" by auto
      qed
      ultimately
      have "\<forall>l\<in>dom f. \<exists>L. finite L \<and> pred_cof L (TConfig (amap(G \<mapsto> T')) fmap) e (the(f l)) T l"
	using pred
	unfolding pred_pre_def pred_cof'_def
	by auto
      then obtain L where
	"finite L" and 
	"\<forall>l\<in>do T. pred_cof L (TConfig (amap(G \<mapsto> T')) fmap) e (the(f l)) T l"
	using `dom f = do T` fmap_ex_cof[of f pred_bnd]
	unfolding pred_cof_def pred_bnd_def
	by auto
      with `ok e` `dom f = do T`
      show "TConfig (amap(G \<mapsto> T')) fmap, e \<turnstile> Obj f T : T"
	by (auto simp: pred_cof_def)
    qed
  next
    case (Bnd C e T l t L)
    note pred = this(3)
    from `finite L` show ?case 
    proof (intro strip, rule_tac x = "L \<union> FV t" in exI, 
	simp add: pred_cof_def, intro strip, elim conjE)
      fix amap fmap G T' s p
      assume 
	"C = TConfig amap fmap" and 
	noGt: "\<forall>E::general_context. t \<noteq> E\<up>ActRef G" and
	"s \<notin> L" and "s \<notin> FV t" and "p \<notin> L" and "p \<notin> FV t" and "s \<noteq> p"
      have "\<forall>E::general_context. (t\<^bsup>[Fvar s, Fvar p]\<^esup>) \<noteq> E\<up>ActRef G"
      proof (intro strip)
	fix E::general_context
	{
	  assume "(t\<^bsup>[Fvar s, Fvar p]\<^esup>) = E\<up>ActRef G"
	  with
	    sclose_and_cclose[of s "ActRef G" p 0 E]
	    sclose_sopen_eq_t[of s t p 0]
	    `s \<notin> FV t` `p \<notin> FV t` `s \<noteq> p`
	  have "\<exists>E::general_context. t = E\<up>ActRef G"
	    by (rule_tac x = "{0 \<leftarrow>\<^sub>c [s,p]} E" in exI, auto simp: openz_def)
	  with noGt have False by simp
	} thus "(t\<^bsup>[Fvar s, Fvar p]\<^esup>) \<noteq> E\<up>ActRef G" by auto
      qed
      with `C = TConfig amap fmap` `s \<notin> L` `p \<notin> L` `s \<noteq> p` pred
      show 
	"TConfig (amap(G \<mapsto> T')) fmap, e\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> 
	 \<turnstile> t\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
	by blast
    qed    
  qed
qed

lemma typing_amap_bigger: 
  "\<lbrakk> TConfig amap fmap, Env empty \<turnstile> t : T'; \<forall>E::general_context. t \<noteq> E\<up>ActRef G \<rbrakk> 
  \<Longrightarrow> TConfig (amap(G \<mapsto> T)) fmap, Env empty \<turnstile> t : T'"
  by (simp add: typing_amap_bigger_pre)

lemma typing_fmap_bigger_pre:
  assumes "C, Env empty \<turnstile> t : T"
  shows 
  "\<forall>amap fmap fn T'. C = TConfig amap fmap 
    \<longrightarrow> (\<forall>E::general_context. t \<noteq> E\<up>FutRef fn) 
    \<longrightarrow> TConfig amap (fmap(fn \<mapsto> T')), Env empty \<turnstile> t : T"
proof -
  def pred_cof \<equiv> "\<lambda>L C e t T l. \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p 
                   \<longrightarrow> C, e\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> \<turnstile> t\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
  from assms show ?thesis
  proof (induct 
      taking: "\<lambda>C e t T l. \<forall>amap fmap fn T'. C = TConfig amap fmap 
                            \<longrightarrow> (\<forall>E::general_context. t \<noteq> E\<up>FutRef fn)
                            \<longrightarrow> (\<exists>L. finite L 
                                   \<and> pred_cof L (TConfig amap (fmap(fn \<mapsto> T'))) e t T l)"
      rule: typing_induct)
    case Fvar thus ?case by auto
  next
    case ActRef thus ?case by auto
  next
    case (FutRef amap fmap e T fi) thus ?case 
    proof (auto)
      fix T'
      have "\<exists>E::general_context. FutRef fi = E\<up>FutRef fi" 
	by (rule_tac x = cHole in exI, auto)
      moreover
      assume "\<forall>E::general_context. FutRef fi \<noteq> E\<up>FutRef fi"
      ultimately show "T' = T" by auto
    qed
  next
    case (Active C e A a) 
    note pred_a = this(3)
    show ?case proof (intro strip)
      fix amap fmap fn T assume "C = TConfig amap fmap" 
      moreover
      assume nofnAct: "\<forall>E::general_context. Active a \<noteq> E\<up>FutRef fn"
      hence "\<forall>E::general_context. a \<noteq> E\<up>FutRef fn"
      proof (intro strip)
	fix E::general_context
	{
	  assume "a = E\<up>FutRef fn"
	  hence "\<exists>E::general_context. Active a = E\<up>FutRef fn"
	    by (rule_tac x = "cActive E" in exI, simp)
	  with nofnAct have False by simp
	} thus "a \<noteq> E\<up>FutRef fn" by auto
      qed
      ultimately
      show "TConfig amap (fmap(fn \<mapsto> T)), e \<turnstile> Active a : A" 
	using `ok e` pred_a by (auto, blast)
    qed
  next
    case (Call C e T t l u) 
    note pred_t = this(2) and pred_u = this(4)
    show ?case proof (intro strip)
      fix amap fmap fn T' 
      assume 
	"C = TConfig amap fmap" and 
	nofnCall: "\<forall>E::general_context. Call t l u \<noteq> E\<up>FutRef fn"
      have "\<forall>E::general_context. t \<noteq> E\<up>FutRef fn"
      proof (intro strip)
	fix E::general_context
	{
	  assume "t = E\<up>FutRef fn"
	  hence "\<exists>E::general_context. Call t l u = E\<up>FutRef fn"
	    by (rule_tac x = "cCallL E l u" in exI, simp)
	  with nofnCall have False by simp
	} thus "t \<noteq> E\<up>FutRef fn" by auto
      qed
      with `C = TConfig amap fmap` pred_t
      have "TConfig amap (fmap(fn \<mapsto> T')), e \<turnstile> t : T" by blast
      moreover
      have "\<forall>E::general_context. u \<noteq> E\<up>FutRef fn"
      proof (intro strip)
	fix E::general_context
	{
	  assume "u = E\<up>FutRef fn"
	  hence "\<exists>E::general_context. Call t l u = E\<up>FutRef fn"
	    by (rule_tac x = "cCallR t l E" in exI, simp)
	  with nofnCall have False by simp
	} thus "u \<noteq> E\<up>FutRef fn" by auto
      qed
      with `C = TConfig amap fmap` pred_u
      have "TConfig amap (fmap(fn \<mapsto> T')), e \<turnstile> u : param(the(T^l))" by blast
      ultimately
      show "TConfig amap (fmap(fn \<mapsto> T')), e \<turnstile> Call t l u : return(the(T^l))"
	using `l \<in> do T` by auto
    qed
  next
    case (Upd C e T t l u)
    note pred_t = this(2) and pred_u = this(4)
    show ?case proof (intro strip)
      fix amap fmap fn T' 
      assume 
	"C = TConfig amap fmap" and 
	nofnUpd: "\<forall>E::general_context. Upd t l u \<noteq> E\<up>FutRef fn"
      have "\<forall>E::general_context. t \<noteq> E\<up>FutRef fn"
      proof (intro strip)
	fix E::general_context
	{
	  assume "t = E\<up>FutRef fn"
	  hence "\<exists>E::general_context. Upd t l u = E\<up>FutRef fn"
	    by (rule_tac x = "cUpdL E l u" in exI, simp)
	  with nofnUpd have False by simp
	} thus "t \<noteq> E\<up>FutRef fn" by auto
      qed
      with `C = TConfig amap fmap` pred_t
      have "TConfig amap (fmap(fn \<mapsto> T')), e \<turnstile> t : T" by blast
      moreover
      have nofnu: "\<forall>E::general_context. u \<noteq> E\<up>FutRef fn"
      proof (intro strip)
	fix E::general_context
	{
	  assume "u = E\<up>FutRef fn"
	  hence "\<exists>E::general_context. Upd t l u = E\<up>FutRef fn"
	    by (rule_tac x = "cUpdR t l E" in exI, simp)
	  with nofnUpd have False by simp
	} thus "u \<noteq> E\<up>FutRef fn" by auto
      qed
      with `C = TConfig amap fmap` pred_u
      obtain L where 
	"finite L" and 
	"pred_cof L (TConfig amap (fmap(fn \<mapsto> T'))) e u T l"
	by blast 
      ultimately
      show "TConfig amap (fmap(fn \<mapsto> T')), e \<turnstile> Upd t l u : T"
	using `l \<in> do T`
	by (auto simp: pred_cof_def)
    qed
  next
    case (Obj C e T f) note pred_o = this(3)
    def pred_pre \<equiv> "\<lambda>amap fmap (fn::FutureRef) (T'::Sigma.type). C = TConfig amap fmap"
    def pred_cof' \<equiv> "\<lambda>amap fmap fn T' b l. (\<forall>E::general_context. the b \<noteq> E\<up>FutRef fn) 
                      \<longrightarrow> (\<exists>L. finite L 
                             \<and> pred_cof L (TConfig amap (fmap(fn \<mapsto> T'))) e (the b) T l)"
    show ?case proof (intro strip)
      fix amap fmap fn T' 
      def pred_bnd \<equiv> "\<lambda>s p b l. TConfig amap (fmap(fn \<mapsto> T')), e\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr> 
                                \<turnstile> (the b\<^bsup>[Fvar s, Fvar p]\<^esup>) :  return(the(T^l))"
      from pred_o
      have 
	pred: "\<forall>amap fmap fn T'. pred_pre amap fmap fn T' 
	        \<longrightarrow> (\<forall>l\<in>dom f. pred_cof' amap fmap fn T' (f l) l)"
	by (intro fmap_ball_all_4'[of f pred_pre pred_cof'], 
	  unfold pred_pre_def pred_cof'_def, simp)

      assume "C = TConfig amap fmap"
      moreover
      assume nofnObj: "\<forall>E::general_context. Obj f T \<noteq> E\<up>FutRef fn"
      hence "\<forall>l\<in>dom f. \<forall>E::general_context. the (f l) \<noteq> E\<up>FutRef fn"
      proof (intro strip)
	fix l and E::general_context
	assume "l \<in> dom f"
	{
	  assume "the (f l) = E\<up>FutRef fn"
	  with `l \<in> dom f` have "f = f(l \<mapsto> E\<up>FutRef fn)" 
	    by (auto simp: map_upd_triv[of f l "E\<up>FutRef fn"])
	  with `l \<in> dom f` have "\<exists>E::general_context. Obj f T = E\<up>FutRef fn"
	    by (rule_tac x = "cObj (Abs_FmapLabel(f,l)) T E" in exI, auto)
	  with nofnObj have False by simp
	} thus "the (f l) \<noteq> E\<up>FutRef fn" by auto
      qed
      ultimately
      have "\<forall>l\<in>dom f. \<exists>L. finite L \<and> pred_cof L (TConfig amap (fmap(fn \<mapsto> T'))) e (the(f l)) T l"
	using pred
	unfolding pred_pre_def pred_cof'_def
	by auto
      then obtain L where
	"finite L" and 
	"\<forall>l\<in>do T. pred_cof L (TConfig amap (fmap(fn \<mapsto> T'))) e (the(f l)) T l"
	using `dom f = do T` fmap_ex_cof[of f pred_bnd]
	unfolding pred_cof_def pred_bnd_def
	by auto
      with `ok e` `dom f = do T`
      show "TConfig amap (fmap(fn \<mapsto> T')), e \<turnstile> Obj f T : T"
	by (auto simp: pred_cof_def)
    qed
  next
    case (Bnd C e T l t L)
    note pred = this(3)
    from `finite L` show ?case 
    proof (intro strip, rule_tac x = "L \<union> FV t" in exI, 
	simp add: pred_cof_def, intro strip, elim conjE)
      fix amap fmap fn T' s p
      assume 
	"C = TConfig amap fmap" and 
	nofnt: "\<forall>E::general_context. t \<noteq> E\<up>FutRef fn" and
	"s \<notin> L" and "s \<notin> FV t" and "p \<notin> L" and "p \<notin> FV t" and "s \<noteq> p"
      have "\<forall>E::general_context. (t\<^bsup>[Fvar s, Fvar p]\<^esup>) \<noteq> E\<up>FutRef fn"
      proof (intro strip)
	fix E::general_context
	{
	  assume "(t\<^bsup>[Fvar s, Fvar p]\<^esup>) = E\<up>FutRef fn"
	  with
	    sclose_and_cclose[of s "FutRef fn" p 0 E]
	    sclose_sopen_eq_t[of s t p 0]
	    `s \<notin> FV t` `p \<notin> FV t` `s \<noteq> p`
	  have "\<exists>E::general_context. t = E\<up>FutRef fn"
	    by (rule_tac x = "{0 \<leftarrow>\<^sub>c [s,p]} E" in exI, auto simp: openz_def)
	  with nofnt have False by simp
	} thus "(t\<^bsup>[Fvar s, Fvar p]\<^esup>) \<noteq> E\<up>FutRef fn" by auto
      qed
      with `C = TConfig amap fmap` `s \<notin> L` `p \<notin> L` `s \<noteq> p` pred
      show 
	"TConfig amap (fmap(fn \<mapsto> T')), e\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> 
	 \<turnstile> t\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
	by blast
    qed    
  qed
qed

lemma typing_fmap_bigger: 
  "\<lbrakk> TConfig amap fmap, Env empty \<turnstile> t : T'; \<forall>E::general_context. t \<noteq> E\<up>FutRef fn \<rbrakk> 
  \<Longrightarrow> TConfig amap (fmap(fn \<mapsto> T)), Env empty \<turnstile> t : T'"
  by (simp add: typing_fmap_bigger_pre)

lemma typing_fmap_eq: 
  assumes 
  "dom c \<subseteq> dom c'" and 
  "\<forall>A\<in>dom c. dom (c.\<^sub>RA) = dom (c'.\<^sub>RA)" and
  "\<forall>A\<in>(dom c' - dom c). dom (c'.\<^sub>RA) = {}"
  shows
  "(\<Union>A\<in>dom c. dom (c.\<^sub>RA)) = (\<Union>A\<in>dom c'. dom (c'.\<^sub>RA))"
proof
  from `dom c \<subseteq> dom c'` assms(2)
  show "(\<Union>A\<in>dom c. dom (c.\<^sub>RA)) \<subseteq> (\<Union>A\<in>dom c'. dom (c'.\<^sub>RA))"
    by (intro UN_mono[of "dom c" "dom c'" "\<lambda>A. dom (c.\<^sub>RA)" "\<lambda>A. dom (c'.\<^sub>RA)"], auto)
next
  from `dom c \<subseteq> dom c'` have "dom c' - (dom c' - dom c) \<subseteq> dom c" by auto
  with assms(2)
  have "(\<Union>A\<in>dom c' - (dom c' - dom c). dom (c'.\<^sub>RA)) \<subseteq> (\<Union>A\<in>dom c. dom (c.\<^sub>RA))" 
    by (intro 
      UN_mono[of "dom c' - (dom c' - dom c)" 
                 "dom c" "\<lambda>A. dom (c'.\<^sub>RA)" "\<lambda>A. dom (c.\<^sub>RA)"], auto)
  moreover
  from assms(3) have "(\<Union>A\<in>dom c' - dom c. dom (c'.\<^sub>RA)) = {}" by auto
  ultimately
  show "(\<Union>A\<in>dom c'. dom (c'.\<^sub>RA)) \<subseteq> (\<Union>A\<in>dom c. dom (c.\<^sub>RA))"
    using `dom c \<subseteq> dom c'`
  proof auto
    fix x A a b y assume "(c'.\<^sub>RA) x = Some y"
    hence "x \<in> dom (c'.\<^sub>RA)" by auto
    hence "dom (c'.\<^sub>RA) \<noteq> {}" by auto
    moreover
    assume "c' A = Some (a, b)"
    ultimately
    have "A \<in> dom c" using assms(3) by auto
    with assms(2) `x \<in> dom (c'.\<^sub>RA)`
    have "x \<in> dom (c.\<^sub>RA)" by auto
    with `A \<in> dom c` show "\<exists>A\<in>dom c. x \<in> dom (c.\<^sub>RA)" by auto
  qed
qed

lemma context_active':
  "(E::general_context)\<up>x = Active a 
  \<Longrightarrow> E = cHole \<or> (\<exists>E'::general_context. a = E'\<up>x \<and> E = cActive E')"
  by (case_tac E, auto)

lemma context_upd':
  "(E::general_context)\<up>x = Upd a l b
  \<Longrightarrow> E = cHole 
      \<or> (\<exists>E'::general_context. a = E'\<up>x \<and> E = cUpdL E' l b)
      \<or> (\<exists>E'::general_context. b = E'\<up>x \<and> E = cUpdR a l E')"
  by (case_tac E, auto)

lemma context_call'[rule_format]: 
  "(E::general_context)\<up>x = Call a l b
  \<Longrightarrow> E = cHole 
      \<or> (\<exists>E'::general_context. a = E'\<up>x \<and> E = cCallL E' l b)
      \<or> (\<exists>E'::general_context. b = E'\<up>x \<and> E = cCallR a l E')"
  by (case_tac E, auto)

lemma context_obj'[rule_format]:
  "\<forall>x f T. ((E::general_context)\<up>x) = Obj f T 
    \<longrightarrow> E = cHole 
        \<or> (\<exists>l (E'::general_context). (f l) = Some (E'\<up>x) 
                                   \<and> E \<approx> cObj (Abs_FmapLabel (f,l)) T E')"
proof (cases E)
  case cHole thus ?thesis by auto
next
  case cActive thus ?thesis by auto
next
  case cCallL thus ?thesis by auto
next
  case cCallR thus ?thesis by auto
next
  case cUpdL thus ?thesis by auto
next
  case cUpdR thus ?thesis by auto
next
  case (cObj fml T E) thus ?thesis 
    by (auto, 
      rule_tac x = "FLlabel fml" in exI, auto,
      rule_tac x = E in exI, auto simp: equiv_and_equals)
qed

lemma fmap_ex_cof':
  fixes
  P1 :: "'c \<Rightarrow> 'c \<Rightarrow> 'b option \<Rightarrow> ('a::fintype) \<Rightarrow> bool" and
  P2 :: "'c \<Rightarrow> 'c \<Rightarrow> 'b option \<Rightarrow> ('a::fintype) \<Rightarrow> bool"
  assumes
  "\<forall>l\<in>dom (f::('a -~> 'b)). 
    \<exists>L. finite L \<and> (\<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p \<longrightarrow> P1 s p (f l) l) 
                 \<and> (\<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p \<longrightarrow> P2 s p (f l) l)"
  shows
  "\<exists>L. finite L \<and> (\<forall>l\<in>dom f. \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p \<longrightarrow> P1 s p (f l) l) 
                \<and> (\<forall>l\<in>dom f. \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p \<longrightarrow> P2 s p (f l) l)"
proof -
  from assms 
  have
    "\<forall>l\<in>dom f. \<exists>L. finite L \<and> (\<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p 
                                 \<longrightarrow> P1 s p (f l) l \<and> P2 s p (f l) l)"
  proof (intro strip)
    fix l assume "l \<in> dom f"
    with assms 
    have
      "\<exists>L. finite L \<and> (\<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p \<longrightarrow> P1 s p (f l) l) 
                    \<and> (\<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p \<longrightarrow> P2 s p (f l) l)"
      by auto
    then obtain L where
      "finite L" and
      "\<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p \<longrightarrow> P1 s p (f l) l" and
      "\<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p \<longrightarrow> P2 s p (f l) l"
      by auto
    thus 
      "\<exists>L. finite L \<and> (\<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p 
                        \<longrightarrow> P1 s p (f l) l \<and> P2 s p (f l) l)"
      by (rule_tac x = L in exI, auto)
  qed
  with fmap_ex_cof[of f "\<lambda>s p b l. P1 s p b l \<and> P2 s p b l"]
  obtain L where 
    "finite L" and 
    "\<forall>l\<in>dom f. \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p 
                 \<longrightarrow> P1 s p (f l) l \<and> P2 s p (f l) l"
    by auto
  thus ?thesis by (rule_tac x = L in exI, auto)
qed

lemma subst_term_typing_pre:
  assumes "C, e \<turnstile> t : T"
  shows "\<forall>(E::general_context) a b T'. t = E\<up>a
          \<longrightarrow> C, e \<turnstile> a : T' \<longrightarrow> C, e \<turnstile> b : T'
          \<longrightarrow> C, e \<turnstile> E\<up>b : T"
proof -
  def type_cof \<equiv> 
    "\<lambda>L C e t T l. \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p \<longrightarrow>
    C, e\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> \<turnstile> t\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
  def pred_cof \<equiv> 
    "\<lambda>L C e t T l. \<forall>s p. s \<notin> L \<and> p \<notin> L \<and> s \<noteq> p \<longrightarrow>
    (\<forall>(E::general_context) a b T'. t = E\<up>a
      \<longrightarrow> C, e \<turnstile> a : T' \<longrightarrow> C, e \<turnstile> b : T'
      \<longrightarrow> C, e\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> \<turnstile> (E\<up>b)\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l)))"
  from assms show ?thesis
  proof (induct 
      taking: "\<lambda>C e t T l. \<exists>L. finite L \<and> type_cof L C e t T l \<and> pred_cof L C e t T l"
      rule: typing_induct)
    case (Fvar C e T x)
    show ?case proof (intro strip)
      fix E::general_context and a b T'
      assume "Fvar x = E\<up>a" and "C, e \<turnstile> a : T'"
      from context_fvar[OF sym[OF this(1)]] 
      have "E = cHole" and "a = Fvar x" by auto
      with 
	`C, e \<turnstile> a : T'` 
	type_unique[OF typing.T_Var[OF `ok e` `x \<in> env_dom e` `the e!x = T`]]
      have "T = T'" by auto
      moreover assume "C, e \<turnstile> b : T'"
      ultimately show "C, e \<turnstile> E\<up>b : T"
	using `E = cHole` by simp
    qed
  next
    case (ActRef amap fmap e T B)
    hence typingaref: "TConfig amap fmap, e \<turnstile> ActRef B : T" by auto
    show ?case proof (intro strip)
      fix E::general_context and a b T'
      assume "ActRef B = E\<up>a" and "TConfig amap fmap, e \<turnstile> a : T'"
      from context_actrefs[OF this(1)] this(2) typingaref
      have "E = cHole" and "T = T'" by auto
      moreover assume "TConfig amap fmap, e \<turnstile> b : T'"
      ultimately show "TConfig amap fmap, e \<turnstile> E\<up>b : T"
	by simp
    qed
  next
    case (FutRef amap fmap e T fk)
    hence typingfref: "TConfig amap fmap, e \<turnstile> FutRef fk : T" by auto
    show ?case proof (intro strip)
      fix E::general_context and a b T'
      assume "FutRef fk = E\<up>a" and "TConfig amap fmap, e \<turnstile> a : T'"
      from context_futrefs[OF this(1)] this(2) typingfref
      have "E = cHole" and "T = T'" by auto
      moreover assume "TConfig amap fmap, e \<turnstile> b : T'"
      ultimately show "TConfig amap fmap, e \<turnstile> E\<up>b : T"
	by simp
    qed
  next
    case (Active C e T t) note pred = this(3)
    from `ok e` `C, e \<turnstile> t : T` have typingact: "C, e \<turnstile> Active t : T" by auto
    show ?case proof (intro strip)
      fix E::general_context and a b T'
      assume "Active t = E\<up>a" and "C, e \<turnstile> a : T'" and "C, e \<turnstile> b : T'"
      from context_active'[OF sym[OF this(1)]] show "C, e \<turnstile> E\<up>b : T"
      proof (cases "E = cHole", simp_all)
	case True with `Active t = E\<up>a` `C, e \<turnstile> a : T'` typingact
	have "T = T'" by (auto simp: type_unique)
	with `C, e \<turnstile> b : T'` show "C, e \<turnstile> b : T" by simp
      next
	case False assume "\<exists>(E'::general_context). t = E'\<up>a \<and> E = cActive E'"
	then obtain E'::general_context where "t = E'\<up>a" and "E = cActive E'" 
	  by auto
	with pred `C, e \<turnstile> a : T'` `C, e \<turnstile> b : T'`
	have "C, e \<turnstile> E'\<up>b : T" by blast
	with `E = cActive E'` `ok e` show "C, e \<turnstile> E\<up>b : T" by auto
      qed
    qed
  next
    case (Call C e T t l u)
    note pred_t = this(2) and pred_u = this(4)
    show ?case proof (intro strip)
      fix a b T' and E::general_context
      assume "Call t l u = E\<up>a" and "C, e \<turnstile> a : T'" and "C, e \<turnstile> b : T'"
      from context_call'[OF sym[OF this(1)]]
      show "C, e \<turnstile> E\<up>b : return (the(T^l))"
      proof (elim disjE exE conjE)
	assume "E = cHole" 
	with 
	  `Call t l u = E\<up>a` `C, e \<turnstile> t : T` `C, e \<turnstile> u : param(the(T^l))` `l \<in> do T`
	have "C, e \<turnstile> a : return(the(T^l))" by auto
	from type_unique[OF this `C, e \<turnstile> a : T'`] `C, e \<turnstile> b : T'` `E = cHole`
	show ?thesis by simp
      next
	fix E'::general_context
	assume "t = E'\<up>a"
	with pred_t `C, e \<turnstile> a : T'` `C, e \<turnstile> b : T'`
	have "C, e \<turnstile> E'\<up>b : T" by blast
	moreover assume "E = cCallL E' l u"
	ultimately show ?thesis
	  using `l \<in> do T` `C, e \<turnstile> u : param(the(T^l))`
	  by auto
      next
	fix E'::general_context
	assume "u = E'\<up>a"
	with pred_u `C, e \<turnstile> a : T'` `C, e \<turnstile> b : T'`
	have "C, e \<turnstile> E'\<up>b : param(the(T^l))" by blast
	moreover assume "E = cCallR t l E'"
	ultimately show ?thesis
	  using `l \<in> do T` `C, e \<turnstile> t : T`
	  by auto
      qed
    qed
  next
    case (Upd C e T t l u)
    note pred_t = this(2) and this(4)
    from this(2) obtain L where 
      "finite L" and "type_cof L C e u T l" and "pred_cof L C e u T l"
      by auto
    show ?case proof (intro strip)
      fix a b T' and E::general_context
      assume "Upd t l u = E\<up>a" and "C, e \<turnstile> a : T'" and "C, e \<turnstile> b : T'"
      from context_upd'[OF sym[OF this(1)]]
      show "C, e \<turnstile> E\<up>b : T"
      proof (elim disjE exE conjE)
	assume "E = cHole"
	with 
	  sym[OF `Upd t l u = E\<up>a`] `type_cof L C e u T l` 
	  `finite L` `C, e \<turnstile> t : T` `l \<in> do T`
	have "C, e \<turnstile> a : T" by (auto simp: type_cof_def)
	from type_unique[OF this `C, e \<turnstile> a : T'`] `C, e \<turnstile> b : T'` `E = cHole`
	show ?thesis by simp
      next
	fix E'::general_context
	assume "t = E'\<up>a"
	with pred_t `C, e \<turnstile> a : T'` `C, e \<turnstile> b : T'`
	have "C, e \<turnstile> E'\<up>b : T" by blast
	moreover assume "E = cUpdL E' l u"
	ultimately show ?thesis
	  using `l \<in> do T` `finite L` `type_cof L C e u T l`
	  by (auto simp: type_cof_def)
      next
	fix E'::general_context
	assume "u = E'\<up>a" and "E = cUpdR t l E'"
	with 
	  `finite L` `pred_cof L C e u T l` `C, e \<turnstile> a : T'` `C, e \<turnstile> b : T'`
	  `l \<in> do T` `C, e \<turnstile> t : T`
	show ?thesis by (auto simp: pred_cof_def)
      qed
    qed
  next
    case (Obj C e T f) note pred = this(3)
    def type_term \<equiv> 
      "\<lambda>s p b l. C, e\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> \<turnstile> the b\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
    def type_ctxt \<equiv>
      "\<lambda>s p b l. \<forall>(E::general_context) a c T'. the b = E\<up>a
      \<longrightarrow> C, e \<turnstile> a : T' \<longrightarrow> C, e \<turnstile> c : T'
      \<longrightarrow> C, e\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> \<turnstile> (E\<up>c)\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
    from pred sym[OF `dom f = do T`] fmap_ex_cof'[of f type_term type_ctxt]
    obtain L where 
      "finite L" and 
      "\<forall>l\<in>do T. type_cof L C e (the(f l)) T l" and
      "\<forall>l\<in>do T. pred_cof L C e (the(f l)) T l"
      unfolding type_cof_def pred_cof_def type_term_def type_ctxt_def
      by auto
    show ?case proof (intro strip)
      fix a b T' and E::general_context
      assume "Obj f T = E\<up>a" and "C, e \<turnstile> a : T'" and "C, e \<turnstile> b : T'"
      from context_obj'[OF sym[OF this(1)]]
      show "C, e \<turnstile> E\<up>b : T"
      proof (elim disjE exE conjE)
	assume "E = cHole"
	with 
	  sym[OF `Obj f T = E\<up>a`] `\<forall>l\<in>do T. type_cof L C e (the(f l)) T l` 
	  `finite L` `ok e` `dom f = do T`
	have "C, e \<turnstile> a : T" by (auto simp: type_cof_def)
	from type_unique[OF this `C, e \<turnstile> a : T'`] `C, e \<turnstile> b : T'` `E = cHole`
	show ?thesis by simp
      next
	fix l and E'::general_context
	assume "f l = Some (E'\<up>a)" with `dom f = do T`
	have "l \<in> do T" and "the (f l) = E'\<up>a" by auto
	with `\<forall>l\<in>do T. pred_cof L C e (the(f l)) T l`
	have "pred_cof L C e (the(f l)) T l" by auto
	moreover assume "E \<approx> cObj (Abs_FmapLabel (f,l)) T E'"
	note equiv_and_fill[OF this]
	note this[of b]
	ultimately show ?thesis
	  using 
	    `ok e` `dom f = do T` `finite L` 
	    `\<forall>l\<in>do T. pred_cof L C e (the(f l)) T l` 
	    `\<forall>l\<in>do T. type_cof L C e (the(f l)) T l` 
	    `l \<in> do T` `the (f l) = E'\<up>a` `C, e \<turnstile> a : T'` `C, e \<turnstile> b : T'`
	  unfolding pred_cof_def type_cof_def
	  by (simp (no_asm_simp), intro typing.T_Obj, auto)
      qed
    qed
  next
    case (Bnd C e T l t L) note pred = this(3)
    with `finite L` ok_finite[OF `ok e`]
    show ?case 
    proof (rule_tac x = "L \<union> env_dom e" in exI, 
	simp add: type_cof_def, 
	unfold pred_cof_def, intro strip)
      fix s p a b T' and E::general_context
      assume 
	"s \<notin> L \<union> env_dom e \<and> p \<notin> L \<union> env_dom e \<and> s \<noteq> p" and
	"C, e \<turnstile> a : T'" and "C, e \<turnstile> b : T'"
      hence 
	"s \<notin> L" and "p \<notin> L" and "s \<noteq> p" and 
	"s \<notin> env_dom e" and "p \<notin> env_dom e"
	by auto
      note 
	this(1-3)
	bigger_env_lemma[OF bigger_env_lemma[OF `C, e \<turnstile> a : T'` 
	                                        this(4)] 
	                    not_in_env_bigger[OF this(5) not_sym[OF this(3)]]] and
	bigger_env_lemma[OF bigger_env_lemma[OF `C, e \<turnstile> b : T'` 
	                                        this(4)] 
	                    not_in_env_bigger[OF this(5) not_sym[OF this(3)]]]
      moreover
      assume "t = E\<up>a" with sopen_and_copen[OF typing_regular''[OF `C, e \<turnstile> a : T'`]]
      have "(t\<^bsup>[Fvar s, Fvar p]\<^esup>) = {0 \<rightarrow>\<^sub>c [Fvar s, Fvar p]} E\<up>a"
	by (auto simp: openz_def)
      ultimately
      have "C, e\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> \<turnstile> {0 \<rightarrow>\<^sub>c [Fvar s, Fvar p]} E\<up>b : return(the(T^l))"
	using pred by blast
      with sopen_and_copen[OF typing_regular''[OF `C, e \<turnstile> b : T'`]]
      show "C, e\<lparr>s:T\<rparr>\<lparr>p:param (the(T^l))\<rparr> \<turnstile> (E\<up>b)\<^bsup>[Fvar s, Fvar p]\<^esup> : return(the(T^l))"
	by (auto simp: openz_def)
    qed
  qed
qed

lemma subst_term_typing:
  "\<lbrakk> C, Env empty \<turnstile> (E::general_context)\<up>a : T;
     C, Env empty \<turnstile> a : T'; C, Env empty \<turnstile> b: T' \<rbrakk> 
  \<Longrightarrow> C, Env empty \<turnstile> (E::general_context)\<up>b : T"
  by (simp add: subst_term_typing_pre)

lemma noActRef_WF1a:
  assumes 
  "wf_1a c" and "A \<in> dom c" and 
  "\<exists>E::general_context. c.\<^sub>AA = E\<up>t" and "G \<notin> dom c"
  shows "\<forall>E::general_context. t \<noteq> E\<up>ActRef G"
  using assms
proof (intro strip, auto)
  fix E E'::general_context 
  assume "c.\<^sub>AA = E\<up>E'\<up>ActRef G" hence "c.\<^sub>AA = (E\<circ>E')\<up>ActRef G" by simp
  with `wf_1a c` `A \<in> dom c` `G \<notin> dom c` 
  show False unfolding wf_1a_def by blast
qed

lemma noActRef_WF1b:
  assumes 
  "wf_1b c" and "A \<in> dom c" and "fk \<in> dom (c.\<^sub>RA)" and
  "\<exists>E::general_context. c.\<^sub>FA<fk> = E\<up>t" and "G \<notin> dom c"
  shows "\<forall>E::general_context. t \<noteq> E\<up>ActRef G"
  using assms
proof (intro strip, auto)
  fix E E'::general_context 
  assume "c.\<^sub>FA<fk> = E\<up>E'\<up>ActRef G" hence "c.\<^sub>FA<fk> = (E\<circ>E')\<up>ActRef G" by simp 
  with `wf_1b c` `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` `G \<notin> dom c`
  show False unfolding wf_1b_def by blast
qed

lemma noFutRef_WF2a:
  assumes 
  "wf_2a c" and "A \<in> dom c" and 
  "\<exists>E::general_context. c.\<^sub>AA = E\<up>t" and "\<forall>A\<in>dom c. fn \<notin> dom (c.\<^sub>RA)"
  shows "\<forall>E::general_context. t \<noteq> E\<up>FutRef fn"
  using assms
proof (intro strip, auto)
  fix E E'::general_context 
  assume "c.\<^sub>AA = E\<up>E'\<up>FutRef fn" hence "c.\<^sub>AA = (E\<circ>E')\<up>FutRef fn" by simp
  with `wf_2a c` `A \<in> dom c` 
  have "\<exists>B\<in>dom c. fn \<in> dom (c.\<^sub>RB)" unfolding wf_2a_def by blast
  with `\<forall>A\<in>dom c. fn \<notin> dom (c.\<^sub>RA)` show False by simp
qed

lemma noFutRef_WF2b:
  assumes 
  "wf_2b c" and "A \<in> dom c" and "fk \<in> dom (c.\<^sub>RA)" and
  "\<exists>E::general_context. c.\<^sub>FA<fk> = E\<up>t" and "\<forall>A\<in>dom c. fn \<notin> dom (c.\<^sub>RA)"
  shows "\<forall>E::general_context. t \<noteq> E\<up>FutRef fn"
  using assms
proof (intro strip, auto)
  fix E E'::general_context 
  assume "c.\<^sub>FA<fk> = E\<up>E'\<up>FutRef fn" hence "c.\<^sub>FA<fk> = (E\<circ>E')\<up>FutRef fn" by simp
  with `wf_2b c` `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` 
  have "\<exists>B\<in>dom c. fn \<in> dom (c.\<^sub>RB)" unfolding wf_2b_def by blast
  with `\<forall>A\<in>dom c. fn \<notin> dom (c.\<^sub>RA)` show False by simp
qed

lemma map_le_upd: "l \<notin> dom f \<Longrightarrow> f \<subseteq>\<^sub>m f(l \<mapsto> t)"
  by (auto simp: map_le_def)

lemma Csubject_reduction_pre: 
  assumes "\<turnstile> c : CT"
  shows "\<And>c' amap fmap. \<lbrakk> CT = TConfig amap fmap; c \<rightarrow>\<^sub>\<zeta> c' \<rbrakk> \<Longrightarrow> \<exists>amap' fmap'. \<turnstile> c' : TConfig amap' fmap' \<and> amap \<subseteq>\<^sub>m amap' \<and> fmap \<subseteq>\<^sub>m fmap'"
  using assms
proof (induct rule: Ctyping.induct)
  case (T_conf amap' c fmap' c')
  note domfmap = this(2) and pred = this(3)
  from `c \<rightarrow>\<^sub>\<zeta> c'` show ?case
  proof (cases c c')
    case (zeta_local A ca fk a)
    note def_c' = this(2)
    with `A \<in> dom ca` `c = ca` have "dom c = dom c'" by auto
    from `c = ca` def_c' `fk \<in> dom (ca.\<^sub>RA)` `A \<in> dom ca`
    have domrequ: "\<forall>B\<in>dom c. dom (c.\<^sub>RB) = dom (c'.\<^sub>RB)" 
      by (auto simp: RequestQueue_def)
    from def_c' `A \<in> dom ca` `c = ca` 
    have actobj: "\<forall>B\<in>dom c. c.\<^sub>AB = (c'.\<^sub>AB)" by (auto simp: ActiveObject_def)
    show ?thesis
    proof (rule_tac x = amap' in exI, rule_tac x = fmap' in exI,
	intro conjI Ctyping.intros)
      from `dom amap' = dom c` `dom c = dom c'` 
      show "dom amap' = dom c'" by auto
    next
      from domfmap `dom c = dom c'` domrequ
      show "dom fmap' = (\<Union>A\<in>dom c'. dom (c'.\<^sub>RA))" by auto
    next
      show 
	"\<forall>A\<in>dom c'. 
	   TConfig amap' fmap', Env empty \<turnstile> c'.\<^sub>AA : the (amap' A) \<and>
	   (\<forall>fi\<in>dom(c'.\<^sub>RA). TConfig amap' fmap', Env empty \<turnstile> c'.\<^sub>FA<fi> : the (fmap' fi))"
      proof (intro strip conjI)
	fix B assume "B \<in> dom c'" with `dom c = dom c'` have "B \<in> dom c" by simp
	with `c = ca` def_c' have "(c'.\<^sub>AB) = (c.\<^sub>AB)" by (auto simp: ActiveObject_def)
	with pred `B \<in> dom c` show "TConfig amap' fmap', Env empty \<turnstile> c'.\<^sub>AB : the (amap' B)"
	  by auto
      next
	fix B assume "B \<in> dom c'" with `dom c = dom c'` have "B \<in> dom c" by simp
	moreover
	fix fi assume "fi \<in> dom (c'.\<^sub>RB)"
	with `B \<in> dom c` domrequ have "fi \<in> dom (c.\<^sub>RB)" by auto
	ultimately
	have typingc: "TConfig amap' fmap', Env empty \<turnstile> c.\<^sub>FB<fi> : the (fmap' fi)"
	  using pred by auto
	thus "TConfig amap' fmap', Env empty \<turnstile> c'.\<^sub>FB<fi> : the (fmap' fi)"
	proof (cases "A=B")
	  case True thus ?thesis
	  proof (cases "fi=fk")
	    case True with `A = B` def_c'
	    have "c'.\<^sub>FB<fi> = a" by (auto simp: RequestQueue_def OneRequest_def)
	    moreover
	    from `A = B` `fi = fk` typingc `c = ca`
	    have "TConfig amap' fmap', Env empty \<turnstile> ca.\<^sub>FA<fk> : the (fmap' fk)"
	      by simp
	    note subject_reduction[OF this `ca.\<^sub>FA<fk> \<rightarrow>\<^sub>\<beta> a`]
	    ultimately show ?thesis using `fi = fk` by simp
	  next
	    from `A = B` `c = ca` `fi \<in> dom (c.\<^sub>RB)` have "fi \<in> dom (ca.\<^sub>RA)" by simp
	    case False note OneRequest_diff[OF _ not_sym[OF this] `fi \<in> dom (ca.\<^sub>RA)`]
	    from this[of ca A "ca.\<^sub>AA" a] `B \<in> dom c` `A = B` def_c' `c = ca`
	    have "c'.\<^sub>FB<fi> = (c.\<^sub>FB<fi>)" by (auto simp: RequestQueue_def ActiveObject_def)
	    thus ?thesis using typingc by simp
	  qed
	next
	  case False from OneRequest_diff2[OF `B \<in> dom c` this] def_c' `c = ca`
	  have "c'.\<^sub>FB<fi> = (c.\<^sub>FB<fi>)" by auto
	  thus ?thesis using typingc by simp
	qed
      qed
    next
      from `TConfig amap' fmap' = TConfig amap fmap`
      show "amap \<subseteq>\<^sub>m amap'" by auto
    next
      from `TConfig amap' fmap' = TConfig amap fmap`
      show "fmap \<subseteq>\<^sub>m fmap'" by auto
    qed
  next
    case (active G ca A fk E a)
    note def_c' = this (2)
    with `A \<in> dom ca` `c = ca` have "dom c \<union> {G} = dom c'" by auto
    with `G \<notin> dom ca` `c = ca` have "dom c \<subseteq> dom c'" by auto
    from def_c' have "dom (c'.\<^sub>RG) = {}" by (auto simp: RequestQueue_def)
    from `c = ca` def_c' `fk \<in> dom (ca.\<^sub>RA)` `A \<in> dom ca` `G \<notin> dom ca`
    have domrequ: "\<forall>B\<in>dom c. dom (c.\<^sub>RB) = dom (c'.\<^sub>RB)" 
      by (auto simp: RequestQueue_def)
    from 
      `A \<in> dom ca` `c = ca` `fk \<in> dom (ca.\<^sub>RA)` 
      sym[OF `(ca.\<^sub>FA<fk>) = E\<up>Active a`] pred
    have "TConfig amap' fmap', Env empty \<turnstile> E\<up>Active a : the (fmap' fk)" 
      by auto
    from closed_subterm_type[OF this lc.lc_Act[OF `lc a`]] `closed a`
    obtain T where typinga: "TConfig amap' fmap', Env empty \<turnstile> a : T" 
      by (auto simp: closed_def)

    show ?thesis
    proof (rule_tac x = "(amap'(G \<mapsto> T))" in exI, rule_tac x = fmap' in exI,
	intro conjI Ctyping.intros)
      from `dom amap' = dom c` `dom c \<union> {G} = dom c'` 
      show "dom (amap'(G \<mapsto> T)) = dom c'" by auto
    next
      from domfmap show "dom fmap' = (\<Union>A\<in>dom c'. dom (c'.\<^sub>RA))"
      proof (simp, intro typing_fmap_eq[OF `dom c \<subseteq> dom c'` domrequ])
	from `dom c \<union> {G} = dom c'` `G \<notin> dom ca` `c = ca`
	have "dom c' - dom c = {G}" by auto
	with `dom (c'.\<^sub>RG) = {}`
	show "\<forall>A\<in>dom c' - dom c. dom (c'.\<^sub>RA) = {}" by auto
      qed
    next
      show 
	"\<forall>A\<in>dom c'. 
	   TConfig (amap'(G \<mapsto> T)) fmap', Env empty \<turnstile> c'.\<^sub>AA : the ((amap'(G \<mapsto> T)) A) \<and>
	   (\<forall>fi\<in>dom(c'.\<^sub>RA). TConfig (amap'(G \<mapsto> T)) fmap', Env empty 
	                    \<turnstile> c'.\<^sub>FA<fi> : the (fmap' fi))"
      proof (intro strip conjI)
	fix B assume "B \<in> dom c'"
	thus "TConfig (amap'(G \<mapsto> T)) fmap', Env empty \<turnstile> c'.\<^sub>AB : the ((amap'(G \<mapsto> T)) B)"
	proof (cases "G=B")
	  case True with def_c' have "c'.\<^sub>AB = a" by (auto simp: ActiveObject_def)
	  with typinga `G = B`
	  have typingao: "TConfig amap' fmap', Env empty \<turnstile> c'.\<^sub>AB : the ((amap'(G \<mapsto> T)) B)" 
	    by simp
	  with 
	    `A \<in> dom ca` `fk \<in> dom (ca.\<^sub>RA)` `c = ca` `G \<notin> dom ca`
	    `c'.\<^sub>AB = a` `(ca.\<^sub>FA<fk>) = E \<up> Active a`
	  show ?thesis
	    by (intro 
	      typing_amap_bigger[OF typingao 
	                            noActRef_WF1b[OF Typing_implies_WF1b[OF 
	                                               Ctyping.T_conf[OF prems(2-4)]]]], 
	      simp_all, rule_tac x = "E\<circ>(cActive cHole)" in exI, simp)
	next
	  case False with `dom c \<union> {G} = dom c'` `B \<in> dom c'` 
	  have "B \<in> dom c" by auto
	  with pred 
	  have typingao: "TConfig amap' fmap', Env empty \<turnstile> c.\<^sub>AB : the (amap' B)"
	    by auto
	  from `G \<noteq> B` have "the ((amap'(G \<mapsto> T)) B) = the (amap' B)" by auto
	  moreover
	  from `c = ca` `G \<noteq> B` def_c' have "(c'.\<^sub>AB) = (c.\<^sub>AB)"
	    by (auto simp: ActiveObject_def)
	  ultimately show ?thesis
	    using sym[OF `c = ca`] `G \<notin> dom ca`
	    by (simp, intro 
	      typing_amap_bigger[OF typingao 
	                            noActRef_WF1a[OF Typing_implies_WF1a[OF 
	                                               Ctyping.T_conf[OF prems(2-4)]]
	                                             `B \<in> dom c`]], 
	      simp_all, rule_tac x = cHole in exI, simp)
	qed
      next
	fix B fi assume "fi \<in> dom (c'.\<^sub>RB)"
	with `dom (c'.\<^sub>RG) = {}` have "G \<noteq> B" by auto
	moreover assume "B \<in> dom c'"
	ultimately have "B \<in> dom c"
	  using def_c' `c = ca` `A \<in> dom ca`
	  by auto
	with pred domrequ `fi \<in> dom (c'.\<^sub>RB)`
	have typingrequ: "TConfig amap' fmap', Env empty \<turnstile> c.\<^sub>FB<fi> : the (fmap' fi)"
	  by auto
	from `B \<in> dom c` domrequ `fi \<in> dom (c'.\<^sub>R B)`
	have "fi \<in> dom (c.\<^sub>RB)" by auto

	show "TConfig (amap'(G \<mapsto> T)) fmap', Env empty \<turnstile> c'.\<^sub>FB<fi> : the (fmap' fi)"
	proof (cases "A=B")
	  case True
	  show ?thesis proof (cases "fi=fk")
	    case True with `A = B` `G \<noteq> B` `c = ca` def_c'
	    have "(c'.\<^sub>FB<fi>) = E\<up>ActRef G" by (auto simp: OneRequest_def)
	    moreover
	    with `c = ca` `G \<notin> dom ca`
	    have "TConfig (amap'(G \<mapsto> T)) fmap', Env empty \<turnstile> c.\<^sub>FB<fi> : the (fmap' fi)"
	      by (intro 
		typing_amap_bigger[OF typingrequ
	                              noActRef_WF1b[OF Typing_implies_WF1b[OF 
	                                                 Ctyping.T_conf[OF prems(2-4)]]
	                                               `B \<in> dom c` `fi \<in> dom (c.\<^sub>RB)`]], 
		simp_all, rule_tac x = cHole in exI, simp)
	    with `c = ca` `A = B` `fi = fk` `ca.\<^sub>FA<fk> = E\<up>Active a`
	    have "TConfig (amap'(G \<mapsto> T)) fmap', Env empty \<turnstile> E\<up>Active a : the (fmap' fi)"
	      by simp
	    moreover
	    from 
	      `closed a` `A \<in> dom ca` `fk \<in> dom (ca.\<^sub>RA)` 
	      `c = ca` `ca.\<^sub>FA<fk> = E\<up>Active a` `G \<notin> dom ca`
	    have "TConfig (amap'(G \<mapsto> T)) fmap', Env empty \<turnstile> Active a : T"
	      by (intro
		typing_amap_bigger[OF T_Active[OF _ typinga]
	                              noActRef_WF1b[OF Typing_implies_WF1b[OF 
	                                                 Ctyping.T_conf[OF prems(2-4)]]]], 
		simp_all, rule_tac x = E in exI, simp) 
	    ultimately show ?thesis
	      using `c'.\<^sub>FB<fi> = E\<up>ActRef G`
	      by (simp, 
		intro subst_term_typing[of "TConfig (amap'(G \<mapsto> T)) fmap'" 
		                           E "Active a" "the (fmap' fi)" 
		                           T "ActRef G"], auto)
	  next
	    case False with `A = B` `G \<noteq> B` def_c' `c = ca`
	    have "c'.\<^sub>FB<fi> = (c.\<^sub>FB<fi>)" by (auto simp: RequestQueue_def OneRequest_def)
	    with `G \<notin> dom ca` sym[OF `c = ca`] show ?thesis
	      by (simp, intro
		typing_amap_bigger[OF typingrequ
	                              noActRef_WF1b[OF Typing_implies_WF1b[OF 
	                                                 Ctyping.T_conf[OF prems(2-4)]]
		                                       `B \<in> dom c` `fi \<in> dom (c.\<^sub>RB)`]],
		simp_all, rule_tac x = cHole in exI, simp)
	  qed
	next
	  case False with `G \<noteq> B` def_c' `c = ca`
	  have "c'.\<^sub>FB<fi> = (c.\<^sub>FB<fi>)" by (auto simp: OneRequest_def)
	  with `G \<notin> dom ca` sym[OF `c = ca`] show ?thesis
	    by (simp, intro
	      typing_amap_bigger[OF typingrequ
	                            noActRef_WF1b[OF Typing_implies_WF1b[OF 
	                                               Ctyping.T_conf[OF prems(2-4)]]
		                                     `B \<in> dom c` `fi \<in> dom (c.\<^sub>RB)`]],
	      simp_all, rule_tac x = cHole in exI, simp)
 	qed
      qed
    next
      from `G \<notin> dom ca` `c = ca` `dom amap' = dom c`
      have "G \<notin> dom amap'" by auto
      from `TConfig amap' fmap' = TConfig amap fmap` map_le_upd[OF this]
      show "amap \<subseteq>\<^sub>m amap'(G \<mapsto> T)" by auto
    next
      from `TConfig amap' fmap' = TConfig amap fmap`
      show "fmap \<subseteq>\<^sub>m fmap'" by auto      
    qed
  next
    case (request ca fn A B fk E l u)
    note def_c' = this(2)
    def c'' \<equiv> "ca(A \<mapsto> ((ca.\<^sub>RA)(fk \<mapsto> E\<up>FutRef fn), ca.\<^sub>AA))"
    with `A \<in> dom ca` `fk \<in> dom (ca.\<^sub>RA)`
    have 
      "dom ca = dom c''" and 
      domrequ_cac'': "\<forall>A\<in>dom ca. dom (ca.\<^sub>RA) = dom (c''.\<^sub>RA)"
      by (auto simp: RequestQueue_def)
    from 
      c''_def `A \<noteq> B` `B \<in> dom ca` `A \<in> dom ca` `fk \<in> dom (ca.\<^sub>RA)` 
      def_c' `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)`
    have
      "B \<in> dom c''" and fnc'': "\<forall>A\<in>dom c''. fn \<notin> dom (c''.\<^sub>RA)" and
      def_c'c'': "c' = c''(B \<mapsto> ((c''.\<^sub>RB)(fn \<mapsto> Call (c''.\<^sub>AB) l u), c''.\<^sub>AB))"
      by (auto simp: RequestQueue_def ActiveObject_def)
    with `dom ca = dom c''` `c = ca`
    have 
      "dom c'' = dom c'" and "dom c = dom c'" and
      domrequ_c''c': "\<forall>A\<in>dom c''. (A \<noteq> B \<longrightarrow> dom (c'.\<^sub>RA) = dom (c''.\<^sub>RA)) 
                                \<and> (A = B \<longrightarrow> dom (c'.\<^sub>RA) = dom (c''.\<^sub>RA) \<union> {fn})"
      by (auto simp: RequestQueue_def)
    from 
      `A \<in> dom ca` `c = ca` `fk \<in> dom (ca.\<^sub>RA)` 
      sym[OF `(ca.\<^sub>FA<fk>) = E\<up>Call (ActRef B) l u`] pred
    have typingctxt: "TConfig amap' fmap', Env empty \<turnstile> E\<up>Call (ActRef B) l u : the (fmap' fk)" 
      by auto
    from closed_subterm_type[OF this] `lc u` `closed u`
    obtain T where typingcall: "TConfig amap' fmap', Env empty \<turnstile> Call (ActRef B) l u : T" 
      by (auto simp: closed_def)
    then obtain T' where 
      typingactr: "TConfig amap' fmap', Env empty \<turnstile> ActRef B : T'" and 
      "T = return(the(T'^l))" 
      by auto
    from this(1) have "the (amap' B) = T'" by auto

    show ?thesis
    proof (rule_tac x = amap' in exI, rule_tac x = "fmap'(fn \<mapsto> T)" in exI,
	intro conjI Ctyping.intros)
      from `dom amap' = dom c` `dom c = dom c'` 
      show "dom amap' = dom c'" by simp
    next
      from `dom ca = dom c''` domrequ_cac'' domfmap `c = ca`
      have "dom fmap' = (\<Union>A\<in>dom c''. dom (c''.\<^sub>RA))" by auto
      moreover
      from `dom c'' = dom c'` domrequ_c''c'
      have "(\<Union>A\<in>dom c''. dom (c''.\<^sub>RA)) \<union> {fn} = (\<Union>A\<in>dom c'. dom (c'.\<^sub>RA))"
	using `B \<in> dom c''` by blast
      ultimately show "dom (fmap'(fn \<mapsto> T)) = (\<Union>A\<in>dom c'. dom (c'.\<^sub>RA))" by auto
    next
      show 
	"\<forall>A\<in>dom c'. 
	   TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> c'.\<^sub>AA : the (amap' A) \<and>
	   (\<forall>fi\<in>dom(c'.\<^sub>RA). TConfig amap' (fmap'(fn \<mapsto> T)), Env empty 
	                    \<turnstile> c'.\<^sub>FA<fi> : the ((fmap'(fn \<mapsto> T)) fi))"
      proof (intro strip conjI)
	fix G assume "G \<in> dom c'"
	with `dom c = dom c'` have "G \<in> dom c" by simp
	moreover
	from def_c' `A \<in> dom ca` `B \<in> dom ca` `c = ca`
	have "\<forall>A\<in>dom c. (c.\<^sub>AA) = (c'.\<^sub>AA)" by (auto simp: ActiveObject_def)
	ultimately
	have typingao: "TConfig amap' fmap', Env empty \<turnstile> c'.\<^sub>AG : the (amap' G)" 
	  using pred by auto
	from `G \<in> dom c` `c = ca` `\<forall>A\<in>dom c. (c.\<^sub>AA) = (c'.\<^sub>AA)` `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)`
	show "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> c'.\<^sub>AG : the (amap' G)"
	  by (intro
	    typing_fmap_bigger[OF typingao
	                          noFutRef_WF2a[OF Typing_implies_WF2a[OF 
	                                             Ctyping.T_conf[OF prems(2-4)]]]],
	    simp_all, rule_tac x = cHole in exI, simp)
      next
	fix G fi assume "G \<in> dom c'" and "fi \<in> dom (c'.\<^sub>RG)"
	with `dom c'' = dom c'` have "G \<in> dom c''" by simp
	from 
	  `A \<in> dom ca` `fk \<in> dom (ca.\<^sub>RA)` sym[OF `c = ca`] 
	  `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)` `ca.\<^sub>FA<fk> = E\<up>(Call (ActRef B) l u)`
	have typingctxt':
	  "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> (cCallL cHole l u)\<up>ActRef B : T"
	  by (simp, intro
	    typing_fmap_bigger[OF typingcall
	                            noFutRef_WF2b[OF Typing_implies_WF2b[OF 
	                                               Ctyping.T_conf[OF prems(2-4)]]]],
	    simp_all, rule_tac x = E in exI, simp)

	show "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> c'.\<^sub>FG<fi> : the ((fmap'(fn \<mapsto> T)) fi)"
	proof (cases "fi=fn")
	  case True
	  with 
	    domrequ_c''c' `G \<in> dom c''` `fi \<in> dom (c'.\<^sub>RG)` 
	    `\<forall>A\<in>dom c''. fn \<notin> dom (c''.\<^sub>RA)`
	  have "G = B" by blast 
	  with `fi = fn` def_c' `A \<noteq> B` 
	  have "(c'.\<^sub>FG<fi>) = Call (ca.\<^sub>AB) l u" by (auto simp: OneRequest_def)
	  moreover
	  from `fi = fn` have "the ((fmap'(fn \<mapsto> T)) fi) = T" by auto
	  moreover
	  {
	    from pred `B \<in> dom ca` `c = ca` 
	    have typingaoB: "TConfig amap' fmap', Env empty \<turnstile> ca.\<^sub>AB : the (amap' B)"
	      by blast
	    from `B \<in> dom ca` `c = ca` `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)`
	    have "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> ca.\<^sub>AB : the (amap' B)"
	      by (intro
		typing_fmap_bigger[OF typingaoB
	                            noFutRef_WF2a[OF Typing_implies_WF2a[OF 
	                                               Ctyping.T_conf[OF prems(2-4)]]]],
		simp_all, rule_tac x = cHole in exI, simp)
	    moreover
	    from `the (amap' B) = T'`
	    have "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> ActRef B : the (amap' B)"
	      by (simp, intro typing_fmap_bigger[OF typingactr diff_futactrefs])
	    ultimately
	    have "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> cCallL cHole l u\<up>(ca.\<^sub>AB) : T"
	      by (intro subst_term_typing[OF typingctxt'], simp)
	  } ultimately show ?thesis by simp
	next
	  case False hence "(fmap'(fn \<mapsto> T)) fi = fmap' fi" by auto
	  show ?thesis proof (cases "A=G")
	    case True 
	    show ?thesis proof (cases "fi=fk")
	      case True with `A = G` `A \<noteq> B` def_c'
	      have "(c'.\<^sub>FG<fi>) = E\<up>FutRef fn" by (auto simp: OneRequest_def)
	      moreover
	      {
		from 
		  `fi = fk` `fk \<in> dom (ca.\<^sub>RA)` `A \<in> dom ca` 
		  `(fmap'(fn \<mapsto> T)) fi = fmap' fi`
		have "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> FutRef fn : T" by auto
		moreover
		from 
		  `fi = fk` `(fmap'(fn \<mapsto> T)) fi = fmap' fi`
		  `A \<in> dom ca` `fk \<in> dom (ca.\<^sub>RA)` sym[OF `c = ca`]
		  `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)` `ca.\<^sub>FA<fk> = E\<up>(Call (ActRef B) l u)`
		have 
		  "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty 
		  \<turnstile> E\<up>Call (ActRef B) l u : the ((fmap'(fn \<mapsto> T)) fi)"
		  by (simp, intro
		    typing_fmap_bigger[OF typingctxt
	                                  noFutRef_WF2b[OF Typing_implies_WF2b[OF 
	                                                     Ctyping.T_conf[OF prems(2-4)]]]],
		    simp_all, rule_tac x = cHole in exI, simp)
		ultimately
		have 
		  "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty
		  \<turnstile> E\<up>FutRef fn : the ((fmap'(fn \<mapsto> T)) fi)"
		  using typingctxt'[simplified]
		  by (intro 
		    subst_term_typing[of "TConfig amap' (fmap'(fn \<mapsto> T))" E 
		                         "Call (ActRef B) l u" 
		                         "the ((fmap'(fn \<mapsto> T)) fi)" T "FutRef fn"],
		    simp)
	      } ultimately show ?thesis by simp
	    next
	      case False 
	      with 
		`A = G` `A \<noteq> B` `G \<in> dom c''` domrequ_c''c' `fi \<in> dom (c'.\<^sub>RG)`
		`dom ca = dom c''` domrequ_cac''
	      have "fi \<in> dom (ca.\<^sub>RG)" by blast
	      with `A \<in> dom ca` `c = ca` pred `A = G`
	      have typingcaG: "TConfig amap' fmap', Env empty \<turnstile> ca.\<^sub>FG<fi> : the (fmap' fi)"
		by blast
	      
	      from `fi \<noteq> fk` `A = G` `A \<noteq> B` def_c' have "(c'.\<^sub>FG<fi>) = (ca.\<^sub>FG<fi>)" 
		by (auto simp: OneRequest_def RequestQueue_def)
	      moreover from `fi \<noteq> fn` have "(fmap'(fn \<mapsto> T)) fi = fmap' fi" by simp
	      ultimately show ?thesis
		using 
		  `A = G` `A \<in> dom ca` `fi \<in> dom (ca.\<^sub>RG)` `c = ca`
		  `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)`
		by (simp, intro
		  typing_fmap_bigger[OF typingcaG
	                                noFutRef_WF2b[OF Typing_implies_WF2b[OF 
	                                                   Ctyping.T_conf[OF prems(2-4)]]]],
		  simp_all, rule_tac x = cHole in exI, simp)
	    qed
	  next
	    case False
	    with 
	      `fi \<noteq> fn` `G \<in> dom c''` domrequ_c''c' `fi \<in> dom (c'.\<^sub>RG)`
	      `dom ca = dom c''` domrequ_cac''
	    have "fi \<in> dom (ca.\<^sub>RG)" by blast
	    with `G \<in> dom c''` `dom ca = dom c''` pred `c = ca`
	    have typingcaG: "TConfig amap' fmap', Env empty \<turnstile> ca.\<^sub>FG<fi> : the (fmap' fi)"
	      by blast
	    
	    from `A \<noteq> G` `fi \<noteq> fn` def_c' have "(c'.\<^sub>FG<fi>) = (ca.\<^sub>FG<fi>)"
	      by (auto simp: OneRequest_def RequestQueue_def)
	    with 
	      `(fmap'(fn \<mapsto> T)) fi = fmap' fi` `G \<in> dom c''` 
	      `dom ca = dom c''` `fi \<in> dom (ca.\<^sub>RG)` `c = ca`
	      `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)`
	    show ?thesis
	      by (simp, intro
		typing_fmap_bigger[OF typingcaG
	                                noFutRef_WF2b[OF Typing_implies_WF2b[OF 
	                                                   Ctyping.T_conf[OF prems(2-4)]]]],
		simp_all, rule_tac x = cHole in exI, simp)
	  qed
	qed
      qed
    next
      from `TConfig amap' fmap' = TConfig amap fmap`
      show "amap \<subseteq>\<^sub>m amap'" by auto      
    next
      from domfmap `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)` `c = ca`
      have "fn \<notin> dom fmap'" by auto
      from `TConfig amap' fmap' = TConfig amap fmap` map_le_upd[OF this]
      show "fmap \<subseteq>\<^sub>m fmap'(fn \<mapsto> T)" by auto
    qed
  next
    case (request_self ca fn A fk E l u)
    note def_c' = this(2)
    with `A \<in> dom ca` `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)` `fk \<in> dom (ca.\<^sub>RA)`
    have 
      "dom ca = dom c'" and
      domrequ: "\<forall>B\<in>dom c'. (B \<noteq> A \<longrightarrow> dom (c'.\<^sub>RB) = dom (ca.\<^sub>RB)) 
                         \<and> (B = A \<longrightarrow> dom (c'.\<^sub>RB) = dom (ca.\<^sub>RB) \<union> {fn})" and
      "\<forall>A\<in>dom c'. (ca.\<^sub>AA) = (c'.\<^sub>AA)"
      by (auto simp: RequestQueue_def ActiveObject_def)
    from 
      `A \<in> dom ca` `c = ca` `fk \<in> dom (ca.\<^sub>RA)` 
      sym[OF `(ca.\<^sub>FA<fk>) = E\<up>Call (ActRef A) l u`] pred
    have typingctxt: "TConfig amap' fmap', Env empty \<turnstile> E\<up>Call (ActRef A) l u : the (fmap' fk)" 
      by auto
    from closed_subterm_type[OF this] `lc u` `closed u`
    obtain T where typingcall: "TConfig amap' fmap', Env empty \<turnstile> Call (ActRef A) l u : T" 
      by (auto simp: closed_def)
    then obtain T' where 
      typingactr: "TConfig amap' fmap', Env empty \<turnstile> ActRef A : T'" and 
      "T = return(the(T'^l))" 
      by auto
    from this(1) have "the (amap' A) = T'" by auto

    show ?thesis
    proof (rule_tac x = amap' in exI, rule_tac x = "fmap'(fn \<mapsto> T)" in exI,
	intro conjI Ctyping.intros)
      from `dom amap' = dom c` `c = ca` `dom ca = dom c'` 
      show "dom amap' = dom c'" by simp
    next
      from `dom ca = dom c'` domfmap `c = ca`
      have "dom fmap' = (\<Union>A\<in>dom ca. dom (ca.\<^sub>RA))" by auto
      moreover
      from `dom ca = dom c'` domrequ
      have "(\<Union>A\<in>dom ca. dom (ca.\<^sub>RA)) \<union> {fn} = (\<Union>A\<in>dom c'. dom (c'.\<^sub>RA))"
	using `A \<in> dom ca` `c = ca` by blast
      ultimately show "dom (fmap'(fn \<mapsto> T)) = (\<Union>A\<in>dom c'. dom (c'.\<^sub>RA))" by auto
    next
      show 
	"\<forall>A\<in>dom c'. 
	   TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> c'.\<^sub>AA : the (amap' A) \<and>
	   (\<forall>fi\<in>dom(c'.\<^sub>RA). TConfig amap' (fmap'(fn \<mapsto> T)), Env empty 
	                    \<turnstile> c'.\<^sub>FA<fi> : the ((fmap'(fn \<mapsto> T)) fi))"
      proof (intro strip conjI)
	fix B assume "B \<in> dom c'"
	with `dom ca = dom c'` `c = ca` `\<forall>A\<in>dom c'. (ca.\<^sub>AA) = (c'.\<^sub>AA)` pred
	have typingao: "TConfig amap' fmap', Env empty \<turnstile> c'.\<^sub>AB : the (amap' B)" by auto
	from 
	  `B \<in> dom c'` `dom ca = dom c'` `c = ca` 
	  `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)` `\<forall>A\<in>dom c'. (ca.\<^sub>AA) = (c'.\<^sub>AA)`
	show "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> c'.\<^sub>AB : the (amap' B)"
	  by (intro
	    typing_fmap_bigger[OF typingao 
	                          noFutRef_WF2a[OF Typing_implies_WF2a[OF
	                                             Ctyping.T_conf[OF prems(2-4)]]]],
	    simp_all, rule_tac x = cHole in exI, simp)
      next
	fix G fi assume "G \<in> dom c'" and "fi \<in> dom (c'.\<^sub>RG)"
	from 
	  `A \<in> dom ca` `fk \<in> dom (ca.\<^sub>RA)` sym[OF `c = ca`] 
	  `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)` `ca.\<^sub>FA<fk> = E\<up>(Call (ActRef A) l u)`
	have typingctxt':
	  "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> (cCallL cHole l u)\<up>ActRef A : T"
	  by (simp, intro
	    typing_fmap_bigger[OF typingcall
	                            noFutRef_WF2b[OF Typing_implies_WF2b[OF 
	                                               Ctyping.T_conf[OF prems(2-4)]]]],
	    simp_all, rule_tac x = E in exI, simp)

	show "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> c'.\<^sub>FG<fi> : the ((fmap'(fn \<mapsto> T)) fi)"
	proof (cases "fi=fn")
	  case True
	  with 
	    `fi \<in> dom (c'.\<^sub>RG)` domrequ `dom ca = dom c'` 
	    `G \<in> dom c'` `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)`
	  have "G = A" by blast
	  with `fi = fn` def_c' have "c'.\<^sub>FG<fi> = Call (ca.\<^sub>AA) l u"
	    by (auto simp: OneRequest_def)
	  moreover
	  from `fi = fn` have "the ((fmap'(fn \<mapsto> T)) fi) = T" by auto
	  moreover
	  {
	    from pred `A \<in> dom ca` `c = ca` 
	    have typingaoA: "TConfig amap' fmap', Env empty \<turnstile> ca.\<^sub>AA : the (amap' A)"
	      by blast
	    from `A \<in> dom ca` `c = ca` `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)`
	    have "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> ca.\<^sub>AA : the (amap' A)"
	      by (intro
		typing_fmap_bigger[OF typingaoA
	                            noFutRef_WF2a[OF Typing_implies_WF2a[OF 
	                                               Ctyping.T_conf[OF prems(2-4)]]]],
		simp_all, rule_tac x = cHole in exI, simp)
	    moreover
	    from `the (amap' A) = T'`
	    have "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> ActRef A : the (amap' A)"
	      by (simp, intro typing_fmap_bigger[OF typingactr diff_futactrefs])
	    ultimately
	    have "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> cCallL cHole l u\<up>(ca.\<^sub>AA) : T"
	      by (intro subst_term_typing[OF typingctxt'], simp)
	  } ultimately show ?thesis by simp
	next
	  case False hence "(fmap'(fn \<mapsto> T)) fi = fmap' fi" by auto
	  show ?thesis proof (cases "A=G")
	    case True show ?thesis proof (cases "fi=fk")
	      case True with `A = G` `fi \<noteq> fn` def_c'
	      have "(c'.\<^sub>FG<fi>) = E\<up>FutRef fn" by (auto simp: OneRequest_def)
	      moreover
	      {
		from 
		  `fi = fk` `fk \<in> dom (ca.\<^sub>RA)` `A \<in> dom ca` 
		  `(fmap'(fn \<mapsto> T)) fi = fmap' fi`
		have "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty \<turnstile> FutRef fn : T" by auto
		moreover
		from 
		  `fi = fk` `(fmap'(fn \<mapsto> T)) fi = fmap' fi`
		  `A \<in> dom ca` `fk \<in> dom (ca.\<^sub>RA)` sym[OF `c = ca`]
		  `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)` `ca.\<^sub>FA<fk> = E\<up>(Call (ActRef A) l u)`
		have 
		  "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty 
		  \<turnstile> E\<up>Call (ActRef A) l u : the ((fmap'(fn \<mapsto> T)) fi)"
		  by (simp, intro
		    typing_fmap_bigger[OF typingctxt
	                                  noFutRef_WF2b[OF Typing_implies_WF2b[OF 
	                                                     Ctyping.T_conf[OF prems(2-4)]]]],
		    simp_all, rule_tac x = cHole in exI, simp)
		ultimately
		have 
		  "TConfig amap' (fmap'(fn \<mapsto> T)), Env empty
		  \<turnstile> E\<up>FutRef fn : the ((fmap'(fn \<mapsto> T)) fi)"
		  using typingctxt'[simplified]
		  by (intro 
		    subst_term_typing[of "TConfig amap' (fmap'(fn \<mapsto> T))" E 
		                         "Call (ActRef A) l u" 
		                         "the ((fmap'(fn \<mapsto> T)) fi)" T "FutRef fn"], 
		    simp)
	      } ultimately show ?thesis by simp
	    next
	      case False 
	      with 
		`A = G` `G \<in> dom c'` domrequ `fi \<in> dom (c'.\<^sub>RG)` 
		`dom ca = dom c'` `fi \<noteq> fn`
	      have "fi \<in> dom (ca.\<^sub>RG)" by blast
	      with `A \<in> dom ca` `c = ca` pred `A = G`
	      have typingcaG: "TConfig amap' fmap', Env empty \<turnstile> ca.\<^sub>FG<fi> : the (fmap' fi)"
		by blast
	      
	      from `fi \<noteq> fk` `A = G` `fi \<noteq> fn` def_c' have "(c'.\<^sub>FG<fi>) = (ca.\<^sub>FG<fi>)" 
		by (auto simp: OneRequest_def RequestQueue_def)
	      with 
		`(fmap'(fn \<mapsto> T)) fi = fmap' fi` 
		`A = G` `A \<in> dom ca` `fi \<in> dom (ca.\<^sub>RG)` `c = ca`
		`\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)`
	      show ?thesis
		by (simp, intro
		  typing_fmap_bigger[OF typingcaG
	                                noFutRef_WF2b[OF Typing_implies_WF2b[OF 
	                                                   Ctyping.T_conf[OF prems(2-4)]]]],
		  simp_all, rule_tac x = cHole in exI, simp)
	    qed
	  next
	    case False
	    with `fi \<noteq> fn` `G \<in> dom c'` domrequ `fi \<in> dom (c'.\<^sub>RG)` `dom ca = dom c'`
	    have "fi \<in> dom (ca.\<^sub>RG)" by blast
	    with `G \<in> dom c'` `dom ca = dom c'` pred `c = ca`
	    have typingcaG: "TConfig amap' fmap', Env empty \<turnstile> ca.\<^sub>FG<fi> : the (fmap' fi)"
	      by blast
	    
	    from `A \<noteq> G` `fi \<noteq> fn` def_c' have "(c'.\<^sub>FG<fi>) = (ca.\<^sub>FG<fi>)"
	      by (auto simp: OneRequest_def RequestQueue_def)
	    with 
	      `(fmap'(fn \<mapsto> T)) fi = fmap' fi`
	      `G \<in> dom c'` `dom ca = dom c'` `fi \<in> dom (ca.\<^sub>RG)` `c = ca`
	      `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)`
	    show ?thesis
	      by (simp, intro
		typing_fmap_bigger[OF typingcaG
	                                noFutRef_WF2b[OF Typing_implies_WF2b[OF 
	                                                   Ctyping.T_conf[OF prems(2-4)]]]],
		simp_all, rule_tac x = cHole in exI, simp)
	  qed
	qed
      qed
    next
      from `TConfig amap' fmap' = TConfig amap fmap`
      show "amap \<subseteq>\<^sub>m amap'" by auto      
    next
      from domfmap `\<forall>A\<in>dom ca. fn \<notin> dom (ca.\<^sub>RA)` `c = ca`
      have "fn \<notin> dom fmap'" by auto
      from `TConfig amap' fmap' = TConfig amap fmap` map_le_upd[OF this]
      show "fmap \<subseteq>\<^sub>m fmap'(fn \<mapsto> T)" by auto
    qed
  next
    case (reply A ca fk E fn B)
    note def_c' = this(2)
    with `A \<in> dom ca` `fk \<in> dom (ca.\<^sub>RA)` `c = ca`
    have 
      "dom c = dom c'" and
      "\<forall>A\<in>dom c'. (c'.\<^sub>AA) = (c.\<^sub>AA)" and
      "\<forall>A\<in>dom c'. dom (c'.\<^sub>RA) = dom (c.\<^sub>RA)" and
      requ: "\<forall>G\<in>dom c'. \<forall>fi\<in>dom (c'.\<^sub>RG).
                         (A = G \<and> fk = fi \<longrightarrow> (c'.\<^sub>FG<fi>) = E\<up>(c.\<^sub>FB<fn>)) 
                       \<and> (A \<noteq> G \<or> fk \<noteq> fi \<longrightarrow> (c'.\<^sub>FG<fi>) = (c.\<^sub>FG<fi>))"
      by (auto simp: OneRequest_def ActiveObject_def RequestQueue_def)

    show ?thesis
    proof (rule_tac x = amap' in exI, rule_tac x = fmap' in exI,
	intro conjI Ctyping.intros)
      from `dom amap' = dom c` `dom c = dom c'` 
      show "dom amap' = dom c'" by simp
    next
      from `dom c = dom c'` domfmap `\<forall>A\<in>dom c'. dom(c'.\<^sub>RA) = dom (c.\<^sub>RA)`
      show "dom fmap' = (\<Union>A\<in>dom c'. dom (c'.\<^sub>RA))" by auto
    next
      show 
	"\<forall>A\<in>dom c'. 
	   TConfig amap' fmap', Env empty \<turnstile> c'.\<^sub>AA : the (amap' A) \<and>
	   (\<forall>fi\<in>dom(c'.\<^sub>RA). TConfig amap' fmap', Env empty 
	                    \<turnstile> c'.\<^sub>FA<fi> : the (fmap' fi))"
      proof (intro strip conjI)
	fix G assume "G \<in> dom c'" 
	with pred `\<forall>A\<in>dom c'. (c'.\<^sub>AA) = (c.\<^sub>AA)` `dom c = dom c'`
	show "TConfig amap' fmap', Env empty \<turnstile> c'.\<^sub>AG : the (amap' G)"
	  by auto
      next
	fix G fi assume "G \<in> dom c'" and "fi \<in> dom (c'.\<^sub>RG)"
	show "TConfig amap' fmap', Env empty \<turnstile> c'.\<^sub>FG<fi> : the (fmap' fi)"
	proof (cases "A=G")
	  case True show ?thesis proof (cases "fk=fi")
	    case True with `G \<in> dom c'` `fi \<in> dom (c'.\<^sub>RG)` `A = G` requ
	    have "(c'.\<^sub>FG<fi>) = E\<up>(c.\<^sub>FB<fn>)" by auto
	    
	    from 
	      `A \<in> dom ca` `fk \<in> dom (ca.\<^sub>RA)` `c = ca` 
	      sym[OF `ca.\<^sub>FA<fk> = E\<up>FutRef fn`] pred
	    have "TConfig amap' fmap', Env empty \<turnstile> E\<up>FutRef fn : the (fmap' fk)"
	      by simp
	    moreover
	    from domfmap `B \<in> dom ca` `fn \<in> dom (ca.\<^sub>RB)` `c = ca`
	    have "fn \<in> dom (fmap')" by auto
	    hence "TConfig amap' fmap', Env empty \<turnstile> FutRef fn : the (fmap' fn)"
	      by auto
	    moreover
	    from `B \<in> dom ca` `fn \<in> dom (ca.\<^sub>RB)` `c = ca` pred
	    have "TConfig amap' fmap', Env empty \<turnstile> c.\<^sub>FB<fn> : the (fmap' fn)"
	      by auto
	    ultimately
	    show "TConfig amap' fmap', Env empty \<turnstile> c'.\<^sub>FG<fi> : the (fmap' fi)"
	      using `c'.\<^sub>FG<fi> = E\<up>(c.\<^sub>FB<fn>)` sym[OF `fk = fi`]
	      by (simp,
		intro subst_term_typing[of "TConfig amap' fmap'" E 
		                           "FutRef fn" "the (fmap' fk)" 
		                           "the (fmap' fn)" "c.\<^sub>FB<fn>"], 
		simp)
	  next
	    case False with `G \<in> dom c'` `fi \<in> dom (c'.\<^sub>RG)` requ 
	    have "c'.\<^sub>FG<fi> = (c.\<^sub>FG<fi>)" by auto
	    with 
	      `G \<in> dom c'` `fi \<in> dom (c'.\<^sub>RG)` `dom c = dom c'` 
	      `\<forall>A\<in>dom c'. dom (c'.\<^sub>RA) = dom (c.\<^sub>RA)` pred
	    show "TConfig amap' fmap', Env empty \<turnstile> c'.\<^sub>FG<fi> : the (fmap' fi)" by auto
	  qed
	next
	  case False with `G \<in> dom c'` `fi \<in> dom (c'.\<^sub>RG)` requ 
	  have "c'.\<^sub>FG<fi> = (c.\<^sub>FG<fi>)" by auto
	  with 
	    `G \<in> dom c'` `fi \<in> dom (c'.\<^sub>RG)` `dom c = dom c'` 
	    `\<forall>A\<in>dom c'. dom (c'.\<^sub>RA) = dom (c.\<^sub>RA)` pred
	  show "TConfig amap' fmap', Env empty \<turnstile> c'.\<^sub>FG<fi> : the (fmap' fi)" by auto
	qed
      qed
    next
      from `TConfig amap' fmap' = TConfig amap fmap`
      show "amap \<subseteq>\<^sub>m amap'" by auto      
    next
      from `TConfig amap' fmap' = TConfig amap fmap`
      show "fmap \<subseteq>\<^sub>m fmap'" by auto
    qed      
  next
    case (update_AO G ca A B fk E l u)
    note def_c' = this(2)
    with `A \<in> dom ca` `c = ca` have "dom c \<union> {G} = dom c'" by auto
    with `G \<notin> dom ca` `c = ca` have "dom c \<subseteq> dom c'" by auto
    from def_c' have "dom (c'.\<^sub>RG) = {}" by (auto simp: RequestQueue_def)
    from `c = ca` def_c' `fk \<in> dom (ca.\<^sub>RA)` `A \<in> dom ca` `G \<notin> dom ca`
    have 
      domrequ: "\<forall>B\<in>dom c. dom (c.\<^sub>RB) = dom (c'.\<^sub>RB)" and
      actobj: "\<forall>B\<in>dom c. (c.\<^sub>AB) = (c'.\<^sub>AB)" and
      onerequ: "\<forall>B\<in>dom c. \<forall>fi\<in>dom (c.\<^sub>RB).
                         (A = B \<and> fk = fi \<longrightarrow> (c'.\<^sub>FB<fi>) = E\<up>ActRef G) 
                       \<and> (A \<noteq> B \<or> fk \<noteq> fi \<longrightarrow> (c'.\<^sub>FB<fi>) = (c.\<^sub>FB<fi>))"
      by (auto simp: RequestQueue_def ActiveObject_def OneRequest_def)
    from 
      `A \<in> dom ca` `fk \<in> dom (ca.\<^sub>RA)` 
      sym[OF `ca.\<^sub>FA<fk> = E\<up>Upd (ActRef B) l u`] `c = ca` pred
    have typingctxt: "TConfig amap' fmap', Env empty \<turnstile> E\<up>Upd (ActRef B) l u : the (fmap' fk)"
      by simp
    from closed_subterm_type[OF this] `body u` `closed u`
    obtain T where typingupd': "TConfig amap' fmap', Env empty \<turnstile> cUpdL cHole l u\<up>ActRef B : T" 
      by (auto simp: closed_def lc_upd)
    from 
      `A \<in> dom ca` `fk \<in> dom (ca.\<^sub>RA)` `c = ca` `G \<notin> dom ca` 
      `ca.\<^sub>FA<fk> = E\<up>Upd (ActRef B) l u`
    have typingupd: "TConfig (amap'(G \<mapsto> T)) fmap', Env empty \<turnstile> cUpdL cHole l u\<up>ActRef B : T"
      by (
	intro typing_amap_bigger[OF typingupd'
	                            noActRef_WF1b[OF Typing_implies_WF1b[OF 
	                                               Ctyping.T_conf[OF prems(2-4)]]]],
	simp_all, rule_tac x = E in exI, simp)
    
    hence 
      typingaref: "TConfig (amap'(G \<mapsto> T)) fmap', Env empty \<turnstile> ActRef B : T" and
      "the ((amap'(G \<mapsto> T)) B) = T" by auto
    note typingctxt' = subst_term_typing[OF typingupd typingaref]

    show ?thesis
    proof (rule_tac x = "amap'(G \<mapsto> T)" in exI, rule_tac x = fmap' in exI,
	intro conjI Ctyping.intros)
      from `dom amap' = dom c` `dom c \<union> {G} = dom c'`
      show "dom (amap'(G \<mapsto> T)) = dom c'" by auto
    next
      from domfmap show "dom fmap' = (\<Union>A\<in>dom c'. dom (c'.\<^sub>RA))"
      proof (simp, intro typing_fmap_eq[OF `dom c \<subseteq> dom c'` domrequ])
	from `dom c \<union> {G} = dom c'` `G \<notin> dom ca` `c = ca`
	have "dom c' - dom c = {G}" by auto
	with `dom (c'.\<^sub>RG) = {}` 
	show "\<forall>A\<in>dom c' - dom c. dom (c'.\<^sub>RA) = {}" by auto
      qed
    next
      show 
	"\<forall>A\<in>dom c'. 
	   TConfig (amap'(G \<mapsto> T)) fmap', Env empty \<turnstile> c'.\<^sub>AA : the ((amap'(G \<mapsto> T)) A) \<and>
	   (\<forall>fi\<in>dom(c'.\<^sub>RA). TConfig (amap'(G \<mapsto> T)) fmap', Env empty 
	                    \<turnstile> c'.\<^sub>FA<fi> : the (fmap' fi))"
      proof (intro strip conjI)
	fix F assume "F \<in> dom c'"
	show "TConfig (amap'(G \<mapsto> T)) fmap', Env empty \<turnstile> c'.\<^sub>AF : the ((amap'(G \<mapsto> T)) F)"
	proof (cases "F=G")
	  case True with def_c' `A \<in> dom ca` `G \<notin> dom ca` 
	  have "c'.\<^sub>AF = Upd (ca.\<^sub>AB) l u" by (auto simp: ActiveObject_def)
	  moreover
	  from `F = G` have "the ((amap'(G \<mapsto> T)) F) = T" by simp
	  moreover
	  from `B \<in> dom ca` `G \<notin> dom ca` have "B \<noteq> G" by auto
	  with `the ((amap'(G \<mapsto> T)) B) = T` have "T = the (amap' B)" by simp
	  with `B \<in> dom ca` `c = ca` pred `G \<notin> dom ca`
	  have "TConfig (amap'(G \<mapsto> T)) fmap', Env empty \<turnstile> ca.\<^sub>AB : T" 
	    by (simp, 
	      intro typing_amap_bigger[OF _
	                                  noActRef_WF1a[OF Typing_implies_WF1a[OF 
	                                                     Ctyping.T_conf[OF prems(2-4)]]]],
	      simp_all, rule_tac x = cHole in exI, simp)
	  ultimately show ?thesis by (simp, intro typingctxt'[simplified], simp)
	next
	  case False with `dom c \<union> {G} = dom c'` `F \<in> dom c'`
	  have "F \<in> dom c" by auto
	  with actobj have "(c'.\<^sub>AF) = (c.\<^sub>AF)" by simp
	  moreover
	  from `F \<noteq> G` have "the ((amap'(G \<mapsto> T)) F) = the (amap' F)" by simp
	  ultimately show ?thesis
	    using `F \<in> dom c` pred `G \<notin> dom ca` `c = ca`
	    by (simp,
	      intro typing_amap_bigger[OF _
	                                  noActRef_WF1a[OF Typing_implies_WF1a[OF 
	                                                     Ctyping.T_conf[OF prems(2-4)]]]],
	      simp_all, rule_tac x = cHole in exI, simp)
	qed
      next
	fix F fi assume "F \<in> dom c'" and "fi \<in> dom (c'.\<^sub>RF)"
	with `dom (c'.\<^sub>RG) = {}` have "F \<noteq> G" by auto
	with `dom c \<union> {G} = dom c'` `F \<in> dom c'` have "F \<in> dom c" by auto
	with domrequ `fi \<in> dom (c'.\<^sub>RF)` have "fi \<in> dom (c.\<^sub>RF)" by simp
	show "TConfig (amap'(G \<mapsto> T)) fmap', Env empty \<turnstile> c'.\<^sub>FF<fi> : the (fmap' fi)"
	proof (cases "A=F")
	  case True show ?thesis proof (cases "fk=fi")
	    from 
	      `A \<in> dom ca` `fk \<in> dom (ca.\<^sub>RA)` `ca.\<^sub>FA<fk> = E\<up>Upd (ActRef B) l u`
	      `c = ca` `G \<notin> dom ca`
	    have 
	      typingcaAfk: "TConfig (amap'(G \<mapsto> T)) fmap', Env empty 
	      \<turnstile> E\<up>Upd (ActRef B) l u : the (fmap' fk)"
	      by (intro
		typing_amap_bigger[OF typingctxt
		                      noActRef_WF1b[OF Typing_implies_WF1b[OF 
	                                                 Ctyping.T_conf[OF prems(2-4)]]]],
		simp_all, rule_tac x = cHole in exI, simp)

	    case True with `A = F` onerequ `F \<in> dom c` `fi \<in> dom (c.\<^sub>RF)`
	    have "(c'.\<^sub>FF<fi>) = E\<up>ActRef G" by blast
	    with sym[OF `fk = fi`] `body u` typingaref
	    show ?thesis
	      by (simp, 
		intro subst_term_typing[OF typingcaAfk typingctxt'[simplified]],
		auto)
	  next
	    case False with onerequ `F \<in> dom c` `fi \<in> dom (c.\<^sub>RF)`
	    have "(c'.\<^sub>FF<fi>) = (c.\<^sub>FF<fi>)" by blast
	    with `F \<in> dom c` `fi \<in> dom (c.\<^sub>RF)` `G \<notin> dom ca` `c = ca` pred 
	    show ?thesis
	      by (simp,
		intro typing_amap_bigger[OF _ 
		                            noActRef_WF1b[OF Typing_implies_WF1b[OF 
	                                                       Ctyping.T_conf[OF prems(2-4)]]]],
		simp_all, rule_tac x = cHole in exI, simp)
	  qed
	next
	  case False with onerequ `F \<in> dom c` `fi \<in> dom (c.\<^sub>RF)`
	  have "(c'.\<^sub>FF<fi>) = (c.\<^sub>FF<fi>)" by blast
	  with `F \<in> dom c` `fi \<in> dom (c.\<^sub>RF)` `G \<notin> dom ca` `c = ca` pred 
	  show ?thesis
	    by (simp,
	      intro typing_amap_bigger[OF _ 
		                          noActRef_WF1b[OF Typing_implies_WF1b[OF 
	                                                     Ctyping.T_conf[OF prems(2-4)]]]],
	      simp_all, rule_tac x = cHole in exI, simp)
	qed
      qed
    next
      from `G \<notin> dom ca` `c = ca` `dom amap' = dom c`
      have "G \<notin> dom amap'" by auto
      from `TConfig amap' fmap' = TConfig amap fmap` map_le_upd[OF this]
      show "amap \<subseteq>\<^sub>m amap'(G \<mapsto> T)" by auto
    next
      from `TConfig amap' fmap' = TConfig amap fmap`
      show "fmap \<subseteq>\<^sub>m fmap'" by auto      
    qed
  qed
qed

theorem Csubject_reduction: "\<lbrakk> \<turnstile> c : TConfig amap fmap; c \<rightarrow>\<^sub>\<zeta> c' \<rbrakk> 
  \<Longrightarrow> \<exists>amap' fmap'. \<turnstile> c' : TConfig amap' fmap' \<and> amap \<subseteq>\<^sub>m amap' \<and> fmap \<subseteq>\<^sub>m fmap'"
  by (auto simp: Csubject_reduction_pre)

subsubsection{*Progress*}
inductive isremote :: "sterm \<Rightarrow> bool"
where
  remote_Call[simp, intro!]:   "isremote ((E::econtext)\<up>Call (ActRef B) l p)"
| remote_FutRef[simp, intro!]: "isremote ((E::econtext)\<up>FutRef fi)"
| remote_Active[simp, intro!]: "isremote ((E::econtext)\<up>Active a)"
| remote_Upd[simp, intro!]:    "isremote ((E::econtext)\<up>Upd (ActRef B) l a)"

lemma remote_call: "isremote (Call (ActRef B) l p)"
  using remote_Call[of eHole B l p] by simp

lemma remote_futref: "isremote (FutRef fk)"
  using remote_FutRef[of eHole fk] by simp

lemma remote_active: "isremote (Active a)"
  using remote_Active[of eHole a] by simp

lemma remote_upd: "isremote (Upd (ActRef B) l u)"
  using remote_Upd[of eHole B l u] by simp

lemma ectxt_call_lc: 
  assumes "lc_ctxt (ectxt2gctxt (eCall E l t))"
  shows "lc t"
proof -
  fix x 
  from 
    lc_ctxt_and_lc[OF assms lc_Fvar[of x]]
    lc_call[of "(ectxt2gctxt E\<up>Fvar x)" l t]
  show "lc t" by auto
qed

lemma ectxt_upd_lc: 
  assumes "lc_ctxt (ectxt2gctxt (eUpdL E l t))"
  shows "body t"
proof -
  fix x 
  from 
    lc_ctxt_and_lc[OF assms lc_Fvar[of x]]
    lc_upd[of "(ectxt2gctxt E\<up>Fvar x)" l t]
  show "body t" by auto
qed

lemma lc_active: "lc (Active t) = lc t"
  by (rule iffI, erule lc.cases, simp_all)

lemma ectxt_and_fill_lc: "lc ((E::econtext)\<up>t) \<Longrightarrow> lc_ctxt (ectxt2gctxt E) \<and> lc t"  
  by (induct E, auto simp: lc_call lc_upd lc_active body_lc_FV)

lemma ectxt_and_fill_lc': "lc ((E::econtext)\<up>t) \<Longrightarrow> lc_ctxt (ectxt2gctxt E)"  
  by (induct E, auto simp: lc_call lc_upd lc_active body_lc_FV)

lemma ectxt_and_fill_lc'': "lc ((E::econtext)\<up>t) \<Longrightarrow> lc t"
  by (induct E, auto simp: lc_call lc_upd lc_active)

(*
lemma copen_commute[rule_format]:
  "\<forall>n k s p s' p'. n \<noteq> k
     \<longrightarrow> {n \<rightarrow>\<^sub>c [Fvar s', Fvar p']} {k \<rightarrow>\<^sub>c [Fvar s, Fvar p]} E 
         = {k \<rightarrow>\<^sub>c [Fvar s, Fvar p]} {n \<rightarrow>\<^sub>c [Fvar s', Fvar p']} E"
proof -
  {
    fix t u
    have
      "(\<forall>n k s p s' p'. n \<noteq> k
         \<longrightarrow> {n \<rightarrow> [Fvar s', Fvar p']} {k \<rightarrow> [Fvar s, Fvar p]} t 
             = {k \<rightarrow> [Fvar s, Fvar p]} {n \<rightarrow> [Fvar s', Fvar p']} t)
      &(\<forall>n k s p s' p'. n \<noteq> k
         \<longrightarrow> sopen_option n (Fvar s') (Fvar p') (sopen_option k (Fvar s) (Fvar p) u) 
             = sopen_option k (Fvar s) (Fvar p)
               (sopen_option n (Fvar s') (Fvar p') u))"
      by (rule sterm.induct, simp_all split: bVariable.split)
  } note lem = this
  show ?thesis by (induct rule: general_context.induct, auto simp: lem)
qed
*)

lemma remote_context: 
  fixes E::econtext
  assumes "isremote t"
  shows "isremote (E\<up>t)"
proof -
  from eCompose_and_Fill have "\<forall>(E::econtext) (F::econtext) x. E\<up>F\<up>x = (E\<circ>F)\<up>x" 
    by auto
  with assms show ?thesis by (cases rule: isremote.cases, auto)
qed

lemma remote_context_inv: "\<not> isremote ((E::econtext)\<up>t) \<Longrightarrow> \<not> isremote t"
  by (auto simp: remote_context)

lemma sigma_progress[rule_format]:
  "\<forall>C T. C, Env empty \<turnstile> t : T \<longrightarrow> \<not> isvalue t \<longrightarrow> \<not> isremote t
    \<longrightarrow> (\<exists>t'. t \<rightarrow>\<^sub>\<beta> t')"
proof -
  have
  "(\<forall>C T. C, Env empty \<turnstile> t : T \<longrightarrow> \<not> isvalue t \<longrightarrow> \<not> isremote t
     \<longrightarrow> (\<exists>t'. t \<rightarrow>\<^sub>\<beta> t')) 
  \<and>(\<forall>C T. C, Env empty \<turnstile> Obj f T : T \<longrightarrow> \<not> isvalue (Obj f T) \<longrightarrow> \<not> isremote (Obj f T)
     \<longrightarrow> (\<exists>t'. Obj f T \<rightarrow>\<^sub>\<beta> t'))"
  proof (induct rule: sterm_induct)
    case (Bvar b) with no_bvar[of _ "Env empty" b] show ?case 
      by auto (* contradiction *)
  next
    case (Fvar x) with Fvar_beta[of x] show ?case
      by auto (* contradiction *)
  next
    case Obj show ?case by auto (* contradiction *)
  next
    case empty thus ?case by auto (* contradiction *)
  next
    case insert show ?case by auto (* contradiction *)
  next
    case Active with remote_active show ?case by auto (* contradiction *)
  next
    case FutRef with remote_futref show ?case by auto (* contradiction *)
  next
    case ActRef show ?case by auto (* contradiction *)
  next
    case (Call t l u) note pred_t = this(1) and pred_u = this(2)
    show ?case proof (auto)
      fix C T assume 
	"C, Env empty \<turnstile> t : T" and "C, Env empty \<turnstile> u : param(the(T^l))" and 
	"l \<in> do T" and "\<not> isremote (Call t l u)"
      from this(4) remote_context_inv[of "eCall eHole l u" t]
      have "\<not> isremote t" by auto
      with `C, Env empty \<turnstile> t : T` pred_t
      have "isvalue t \<or> (\<exists>t'. t \<rightarrow>\<^sub>\<beta> t')" by auto
      thus "\<exists>t'. Call t l u \<rightarrow>\<^sub>\<beta> t'"
      proof (elim disjE exE)
	fix c B assume "isvalue t"
	then obtain f T' where "t = Obj f T'" 
	  using `\<not> isremote (Call t l u)` remote_call
	  by (cases rule: isvalue.cases, auto)
	with 
	  `C, Env empty \<turnstile> t : T` `l \<in> do T` obj_inv[of C "Env empty" f T' T] 
	  obj_inv_elim[of C "Env empty" f T]
	have "l \<in> dom f" by auto
	with 
	  `t = Obj f T'` beta.beta[of l f T' u]
	  typing_regular''[OF `C, Env empty \<turnstile> t : T`] 
	  typing_regular''[OF `C, Env empty \<turnstile> u : param(the(T^l))`]
	show ?thesis by auto
      next
	fix t' assume "t \<rightarrow>\<^sub>\<beta> t'"
	from beta.beta_CallL[OF this typing_regular''[OF `C, Env empty \<turnstile> u : param(the(T^l))`]] 
	show ?thesis by auto
      qed
    qed
  next
    case (Upd t l u) note pred_t = this(1) and pred_u = this(2) 
    show ?case proof (intro strip)
      fix C T assume "C, Env empty \<turnstile> Upd t l u : T"
      with typing_regular''[OF this] lc_upd[of t l u]
      have 
	"lc t" and "body u" and 
	"C, Env empty \<turnstile> t : T" and "l \<in> do T" by auto

      assume "\<not> isremote (Upd t l u)"
      with remote_context_inv[of "eUpdL eHole l u" t]
      have "\<not> isremote t" by auto
      with `C, Env empty \<turnstile> t : T` pred_t
      have "isvalue t \<or> (\<exists>t'. t \<rightarrow>\<^sub>\<beta> t')" by auto
      thus "\<exists>t'. Upd t l u \<rightarrow>\<^sub>\<beta> t'"
      proof (elim disjE exE)
	fix c B assume "isvalue t"
	then obtain f T' where "t = Obj f T'" 
	  using `\<not> isremote (Upd t l u)` remote_upd
	  by (cases rule: isvalue.cases, auto)
	with 
	  `C, Env empty \<turnstile> t : T` `l \<in> do T` obj_inv[of C "Env empty" f T' T] 
	  obj_inv_elim[of C "Env empty" f T]
	have "l \<in> dom f" by auto
	with 
	  `t = Obj f T'` `body u` typing_regular''[OF `C, Env empty \<turnstile> t : T`] 
	  beta.beta_Upd[of l f T' u]
	show ?thesis by auto
      next
	fix t' assume "t \<rightarrow>\<^sub>\<beta> t'"
	from beta.beta_UpdL[OF this `body u`] show ?thesis by auto
      qed
    qed
  qed
  from conjunct1[OF this] show ?thesis by assumption
qed

lemma econtext_typing_closed[rule_format]: 
  "\<forall>A t T. C, Env empty \<turnstile> (E::econtext)\<up>t : T \<longrightarrow> closed t"
  by (induct E, auto simp: typing_closed)

lemma remote_progress:
  assumes "\<turnstile> c : C" and "nocycle c" and "finite_conf c" and
  "A \<in> dom c" and "fk \<in> dom (c.\<^sub>RA)" and "isremote (c.\<^sub>FA<fk>)"
  shows "\<exists>c'. c \<rightarrow>\<^sub>\<zeta> c' \<and> (c'.\<^sub>FA<fk>) \<noteq> (c.\<^sub>FA<fk>)"
  using assms
proof (induct rule: Ctyping.induct)
  case (T_conf amap c fmap)
  note domfmap = this(2) and pred = this(3)
  from domfmap `finite_conf c` 
  have "finite (dom c)" and "finite (\<Union>A\<in>dom c. dom (c.\<^sub>RA))" 
    by (auto simp: finite_conf_def)
  from 
    ex_new_if_finite[OF FMap.inftype_class.infinite this(1)] 
    ex_new_if_finite[OF FMap.inftype_class.infinite this(2)] 
  obtain G fn where freshG: "G \<notin> dom c" and freshfn: "\<forall>A\<in>dom c. fn \<notin> dom (c.\<^sub>RA)"
    by auto
  with `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` have "A \<noteq> G" and "fk \<noteq> fn" by auto

  from `isremote (c.\<^sub>FA<fk>)` show ?case
  proof (cases rule: isremote.cases)
    case (remote_Call E B l u)
    from sym[OF this] `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` pred
    have typingctxt: "TConfig amap fmap, Env empty \<turnstile> E\<up>Call (ActRef B) l u : the (fmap fk)"
      by auto
    from 
      ectxt_and_fill_lc[OF typing_regular''[OF this]]
      typing_closed[OF this] FV_ctxt_and_Fill'[of "Call (ActRef B) l u" "ectxt2gctxt E"]
      ectxt2gctxt_and_fill[of E "Call (ActRef B) l u"]
    have "lc_ctxt (ectxt2gctxt E)" and "lc u" and "closed u"
      by (auto simp: lc_call closed_def)
    from 
      `(c.\<^sub>FA<fk>) = E\<up>Call (ActRef B) l u`
      ectxt2gctxt_and_fill[of E "Call (ActRef B) l u"]
    have "(c.\<^sub>FA<fk>) = ((ectxt2gctxt E)\<circ>(cCallL cHole l u))\<up>ActRef B" by auto
    with
      Typing_implies_WF1b[OF Ctyping.T_conf[OF prems(7-9)]] 
      `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` 
    have "B \<in> dom c" unfolding wf_1b_def by blast
    have "\<forall>E::general_context. (c.\<^sub>FA<fk>) \<noteq> E\<up>FutRef fn"
      by (
	intro noFutRef_WF2b[OF Typing_implies_WF2b[OF Ctyping.T_conf[OF prems(7-9)]]
	                       `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` _ freshfn],
	rule_tac x = cHole in exI, simp)

    show ?thesis proof (cases "A=B")
      def c' \<equiv> "c(A \<mapsto> ((c.\<^sub>RA)(fk \<mapsto> ectxt2gctxt E \<up> FutRef fn, 
                               fn \<mapsto> Call (c.\<^sub>AA) l u), c.\<^sub>AA))"
      case True
      with
	`(c.\<^sub>FA<fk>) = E\<up>Call (ActRef B) l u`
	ectxt2gctxt_and_fill[of E "Call (ActRef B) l u"]
	zeta.request_self[OF freshfn `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` _ 
	                     `lc_ctxt (ectxt2gctxt E)` `lc u` `closed u`]
      have "c \<rightarrow>\<^sub>\<zeta> c'" by (auto simp: c'_def)
      moreover
      from `fk \<noteq> fn` `\<forall>E::general_context. (c.\<^sub>FA<fk>) \<noteq> E\<up>FutRef fn`
      have "(c'.\<^sub>FA<fk>) \<noteq> (c.\<^sub>FA<fk>)" 
	by (auto simp: c'_def OneRequest_def RequestQueue_def not_sym)
      ultimately show ?thesis by auto
    next
      def c' \<equiv> "c(A \<mapsto> ((c.\<^sub>RA)(fk \<mapsto> ectxt2gctxt E \<up> FutRef fn), c.\<^sub>AA), 
	          B \<mapsto> ((c.\<^sub>RB)(fn \<mapsto> Call (c.\<^sub>AB) l u), c.\<^sub>AB))"
      case False
      with
	`(c.\<^sub>FA<fk>) = E\<up>Call (ActRef B) l u`
	ectxt2gctxt_and_fill[of E "Call (ActRef B) l u"]
	zeta.request[OF freshfn `A \<in> dom c` `B \<in> dom c` `A \<noteq> B` 
	                `fk \<in> dom (c.\<^sub>RA)` _ `lc_ctxt (ectxt2gctxt E)` 
	                `lc u` `closed u`]
      have "c \<rightarrow>\<^sub>\<zeta> c'" by (auto simp: c'_def)
      moreover
      from `A \<noteq> B` `\<forall>E::general_context. (c.\<^sub>FA<fk>) \<noteq> E\<up>FutRef fn`
      have "(c'.\<^sub>FA<fk>) \<noteq> (c.\<^sub>FA<fk>)" 
	by (auto simp: c'_def OneRequest_def RequestQueue_def not_sym)
      ultimately show ?thesis by auto
    qed
  next
    case (remote_FutRef E fi)
    with ectxt2gctxt_and_fill[of E "FutRef fi"]
    have "(c.\<^sub>FA<fk>) = ectxt2gctxt E\<up>FutRef fi" by auto
    with 
      `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` 
      Typing_implies_WF2b[OF Ctyping.T_conf[OF prems(7-9)]] 
    obtain B where "B \<in> dom c" and "fi \<in> dom (c.\<^sub>RB)"
      unfolding wf_2b_def by blast
    def c' \<equiv> "c(A \<mapsto> ((c.\<^sub>RA)(fk \<mapsto> ectxt2gctxt E\<up>(c.\<^sub>FB<fi>)), c.\<^sub>AA))"
    from 
      zeta.reply[OF `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` 
                    `(c.\<^sub>FA<fk>) = ectxt2gctxt E\<up>FutRef fi` 
                    `B \<in> dom c` `fi \<in> dom (c.\<^sub>RB)`]
    have "c \<rightarrow>\<^sub>\<zeta> c'" by (auto simp: c'_def)
    moreover
    {
      assume "(c'.\<^sub>FA<fk>) = (c.\<^sub>FA<fk>)"
      with `(c.\<^sub>FA<fk>) = ectxt2gctxt E\<up>FutRef fi`
      have "FutRef fi = (c.\<^sub>FB<fi>)"
	by (intro eq_context[of "ectxt2gctxt E" "FutRef fi"],
	  auto simp: c'_def OneRequest_def)
      hence "\<exists>E::general_context. (c.\<^sub>FB<fi>) = E\<up>FutRef fi"
	by (rule_tac x = cHole in exI, simp)
      with `B \<in> dom c` `fi \<in> dom (c.\<^sub>RB)`
      have "\<exists>B\<in>dom c. \<exists>E::general_context. fi \<in> dom (c.\<^sub>RB) \<and> (c.\<^sub>FB<fi>) = E\<up>FutRef fi"
	by auto
      hence "(RefFut fi, RefFut fi) \<in> (knows c)\<^sup>+"
	unfolding knows_def by blast
      with `nocycle c` have False by (auto simp: nocycle_def)
    } hence "(c'.\<^sub>FA<fk>) \<noteq> (c.\<^sub>FA<fk>)" by auto
    ultimately show ?thesis by auto
  next
    case (remote_Active E a)
    def c' \<equiv> "c(A \<mapsto> ((c.\<^sub>RA)(fk \<mapsto> ectxt2gctxt E\<up>ActRef G), c.\<^sub>AA), G \<mapsto> (Map.empty, a))"
    from sym[OF `(c.\<^sub>FA<fk>) = E\<up>Active a`] `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` pred
    have typingctxt: "TConfig amap fmap, Env empty \<turnstile> E\<up>Active a : the (fmap fk)"
      by auto
    from 
      ectxt_and_fill_lc[OF typing_regular''[OF this]]
      typing_closed[OF this] FV_ctxt_and_Fill'[of "Active a" "ectxt2gctxt E"]
      ectxt2gctxt_and_fill[of E "Active a"]
      `(c.\<^sub>FA<fk>) = E\<up>Active a`
    have 
      "(c.\<^sub>FA<fk>) = ectxt2gctxt E\<up>Active a" and
      "lc_ctxt (ectxt2gctxt E)" and "lc a" and "closed a"
      by (auto simp: lc_active closed_def)
    from zeta.active[OF `G \<notin> dom c` `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` this]
    have "c \<rightarrow>\<^sub>\<zeta> c'" by (auto simp: c'_def)
    moreover
    {
      have "\<forall>E::general_context. (c.\<^sub>FA<fk>) \<noteq> E\<up>ActRef G"
	by (
	  intro noActRef_WF1b[OF Typing_implies_WF1b[OF Ctyping.T_conf[OF prems(7-9)]]
	                         `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` _ freshG],
	  rule_tac x = cHole in exI, simp)
      moreover
      from `A \<noteq> G` have "\<exists>E::general_context. (c'.\<^sub>FA<fk>) = E\<up>ActRef G"
	by (rule_tac x = "ectxt2gctxt E" in exI, 
	  auto simp: c'_def OneRequest_def)
      ultimately have "(c.\<^sub>FA<fk>) \<noteq> (c'.\<^sub>FA<fk>)" by auto
    } ultimately show ?thesis by auto
  next
    case (remote_Upd E B l u)
    with ectxt2gctxt_and_fill[of E "Upd (ActRef B) l u"]
    have "(c.\<^sub>FA<fk>) = ((ectxt2gctxt E)\<circ>(cUpdL cHole l u))\<up>ActRef B" by auto
    with 
      `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` 
      Typing_implies_WF1b[OF Ctyping.T_conf[OF prems(7-9)]] 
    have "B \<in> dom c" unfolding wf_1b_def by blast
    with `G \<notin> dom c` have "B \<noteq> G" by auto

    def c' \<equiv> "c(A \<mapsto> ((c.\<^sub>RA)(fk \<mapsto> ectxt2gctxt E\<up>ActRef G), c.\<^sub>AA), 
                G \<mapsto> (Map.empty, Upd (c.\<^sub>AB) l u))"

    from sym[OF `(c.\<^sub>FA<fk>) = E\<up>Upd (ActRef B) l u`] `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` pred
    have typingctxt: "TConfig amap fmap, Env empty \<turnstile> E\<up>Upd (ActRef B) l u : the (fmap fk)"
      by auto
    from 
      ectxt_and_fill_lc[OF typing_regular''[OF this]]
      typing_closed[OF this] FV_ctxt_and_Fill'[of "Upd (ActRef B) l u" "ectxt2gctxt E"]
      ectxt2gctxt_and_fill[of E "Upd (ActRef B) l u"]
    have "lc_ctxt (ectxt2gctxt E)" and "body u" and "closed u"
      by (auto simp: lc_upd closed_def)
    from 
      zeta.update_AO[OF `G \<notin> dom c` `A \<in> dom c` `B \<in> dom c` 
                        `fk \<in> dom (c.\<^sub>RA)` _ this]
      ectxt2gctxt_and_fill[of E "Upd (ActRef B) l u"]
      `(c.\<^sub>FA<fk>) = E\<up>Upd (ActRef B) l u`
    have "c \<rightarrow>\<^sub>\<zeta> c'" by (auto simp: c'_def)
    moreover
    have "\<forall>E::general_context. (c.\<^sub>FA<fk>) \<noteq> E\<up>ActRef G"
      by (intro noActRef_WF1b[OF Typing_implies_WF1b[OF Ctyping.T_conf[OF prems(7-9)]]
                                 `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` _ `G \<notin> dom c`],
	rule_tac x = cHole in exI, simp)
    with `A \<noteq> G` `B \<noteq> G` have "(c.\<^sub>FA<fk>) \<noteq> (c'.\<^sub>FA<fk>)"
      by (auto simp: c'_def OneRequest_def)
    ultimately show ?thesis by auto
  qed
qed

text {* Final Type Soundness Lemma *}
theorem progress: 
  assumes 
  "\<turnstile> c : C" and "nocycle c" and "finite_conf c" and 
  "A \<in> dom c" and "fk \<in> dom (c.\<^sub>RA)"
  shows 
  "isvalue (c.\<^sub>FA<fk>) \<or> 
  (\<exists>c'. (c \<rightarrow>\<^sub>\<zeta> c') \<and> ((c'.\<^sub>FA<fk>) \<noteq> (c.\<^sub>FA<fk>) \<or> (c.\<^sub>FA<fk>) \<rightarrow>\<^sub>\<beta> (c'.\<^sub>FA<fk>)))"
proof (cases "isvalue (c.\<^sub>FA<fk>)")
  case True thus ?thesis by simp
next case False show ?thesis
  proof (cases "isremote (c.\<^sub>FA<fk>)")
    case True from remote_progress[OF assms this] show ?thesis by auto
  next
    case False
    from `\<turnstile> c : C` `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)`
    obtain amap fmap where "TConfig amap fmap, Env empty \<turnstile> (c.\<^sub>FA<fk>) : the (fmap fk)"
      by (elim Ctyping.cases, blast)
    from sigma_progress[OF this `\<not> isvalue (c.\<^sub>FA<fk>)` `\<not> isremote (c.\<^sub>FA<fk>)`]
    obtain t where "(c.\<^sub>FA<fk>) \<rightarrow>\<^sub>\<beta> t" by auto
    with 
      zeta.zeta_local[OF `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)` this] 
      `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)`
    show ?thesis by (auto simp: OneRequest_def RequestQueue_def)
  qed
qed

lemma sterm_induct1:
  fixes t :: sterm and P1 :: "sterm \<Rightarrow> bool" and 
        f :: "Label -~> sterm" and P3 :: "(Label -~> sterm) \<Rightarrow> bool"
  assumes
  "\<And>b. P1 (Bvar b)"
  "\<And>x. P1 (Fvar x)"
  "\<And>f. (! x: dom f. P1 (the (f x))) \<Longrightarrow> P3 f"
  "\<And>f T. P3 f \<Longrightarrow> P1 (Obj f T)"
  "\<And>t1 l t2. \<lbrakk>P1 t1; P1 t2\<rbrakk> \<Longrightarrow> P1 (Call t1 l t2)"
  "\<And>t1 l t2. \<lbrakk>P1 t1; P1 t2\<rbrakk> \<Longrightarrow> P1 (Upd t1 l t2)"
  "\<And>t. P1 t \<Longrightarrow> P1 (Active t)"
  "\<And>a. P1 (ActRef a)"
  "\<And>f. P1 (FutRef f)"
  shows "P1 t \<and> P3 f"
proof -
  have "\<forall>t (f::Label -~> sterm) l.  P1 t \<and> (applyPropOnOption P1) (f l)"
  proof (intro strip)
    fix t :: sterm and f :: "Label -~> sterm" and l :: Label 
    def foobar \<equiv> "f l"
    from assms show "P1 t \<and> applyPropOnOption P1 foobar"
    proof (induct_tac t and foobar, auto)
      fix f' :: "Label \<rightharpoonup> sterm" and T :: "type"
      assume p1: "\<And>x. applyPropOnOption P1 (f' x)" and
	     p3: "\<And>f. \<forall>x\<in>dom f. P1 (the (f x)) \<Longrightarrow> P3 f"
      hence "\<forall>l\<in>dom f'. P1 (the (f' l))"
      proof (auto)
	fix l y from p1 have "applyPropOnOption P1 (f' l)" by auto
	moreover assume "f' l = Some y"
	ultimately have "applyPropOnOption P1 (Some y)" by simp
	thus "P1 y" by simp
      qed
      with p3 have "P3 f'" by simp
      with assms show "P1 (Obj f' T)" by simp
    qed
  qed
  with assms(3) show ?thesis by auto
qed

lemma not_noRef_lem[rule_format]:
  shows
  "\<not> noRef a \<Longrightarrow> 
  (\<exists>G E::general_context. a = E\<up>ActRef G) \<or> (\<exists>fk E::general_context. a = E\<up>FutRef fk)"
proof (induct a
  taking: 
    "\<lambda>b. \<not> noRef_option b 
    \<longrightarrow> ((\<exists>G E::general_context. the b = E \<up> ActRef G) 
        \<or>(\<exists>fk E::general_context. the b = E \<up> FutRef fk))")
  case Bvar thus ?case by auto
next
  case Fvar thus ?case by auto
next
  case (ActRef G) show ?case 
    by (intro disjI1, 
      rule_tac x = G in exI, rule_tac x = cHole in exI,
      auto)
next
  case (FutRef fk) show ?case 
    by (intro disjI2, 
      rule_tac x = fk in exI, rule_tac x = cHole in exI,
      auto)
next
  case (Active a) note pred_a = this(1)
  from `\<not> noRef (Active a)` have "\<not> noRef a" by auto
  with pred_a have 
    "(\<exists>G E::general_context. a = E \<up> ActRef G) 
    \<or>(\<exists>fk E::general_context. a = E \<up> FutRef fk)" by simp
  thus ?case proof (elim disjE exE)
    fix G and E::general_context
    assume "a = E\<up>ActRef G" thus ?thesis 
      by (intro disjI1, 
	rule_tac x = G in exI, rule_tac x = "cActive E" in exI,
	auto)
  next
    fix fk and E::general_context
    assume "a = E\<up>FutRef fk" thus ?thesis 
      by (intro disjI2, 
	rule_tac x = fk in exI, rule_tac x = "cActive E" in exI,
	auto)
  qed
next
  case (Call t l u) 
  note pred_t = this(1) and pred_u = this(2)
  show ?case proof (cases "noRef t")
    case True with `\<not> noRef (Call t l u)` have "\<not> noRef u" by auto
    with pred_u have 
      "(\<exists>G E::general_context. u = E \<up> ActRef G) 
      \<or>(\<exists>fk E::general_context. u = E \<up> FutRef fk)" by simp
    thus ?thesis proof (elim disjE exE)
      fix G and E::general_context
      assume "u = E\<up>ActRef G" thus ?thesis 
	by (intro disjI1, 
	  rule_tac x = G in exI, rule_tac x = "cCallR t l E" in exI,
	  auto)
    next
      fix fk and E::general_context
      assume "u = E\<up>FutRef fk" thus ?thesis 
	by (intro disjI2, 
	  rule_tac x = fk in exI, rule_tac x = "cCallR t l E" in exI,
	  auto)
    qed
  next
    case False with pred_t have 
      "(\<exists>G E::general_context. t = E \<up> ActRef G) 
      \<or>(\<exists>fk E::general_context. t = E \<up> FutRef fk)" by simp
    thus ?thesis proof (elim disjE exE)
      fix G and E::general_context
      assume "t = E\<up>ActRef G" thus ?thesis 
	by (intro disjI1, 
	  rule_tac x = G in exI, rule_tac x = "cCallL E l u" in exI,
	  auto)
    next
      fix fk and E::general_context
      assume "t = E\<up>FutRef fk" thus ?thesis 
	by (intro disjI2, 
	  rule_tac x = fk in exI, rule_tac x = "cCallL E l u" in exI,
	  auto)
    qed
  qed
next
  case (Upd t l u) 
  note pred_t = this(1) and pred_u = this(2)
  show ?case proof (cases "noRef t")
    case True with `\<not> noRef (Upd t l u)` have "\<not> noRef u" by auto
    with pred_u have 
      "(\<exists>G E::general_context. u = E \<up> ActRef G) 
      \<or>(\<exists>fk E::general_context. u = E \<up> FutRef fk)" by simp
    thus ?thesis proof (elim disjE exE)
      fix G and E::general_context
      assume "u = E\<up>ActRef G" thus ?thesis 
	by (intro disjI1, 
	  rule_tac x = G in exI, rule_tac x = "cUpdR t l E" in exI,
	  auto)
    next
      fix fk and E::general_context
      assume "u = E\<up>FutRef fk" thus ?thesis 
	by (intro disjI2, 
	  rule_tac x = fk in exI, rule_tac x = "cUpdR t l E" in exI,
	  auto)
    qed
  next
    case False with pred_t have 
      "(\<exists>G E::general_context. t = E \<up> ActRef G) 
      \<or>(\<exists>fk E::general_context. t = E \<up> FutRef fk)" by simp
    thus ?thesis proof (elim disjE exE)
      fix G and E::general_context
      assume "t = E\<up>ActRef G" thus ?thesis 
	by (intro disjI1, 
	  rule_tac x = G in exI, rule_tac x = "cUpdL E l u" in exI,
	  auto)
    next
      fix fk and E::general_context
      assume "t = E\<up>FutRef fk" thus ?thesis 
	by (intro disjI2, 
	  rule_tac x = fk in exI, rule_tac x = "cUpdL E l u" in exI,
	  auto)
    qed
  qed
next
  case (Some_sterm t) note pred_t = this
  show ?case proof (intro strip)
    assume "\<not> noRef_option (Some t)" with pred_t
    show 
      "(\<exists>G E::general_context. the(Some t) = E \<up> ActRef G) 
      \<or>(\<exists>fk E::general_context. the(Some t) = E \<up> FutRef fk)" 
      by simp
  qed
next
  case None_sterm thus ?case by auto
next
  case (Obj f T) note pred_bnd = this(1)
  from `\<not> noRef (Obj f T)` 
  obtain l where "l \<in> dom f" and "\<not> noRef_option (f l)"
    by auto
  with pred_bnd have 
    "(\<exists>G E::general_context. the (f l) = E \<up> ActRef G)
    \<or>(\<exists>fk E::general_context. the (f l) = E \<up> FutRef fk)"
    by simp
  thus ?case proof (elim disjE exE)
    fix G and E::general_context
    assume "the(f l) = E\<up>ActRef G"
    with `l \<in> dom f` show ?thesis
      by (intro disjI1,
	rule_tac x = G in exI, rule_tac x = "cObj (Abs_FmapLabel(f, l)) T E" in exI,
	auto simp: map_upd_triv)
  next
    fix fk and E::general_context
    assume "the(f l) = E\<up>FutRef fk"
    with `l \<in> dom f` show ?thesis
      by (intro disjI2,
	rule_tac x = fk in exI, rule_tac x = "cObj (Abs_FmapLabel(f, l)) T E" in exI,
	auto simp: map_upd_triv)
  qed
qed

lemma noRef_and_openFV[rule_format]: 
  "noRef t \<Longrightarrow> \<forall>n s p. noRef ({n \<rightarrow> [Fvar s, Fvar p]} t)"
  by (induct t
    taking: "\<lambda>b. noRef_option b \<longrightarrow> (\<forall>n s p. noRef_option (sopen_option n (Fvar s) (Fvar p) b))",
    auto split: bVariable.split)

lemma noRef_option_lem: 
  "\<forall>l\<in>dom f. noRef (the(f l)) = noRef_option (f l)"
  by auto

lemma noRef_typing:
  assumes "TConfig empty empty, e \<turnstile> t : T"
  shows "noRef t \<Longrightarrow> \<forall>C. C, e \<turnstile> t : T"
  using assms
proof (induct rule: typing.induct)
  case T_Var thus ?case by auto
next
  case T_Call thus ?case by auto
next
  case T_Active thus ?case by auto
next
  case T_ActRef thus ?case by auto
next
  case T_FutRef thus ?case by auto
next
  case T_Upd thus ?case by (auto simp: noRef_and_openFV openz_def)
next
  case (T_Obj e f T L C) note pred = this(4)
  from `dom f = do T` `noRef (Obj f T)`
  have nR: "\<forall>l\<in>do T. noRef (the(f l))" 
    by (auto simp: noRef_option_lem[of f])
  from prems show ?case 
  proof auto
    fix Ca l s p assume "l \<in> do T"
    with nR have "noRef (the(f l))" by simp
    from noRef_and_openFV[OF this]
    have "noRef (the(f l)\<^bsup>[Fvar s, Fvar p]\<^esup>)" by (auto simp: openz_def)
    moreover
    assume "s \<notin> L" and "p \<notin> L" and "s \<noteq> p"
    ultimately
    show "Ca, e\<lparr>s:T\<rparr>\<lparr>p:param(the(T^l))\<rparr> \<turnstile> the(f l)\<^bsup>[Fvar s, Fvar p]\<^esup> : return (the(T^l))"
      using `l \<in> do T` pred
      by blast
  qed
qed

lemma typing_empty_noRef:
  assumes "TConfig empty empty, Env empty \<turnstile> a : T"
  shows "noRef a"
proof -
  {
    assume "\<not> noRef a"
    from not_noRef_lem[OF this] have False
    proof (elim disjE exE)
      fix G and E::general_context assume "a = E\<up>ActRef G"
      with assms subterm_typing[of "TConfig empty empty" E "ActRef G" T]
      obtain e T' where "TConfig empty empty, e \<turnstile> ActRef G : T'"
	by auto
      thus False by auto
    next
      fix fk and E::general_context assume "a = E\<up>FutRef fk"
      with assms subterm_typing[of "TConfig empty empty" E "FutRef fk" T]
      obtain e T' where "TConfig empty empty, e \<turnstile> FutRef fk : T'"
	by auto
      thus False by auto
    qed
  } thus "noRef a" by auto
qed

lemma init_config_typing:
  assumes "TConfig empty empty, Env empty \<turnstile> a : T"
  shows "\<turnstile> init_config a : TConfig [aref 0 \<mapsto> Object empty] [fref 0 \<mapsto> T]"
  using assms
  unfolding init_config_def
proof (intro Ctyping.intros, 
    auto simp: RequestQueue_def ActiveObject_def OneRequest_def)
  from noRef_typing[OF assms typing_empty_noRef[OF assms(1)]]
  show "TConfig [aref 0 \<mapsto> Object empty] [fref 0 \<mapsto> T], Env empty \<turnstile> a : T"
    by auto
qed

lemma Csubject_reduction':
  assumes "c \<rightarrow>\<^sub>\<zeta> c'" and "\<turnstile> c : CT"
  shows "\<exists>CT'. \<turnstile> c' : CT'"
  using 
    Csubject_reduction_pre[OF `\<turnstile> c : CT` _ `c \<rightarrow>\<^sub>\<zeta> c'`]
    Ctype.exhaust[of CT "\<exists>CT'. \<turnstile> c' : CT'"]
  by auto

lemma Csubject_reduction'':
  assumes "c \<rightarrow>\<^sub>\<zeta>\<^sup>* c'"
  shows "\<turnstile> c : CT \<Longrightarrow> \<exists>CT'. \<turnstile> c' : CT'"
  using assms
proof (induct rule: rtranclp_induct)
  case base thus ?case by auto
next
  case (step a b)
  from this(3-4) obtain A where "\<turnstile> a : A" by auto
  from Csubject_reduction'[OF `a \<rightarrow>\<^sub>\<zeta> b` this]
  show ?case by simp
qed

theorem init_config_progress:
  assumes 
  "init_config a \<rightarrow>\<^sub>\<zeta>\<^sup>* c" and "TConfig empty empty, Env empty \<turnstile> a : T" and
  "A \<in> dom c" and "fk \<in> dom (c.\<^sub>RA)"
  shows
  "isvalue (c.\<^sub>FA<fk>) \<or> 
  (\<exists>c'. (c \<rightarrow>\<^sub>\<zeta> c') \<and> ((c'.\<^sub>FA<fk>) \<noteq> (c.\<^sub>FA<fk>) \<or> (c.\<^sub>FA<fk>) \<rightarrow>\<^sub>\<beta> (c'.\<^sub>FA<fk>)))"
proof -
  from assms(1) typing_empty_noRef[OF assms(2)]
  have "c \<in> reachable_configs" by (auto simp: reachable_configs_def)
  from Csubject_reduction''[OF assms(1) init_config_typing[OF assms(2)]]
  obtain CT where "\<turnstile> c : CT" by auto
  show ?thesis
    by (intro 
      progress[OF `\<turnstile> c : CT` reachable_nocycle[OF `c \<in> reachable_configs`]
                  finite_conf_rtrancl[OF assms(1) finite_init_config] 
                  `A \<in> dom c` `fk \<in> dom (c.\<^sub>RA)`])
qed

theorem reachable_progress: 
  assumes 
  "c \<in> reachable_configs" and "\<turnstile> c : C" and "A \<in> dom c" and "fk \<in> dom (c.\<^sub>RA)"
  shows 
  "isvalue (c.\<^sub>FA<fk>) \<or> 
  (\<exists>c'. (c \<rightarrow>\<^sub>\<zeta> c') \<and> ((c'.\<^sub>FA<fk>) \<noteq> (c.\<^sub>FA<fk>) \<or> (c.\<^sub>FA<fk>) \<rightarrow>\<^sub>\<beta> (c'.\<^sub>FA<fk>)))"
proof (intro progress[OF assms(2) reachable_nocycle[OF assms(1)] _ assms(3-4)])
  from `c \<in> reachable_configs` obtain a where "init_config a \<rightarrow>\<^sub>\<zeta>\<^sup>* c"
    by (auto simp: reachable_configs_def)
  from finite_conf_rtrancl[OF this finite_init_config] 
  show "finite_conf c" by assumption
qed

end
